c= imhist - plot a histogram of the data
c& bpw
c: map analysis
c+
c  Imhist makes histograms of image data. The output can be written to
c  the terminal, a log file, or a plot.
c  A gaussian curve with the same mean, rms and integral is drawn in
c  the histogram (this can be turned off using the 'nogauss' option).
c  The plotheader can be suppressed by using options=noheader. An
c  alternative title can be put on the plot by options=title. A useful
c  combination is 'options=noh,t,title', to get only the string 'title',
c  instead of the full header.
c
c< in
c< region
c  For the moment imhist only recognizes rectangular boxes.
c
c@ options
c  These options allow to control the plot. They may be abbreviated to
c  uniqueness. Several options may be combined, in random order.
c  Possible options are (# means: give a number):
c
c   'nbin,#'      Select the number of bins of the histogram, default 25
c   'binsize,#'   Select a fixed size for the bins
c   'cumulative'  Make the histogram cumulative
c   'logarithmic' Make the y-scale of the histogram logarithmic
c   'nogauss'     Do not plot the smooth gaussian curve
c   'cutoutliers,#' Loop through dataset twice, first to determine rms.
c                 On second loop, set cut to average plus/minus this
c                 value times the rms
c
c   'noheader'    Do not write the header information, just the numbers,
c                 producing an ASCII file for a plotting program
c   'nolist'      Do not write the statistics to the screen/logfile
c
c   'xmin,#'      Left edge of histogram plot, default determined from
c                 selected data
c   'xmax,#'      Right edge of histogram plot, default determined from
c                 selected data
c   'ymax,#'      The plot will be cut off at this y-value, default
c                 is 1.25 times maximum histogram value
c   'title,#1,#2,#3' Put the string #1 at x-position #2 and y-position #3,
c                 with positions measured in units of the coordinates
c                 on the axes. If 'title' is the last option, the title
c                 is put in the upper left hand corner.
c   'style,#'     This selects the plot style.
c                 #=connect means connect the datapoints
c                 #=step means make one-bin wide connected horizontal
c                 line segments
c                 #=histo means bins are drawn as a horizontal line
c                 surrounded by two vertical lines
c
c@ cutoff
c  All datavalues below the cutoff are not used for the calculation of
c  the histogram.
c  If two values, all datavalues above second value are also not used.
c  If second or third values is the string ',abs', a cutoff in the
c  absolute value of the datavalues is used.
c  Default is no cutoff.
c
c< device
c@ log
c  If specified, output is written to the file given by log= instead
c  of to the terminal.
c--
c
c
c   History:
c
c    20jul91  bpw  Original version
c    29jul91  bpw  Corrected bug for 2-d datasets
c    06aug91  bpw  Corrected channel scale, added flux calculation,
c                  included imspect, created imhist.h
c    19sep91  bpw  Minor overhaul to make splitting easier and to
c                  finish off some leftover things
c    24sep91  bpw  Split off from imstahis.for
c    21jan92  bpw  Add writing position of min and max
c    27mar92  bpw  Changed assert into assertl
c    10sep92  bpw  Noted that coordinates for min and max were switched
c    15jan93  bpw  More decimals in writing results
c    13mar93  mjs  pgplot subr names have less than 7 chars.
c    21may93  mjs  Call pgask(.FALSE.) to override any pgplot default.
c    03jun93   jm  Removed pgask(.FALSE.) call and also added printout
c                  of valid devices if input device name is incorrect.
c    15jun93  bpw  Did paging with pgcurs to get back preferred situation before
c                  pgplot changes, where clicking left resulted in paging
c    30sep93   jm  Corrected pgplot calling sequence.
c    12nov97  rjs  Added 's' flag to boxset call.
c     4jun98  bpw  Added upper cutoff
c    12jun98  bpw  Add 'cutoutliers' option
c    11jul07   tw  Fixed a segfault with gfortran
c    13jul07  pjt  revert back, previous fix uses g95 notations
c     4jun12  pjt  ptrdiff update for large cubes
c
c------------------------------------------------------------------------
      program imhist
c
      character*40     version
      parameter        ( version = 'version 4-jun-2012' )
c
      integer          tinp
      real             cut(3)
      integer          naxis
      INTEGER*8 npixels
      character*80     device
c
      call output( 'IMHIST: ' // version )
      call inputs( tinp, cut, npixels, naxis, device )
      call histo(  tinp, cut, npixels, naxis, device )
      call xyzclose( tinp )
      call logclose
c
      end
c***********************************************************************
      subroutine inputs( tinp, cut, npixels, naxis, device )
c
      integer            tinp
      real               cut(*)
      INTEGER*8 npixels
      integer            naxis
      character*(*)      device
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      integer            MAXBOXES
      parameter          ( MAXBOXES = 1024 )
c
      character*1024     file
      integer            axlen( MAXNAX )
      integer            boxes( MAXBOXES )
      integer            blc(MAXNAX), trc(MAXNAX)
      integer            viraxlen( MAXNAX )
      INTEGER*8 vircsz( MAXNAX )
c
      call keyini
c
      call keyf( 'in', file, ' ' )
      naxis = MAXNAX
      call xyzopen( tinp, file, 'old', naxis, axlen )
c
      call boxinput( 'region', file, boxes, MAXBOXES )
      call boxset(   boxes, naxis, axlen, 's' )
      call boxinfo(  boxes, naxis, blc, trc )
c
      call optinp
c
      call cutinp( cut )
c
      call xyzsetup( tinp, ' ', blc, trc, viraxlen, vircsz )
      npixels = vircsz(naxis)
c
      call outdev( device )
c
      call keyfin
c
      call labset( tinp, file, blc, trc, naxis )
c
      call header( tinp, file )
c
      return
      end
c***********************************************************************
      subroutine cutinp( cut )
c
      real          cut(*)
c
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      character*40  string
      logical       keyprsnt, ok
c
      cut(1) =  MAGICVAL
      cut(2) = -MAGICVAL
      cut(3) =        0.
      if( keyprsnt( 'cutoff' ) ) then
         call keyr( 'cutoff', cut(1), cut(1) )
      endif
      if( keyprsnt( 'cutoff' ) ) then
         call keya( 'cutoff', string, 'noabs' )
         call atorf(string,cut(2),ok)
         if( .not.ok ) cut(2) = -MAGICVAL
         if(      ok ) string = 'noabs'
      endif
      if( keyprsnt( 'cutoff' ) ) then
         call keya( 'cutoff', string, 'noabs' )
      endif
      if( string.ne.'abs') cut(3) = 1.
      if( string.eq.'abs') cut(3) = 2.
c
      return
      end
c**********************************************************************
      subroutine optinp
c
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      character*20  option
      logical       match
      integer       i
      real          v
      integer       matchnr
      integer       len1
      character*80  line
c
      logical       needval( NHISTP )
      real          histdef( NHISTP )
      data          needval  / .true.,.true.,.false.,.false.,.false.,
     *                         .true. /
      data          histdef  / 25., 1., 0.,0.,0., 0. /
      do i = 1, NHISTP
         histpar(FLAG,i)  = 0.
         histpar(VALUE,i) = histdef(i)
      enddo
      plotvar( SEL)     = -1
      plotvar( HEAD )   =  1
      plotvar( LIST )   =  1
      plotvar( HANWD )  =  1
      plotvar( STYLE )  = -1
      plotpar( TITLE )  = ' '
      plotrnge( FLXL )  = 0.
      plotrnge( FLXU )  = 0.
      plotrnge( FLYL )  = 0.
      plotrnge( FLYU )  = 0.
c
      call keya( 'options', option, ' ' )
      do while( option.ne. ' ' )
         call lcase( option )
c
         if( match( option, commonop, i )  ) then
            if(     i.eq.matchnr('style',commonop) ) then
               call keya( 'options', option, ' ' )
               call assertl( match(option,styles,i), 'Illegal style' )
               plotvar(STYLE) = i
            elseif( i.eq.matchnr('noheader',commonop) ) then
               plotvar(HEAD) = 0
            elseif( i.eq.matchnr('nolist',commonop) ) then
               plotvar(LIST) = 0
            elseif( i.eq.matchnr('list',commonop) ) then
               plotvar(LIST) = 1
            elseif( i.eq.matchnr('title',commonop) ) then
               call keya( 'options', plotpar(TITLE),   ' ' )
               call keyr( 'options', plotrnge(XTITLE), MAGICVAL )
               call keyr( 'options', plotrnge(YTITLE), MAGICVAL )
            else
               call keyr( 'options', v, 0. )
               if(     i.eq.matchnr('xmin',commonop) ) then
                  plotrnge(FLXL) = 1.
                  plotrnge(XLOW) = v
               elseif( i.eq.matchnr('xmax',commonop) ) then
                  plotrnge(FLXU) = 1.
                  plotrnge(XUPP) = v
               elseif( i.eq.matchnr('ymin',commonop) ) then
                  plotrnge(FLYL) = 1.
                  plotrnge(YLOW) = v
               elseif( i.eq.matchnr('ymax',commonop) ) then
                  plotrnge(FLYU) = 1.
                  plotrnge(YUPP) = v
               endif
            endif
c
         elseif( match( option, plotopts, i )  ) then
            histpar(FLAG,i) = 1.
            if( needval(i) )
     *          call keyr( 'options', histpar(VALUE,i), histdef(i) )
            if( .not.needval(i) ) histpar(VALUE,i) = 1.
            if( i.eq.NBINP ) call assertl(
     *          histpar(VALUE,NBINP).gt.0., 'Bad number of bins' )
            if( i.eq.BINSP ) call assertl(
     *          histpar(VALUE,BINSP).gt.0., 'Bin size must be >0' )
            if( i.eq.OUTLIERS ) call assertl(
     *          histpar(VALUE,OUTLIERS).gt.0.,
     *                                'Outliers parameter must be >0' )
c
         else
            line = 'Illegal or ambiguous option ' //
     *             option(:len1(option)) // ' ignored'
            call bug( 'w', line )
c
         endif
         call keya( 'options', option, ' ' )
      enddo
c
      if(plotvar(STYLE).eq.-1) plotvar(STYLE)=matchnr('histo',styles)
c
      return
      end
c***********************************************************************
      subroutine outdev( device )
c
      character*(*)  device
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      character*1024 logfile
c
      call keya( 'device', device,  ' ' )
      if( device.eq.' ' .and. plotvar(LIST).eq.0 ) plotvar(LIST) = 1
      call keya( 'log',    logfile, ' ' )
      call logopen( logfile, ' ' )
c
      return
      end
c**********************************************************************
      subroutine labset( tinp,file,blc,trc,naxis )
c
      integer       tinp
      character*(*) file
      integer       blc(*), trc(*), naxis
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      integer       len1
      character*40  string
      character*20  units
      real          rdata
      character*10  rtoaf
      integer       i, j
      character*40  bln, tln
c
      integer       NRECOG
      parameter     ( NRECOG = 4 )
      character*10  runits( NRECOG ), labels( NRECOG )
      data          runits  / 'JY', 'HZ', 'KM/S', 'KMS-1' /
      data          labels / 'Flux','Frequency','Velocity','Velocity' /
c
      call rdhda( tinp, 'bunit', units, 'Unknown' )
      string = units
      call lcase( string )
c
c
      i = 1
      do while( i.le.NRECOG .and.
     *   index( string, runits(i)(:len1(runits(i))) ) .eq. 0  )
         i = i + 1
      enddo
      if( i.le.NRECOG ) string = labels(i)
      if( i.gt.NRECOG ) string = 'Map value'
      write( plotpar(XLABP), '( a, '' ['', a, '']'' )' )
     *       string(:len1(string)), units(:len1(units))
c
      plotpar(YLABP) = ' '
      if( histpar(FLAG,LOGAR).eq.1. ) plotpar(YLABP) = 'Log'
      if( histpar(FLAG,CUMUL).eq.1. )
     *plotpar(YLABP)(len1(plotpar(YLABP))+2:) = 'Cumulative'
      plotpar(YLABP)(len1(plotpar(YLABP))+2:) = 'Counts'
c
      call rdhda( tinp, 'object',   string, ' ' )
      call rdhdr( tinp, 'restfreq', rdata,   0. )
      if( string.ne.' ' ) plotpar(INFOP) = 'Source: ' // string
      if( rdata.ne.0. ) then
          plotpar(INFOP)(len1(plotpar(INFOP))+1:)='; '//rtoaf(rdata,1,6)
          plotpar(INFOP)(len1(plotpar(INFOP))+1:)=' GHz'
      endif
      plotpar(INFOP)(len1(plotpar(INFOP))+1:) = '; File: ' // file
c
      call mitoaf( blc, naxis, bln, i )
      call mitoaf( trc, naxis, tln, j )
      string = 'blc=(' // bln(:i) // '), trc=(' // tln(:j) // ')'
      plotpar(BOXP) = 'Bounding box: ' // string
c
      return
      end
c***********************************************************************
      subroutine header( tinp, file )
c
      integer          tinp
      character*(*)    file
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      integer          len1
      character*80     line
      character*20     units
c
      if( plotvar(HEAD).eq.0 ) return
c
      line = '***** Histogram of image '//file(:len1(file))//' *****'
      call logwrit( line )
c
      line = '      ' // plotpar(BOXP)
      call logwrit( line )
c
      call rdhda( tinp, 'bunit', units, 'Unknown' )
      line = '      Unit of datavalues: ' // units
      call logwrit( line )
c
      return
      end
c***********************************************************************
c***********************************************************************
c***********************************************************************
      subroutine histo( tinp, cut, npixels, naxis, device )
c
      integer       tinp
      real          cut(*)
      integer       naxis
      INTEGER*8 npixels
      character*(*) device
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      logical       doplot
      integer       pgbeg
c
      integer       binmax
c
      integer       HLEN
      parameter     ( HLEN = 1000 )
      real          xvals(0:HLEN+1), hist(0:HLEN+1)
      data          hist / 0., HLEN * 0., 0. /
c
      doplot = device .ne. ' '
      if( doplot ) then
         if( pgbeg( 0,device,1,1 ) .ne. 1 ) then
            call pgldev
            call bug( 'f', 'Error opening plot device' )
         endif
      endif
c
      call histset( tinp, cut, npixels, HLEN, binmax )
c
      call histmake( tinp, binmax, cut, npixels, xvals, hist )
c
      call histout( doplot, naxis, binmax, xvals, hist )
c
      return
      end
c***********************************************************************
      subroutine histset( tinp, cut, npixels, HLEN, binmax )
c
      integer          tinp
      real             cut(*)
      INTEGER*8 npixels
      integer          HLEN, binmax
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      INTEGER*8 npoints
      double precision sum, sumsq, calcrms
      real             minval, maxval
      logical          ok
      integer          delimi
      real             r, gausetup
c
      call histvars( tinp,cut,npixels, npoints,sum,sumsq,minval,maxval )
c
      if( histpar(FLAG,BINSP).eq.0. )
     *histpar(VALUE,BINSP) =
     *    1.001*( plotrnge(XUPP)-plotrnge(XLOW) ) / histpar(VALUE,NBINP)
c
      histvar(NPTS)  = npoints
      histvar(MEANP) = sum / npoints
      histvar(RMSP)  = calcrms( sum, sumsq, npoints, ok )
c
      binmax = int(  ( plotrnge(XUPP)-plotrnge(XLOW) )
     *                 / histpar(VALUE,BINSP)            ) + 1
      call assertl( binmax.le.HLEN, 'Too many bins' )
      binmax = delimi( binmax, 1, HLEN )
c
      r=gausetup( minval, maxval, histpar(VALUE,BINSP),
     *  histvar(NPTS),histvar(MEANP),histvar(RMSP),histpar(FLAG,CUMUL) )
c
      return
      end
c***********************************************************************
      subroutine histvars( tinp, cut, npixels,
     *                     npoints, sum, sumsq, minval, maxval )
c
      integer          tinp
      real             cut(*)
      INTEGER*8 npixels,npoints
      double precision sum, sumsq
      real             maxval, minval
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      integer          nloop, j
      INTEGER*8 i
      real             data
      logical          mask
      logical          unmasked, init, ok
      double precision calcrms
c
      if( cut(3).gt.0. .and. plotrnge(FLXU).eq.1. )
     *    call assertl( cut(1).lt.plotrnge(XUPP),
     *                  'Cutoff is above histogram maximum!' )
c
      if( histpar(FLAG,OUTLIERS).eq.0. ) nloop = 1
      if( histpar(FLAG,OUTLIERS).eq.1. ) nloop = 2
      ok = .true.
      do j = 1, nloop
      if( ok ) then
         init = .true.
         do i = 1, npixels
            call xyzpixrd( tinp, i, data, mask )
            if( unmasked( data, mask, cut ) ) then
               if( init ) then
                  npoints = 0
                  minval  = data
                  maxval  = data
                  call xyzs2c( tinp, i, posmin )
                  call xyzs2c( tinp, i, posmax )
                  sum     = 0.d0
                  sumsq   = 0.d0
                  init    = .false.
               endif
               npoints = npoints + 1
               if( data.lt.minval ) then
                  minval = data
                  call xyzs2c( tinp, i, posmin )
               endif
               if( data.gt.maxval ) then
                  maxval = data
                  call xyzs2c( tinp, i, posmax )
               endif
               sum     = sum   + data
               sumsq   = sumsq + data*data
            endif
         enddo
         if( nloop.eq.2 .and. j.eq.1 ) then
            histvar(RMSP)  = calcrms( sum, sumsq, npoints, ok )
            if( .not.ok ) then
               call bug( 'w', 'Second loop not done' )
            else
               cut(3) = histpar(VALUE,OUTLIERS) * histvar(RMSP)
               cut(1) = sum/npoints - cut(3)
               cut(2) = sum/npoints + cut(3)
               cut(3) = 1.
            endif
         endif
      endif
      enddo
c
      call assertl( .not.init,   'All datapoints are masked' )
c     call assertl( npoints.gt.1,'Histogramming 1 datapoint will fail' )
c
      histvar(MINV) = minval
      histvar(MAXV) = maxval
      if( plotrnge(FLXL).eq.0. ) plotrnge(XLOW) = minval
      if( plotrnge(FLXU).eq.0. ) plotrnge(XUPP) = maxval
      if( cut(3).ne.0. ) call assertl( cut(1).lt.plotrnge(XUPP),
     *                   'Cutoff is above maximum in region!' )
c
      return
      end
c**********************************************************************
      subroutine histmake( tinp, binmax, cut, npixels, xvals, hist )
c
      integer          tinp
      integer          binmax
      real             cut(*)
      INTEGER*8 npixels
      real             xvals(0:*), hist(0:*)
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      integer          bin
      INTEGER*8 i
      real             data
      logical          mask, unmasked
      real             rbin
      integer          delimi
c
      do bin = 0, binmax+1
         xvals(bin) = (bin-1)*histpar(VALUE,BINSP)+plotrnge(XLOW)
      enddo
      xvals(0)        = xvals(0)        + histpar(VALUE,BINSP) / 2.
      xvals(binmax+1) = xvals(binmax+1) + histpar(VALUE,BINSP) / 2.
c
      do i = 1, npixels
         call xyzpixrd( tinp, i, data, mask )
         if( unmasked( data, mask, cut ) ) then
            rbin = (data-plotrnge(XLOW)) / histpar(VALUE,BINSP)
            if( rbin.ge.0. ) bin = int(rbin)+1
            if( rbin.lt.0. ) bin = int(rbin)
            bin = delimi( bin, 0, binmax+1 )
            hist( bin ) = hist( bin ) + 1.
         endif
      enddo
c
      call histmedi( hist, binmax )
c
      if( histpar(FLAG,CUMUL).eq.1. ) then
         do bin = 2, binmax
            hist(bin) = hist(bin) + hist(bin-1)
         enddo
      endif
c
      if( histpar(FLAG,LOGAR).eq.1. ) then
         do bin = 0, binmax+1
            if( hist(bin).ne.0. ) hist(bin) = alog10( hist(bin) )
         enddo
      endif
c
      return
      end
c**********************************************************************
      subroutine histmedi( hist, binmax )
c
      real          hist(0:*)
      integer       binmax
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      real          sum
      integer       bin
c
      sum = 0.
      bin = 0
      do while( sum.lt.histvar(NPTS)/2. .and. bin.le.binmax )
         sum = sum + hist(bin)
         bin = bin + 1
      enddo
      histvar(MEDIANP) = (bin-1)*histpar(VALUE,BINSP) + plotrnge(XLOW)
c
      return
      end
c**********************************************************************
      subroutine histout( doplot, naxis, n, xarr, yarr )
c
      logical       doplot
      integer       naxis
      real          xarr(0:*), yarr(0:*)
      integer       n
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      real          xmin, xmax, ymin, ymax
      integer       ismax
      real          EXTEND
      parameter     ( EXTEND = 0.05 )
c
      integer       bin, i
      INTEGER*8 ny
      character*80  line
      integer       astlen
      parameter     ( astlen = 50 )
      character*(astlen) asterisk
      do i = 1, astlen
         asterisk(i:i) = '*'
      enddo
c
      call histinfo( naxis, xarr(1), xarr(n) )
c
      xmin = xarr(0)   - EXTEND*( xarr(n+1) - xarr(0) )
      xmax = xarr(n+1) + EXTEND*( xarr(n+1) - xarr(0) )
      ymin = -0.1
      ymax = 1.25 * yarr( ismax(n,yarr(1),1) )
      if( plotrnge(FLYU).eq.1. ) ymax = plotrnge(YUPP)
c
      if( doplot ) then
c
         call pgenv( xmin, xmax, ymin, ymax, 0, 0 )
         call pgpts( xarr(1), yarr(1), n, plotvar(STYLE), ymin )
         call undovr( xarr(0),   yarr(0),   ymax )
         call undovr( xarr(n+1), yarr(n+1), ymax )
         if( histpar(FLAG,GAUCRV).eq.0. ) call gaucurve
         call pgident
         call pgend
         call output(' ')
c
      else
c
         call logwrit( ' ' )
         call logwrit(  '  Bin    Value          Number' )
         write( line, '( ''       Underflow   '', i8 )' ) int(yarr(0))
         call logwrit( line )
         do bin = 1, n
            i = (  yarr(bin) / ymax  ) * astlen
            ny = yarr(bin)
            if( i.ge.1 ) then
               write( line, '( i5, 4x, 1pg10.3, i8, 1x, a )' )
     *                bin, xarr(bin), ny, asterisk(:i)
            else
               write( line, '( i5, 4x, 1pg10.3, i8 )' )
     *                bin, xarr(bin), ny
            endif
            call logwrit( line )
         enddo
         ny= yarr(n+1)
         write( line, '(''       Overflow    '', i11 )' ) ny
         call logwrit( line )
c
      endif
c
      return
      end
c***********************************************************************
      subroutine histinfo( naxis, xmin, xmax )
c
      integer          naxis
      real             xmin, xmax
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      integer          i, len1
      INTEGER*8 n
      character*80     line
c
      call logwrit( ' ' )
      call logwrit( 'Histogram information' )
c
      line='# of points         Mean            rms            Median'
      i = len1( line ) + 2
                                       line(i:) = 'between'
      if( histvar(MEDIANP) .le. xmin ) line(i:) = 'below'
      if( histvar(MEDIANP) .ge. xmax ) line(i:) = 'above'
      call logwrit( line )
      n = histvar(NPTS)
c
      write( line, '( i11, 3x, 1pe14.7,1x, 1pe14.7 )' )
     *       n, histvar(MEANP), histvar(RMSP)
c
      i = len1( line ) + 5
      if(     histvar(MEDIANP) .le. xmin   .or.
     *        histvar(MEDIANP) .ge. xmax )       then
         write( line(i:), '( 5x, 1pe14.7 )' ) histvar(MEDIANP)
         write( line(i:), '( 5x, 1pe14.7 )' ) histvar(MEDIANP)
      else
         write( line(i:), '( 1pe14.7, '' and '', 1pe14.7 )' )
     *       histvar(MEDIANP) - histpar(VALUE,BINSP), histvar(MEDIANP)
      endif
      call logwrit( line )
c
      write( line, '( ''Maximum is '',1pe14.7,'' at ('' )' )
     *                histvar(MAXV)
      call mitoaf( posmax, naxis, line(len1(line)+1:), i )
      line(len1(line)+1:) = ') (absolute coordinates)'
      call logwrit( line )
c
      write( line, '( ''Minimum is '',1pe14.7,'' at ('' )' )
     *                histvar(MINV)
      call mitoaf( posmin, naxis, line(len1(line)+1:), i )
      line(len1(line)+1:) = ') (absolute coordinates)'
      call logwrit( line )
c
      return
      end
c***********************************************************************
      subroutine undovr( x, y, ymax )
      real       x, y, ymax
      if( y.ne.0. ) then
         if( y.lt.ymax ) call pgpt( 1, x,      y,       8 )
         if( y.ge.ymax ) call pgpt( 1, x, 0.95*ymax, 2262 )
      endif
      return
      end
c***********************************************************************
      subroutine gaucurve
c
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
c
      real          xmin, xmax
      external      gauss, loggauss, gaussint, loggint
      real          gauss, loggauss, gaussint, loggint
c
      xmin = plotrnge(XLOW) + histpar(VALUE,BINSP)/2.
      xmax = plotrnge(XUPP) - histpar(VALUE,BINSP)/2.
c
      if(     histpar(FLAG,CUMUL).eq.0. ) then
         if(     histpar(FLAG,LOGAR).eq.0. ) then
            call pgfunx( gauss,    1000, xmin, xmax, 1 )
         elseif( histpar(FLAG,LOGAR).eq.1. ) then
            call pgfunx( loggauss, 1000, xmin, xmax, 1 )
         endif
      elseif( histpar(FLAG,CUMUL).eq.1. ) then
         if(     histpar(FLAG,LOGAR).eq.0. ) then
            call pgfunx( gaussint, 1000, xmin, xmax, 1 )
         elseif( histpar(FLAG,LOGAR).eq.1. ) then
            call pgfunx( loggint,  1000, xmin, xmax, 1 )
         endif
      endif
c
      return
      end
      real function gauss( x )
      real       loggauss, gaussint, loggint, gausetup
c
      real       x
c
      real       cum
      real       minval, maxval, binsize
      real       npoints, mean, rms
c
      real       z, t, erf
c >>> ../../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../../src/inc/mirconst.h
      real       gausum
      integer    bin, binmin, binmax
      real       norm, x0, sigma
      save       norm, x0, sigma
c
      gauss = norm * exp( -(x-x0)**2/(2.*sigma**2) )
      return
c
      entry loggauss( x )
      loggauss = alog10( norm * exp( -(x-x0)**2/(2.*sigma**2) ) )
      return
c
      entry gaussint( x )
      z = abs( (x-x0)/sqrt(2.)/sigma )
      t = 1. / ( 1.+0.5*z )
      erf = 1. - t*exp( -z*z -
     *          1.26551223 + t*( 1.00002368 + t*( 0.37409196 +
     *      t*( 0.09678418 + t*(-0.18628806 + t*( 0.27886807 +
     *      t*(-1.13520398 + t*( 1.48851587 + t*(-0.82215223 +
     *      t*( 0.17087277
     *      ))))))))))
      gaussint = norm * (  0.5 + 0.5*erf*sign(1.,(x-x0))  )
      return
c
      entry loggint( x )
      z = abs( (x-x0)/sqrt(2.)/sigma )
      t = 1. / ( 1.+0.5*z )
      erf = 1. - t*exp( -z*z -
     *          1.26551223 + t*( 1.00002368 + t*( 0.37409196 +
     *      t*( 0.09678418 + t*(-0.18628806 + t*( 0.27886807 +
     *      t*(-1.13520398 + t*( 1.48851587 + t*(-0.82215223 +
     *      t*( 0.17087277
     *      ))))))))))
      loggint = alog10(  norm * (  0.5 + 0.5*erf*sign(1.,(x-x0))  ) )
      return
c
c
c
      entry gausetup( minval,maxval,binsize, npoints,mean,rms, cum )
      gausum = 0.
      binmin = int( (minval-mean) / binsize )
      binmax = int( (maxval-mean) / binsize ) + 1
      do bin = binmin, binmax
         gausum = gausum + exp( -(bin*binsize)**2 / (2.*rms**2) )
      enddo
      if( cum.eq.0. ) norm = npoints / gausum
      if( cum.eq.1. ) norm = npoints
      x0    = mean
      sigma = rms
c
      return
      end
c***********************************************************************
c***********************************************************************
c***********************************************************************
      logical function unmasked( data, mask, cut )
      real       data
      logical    mask
      real       cut(*)
      real       x
      if( cut(3).gt.0. ) then
         if( cut(3).eq.1 ) x =     data
         if( cut(3).eq.2 ) x = abs(data)
         unmasked = mask .and. x.ge.cut(1) .and. x.le.cut(2)
      else
         unmasked = mask
      endif
      return
      end
c**********************************************************************
      double precision function calcrms( sum, sumsq, npoints, ok )
c
      double precision sum, sumsq
      integer          npoints
      logical          ok
      double precision rms
c
      if(     npoints.ge.2 ) then
         rms = ( sumsq - sum**2/dfloat(npoints) ) / dfloat(npoints-1)
         if( rms.ge.0.d0 ) then
            rms = sqrt(rms)
            ok = .true.
         else
            call bug( 'w', 'Rms^2 is negative!! Square root not taken' )
            ok = .false.
         endif
      elseif( npoints.eq.1 ) then
         rms = 0.d0
         ok = .true.
      elseif( npoints.eq.0 ) then
         ok = .false.
      endif
      calcrms = rms
      return
      end
c***********************************************************************
      subroutine pgpts( xarr, yarr, n, style, ymin )
      real       xarr(*), yarr(*)
      integer    n, style
      real       ymin
      if( style.eq.1 ) call pgline( n, xarr, yarr )
      if( style.eq.2 ) call pgbin(  n, xarr, yarr, .false. )
      if( style.eq.3 ) call pgcbin( n, xarr, yarr, .false., ymin )
      return
      end
      subroutine pgcbin( n, xarr, yarr, center, ymin )
c
      integer      n
      real         xarr(*), yarr(*)
      logical      center
      real         ymin
c
      real         xpts(4), ypts(4)
      real         dx, xoff, x
      integer      i
      do i = 1, n
         if( i.lt.n ) dx = xarr(i+1) - xarr(i)
         if( i.eq.n ) dx = xarr(n) - xarr(n-1)
         if(      center ) xoff = dx / 2.
         if( .not.center ) xoff = 0.
         x       = xarr(i) - xoff
         xpts(1) = x
         ypts(1) = ymin
         xpts(2) = x
         ypts(2) = yarr(i)
         xpts(3) = x + dx
         ypts(3) = yarr(i)
         xpts(4) = x + dx
         ypts(4) = ymin
         call pgline( 4, xpts, ypts )
      enddo
      return
      end
      subroutine pgident
c >>> /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*80 plotopts
      parameter    ( plotopts =
     *'nbin,binsize,cumulative,logarithmic,nogauss,cutoutliers' )
c
      character*60 commonop
      parameter    ( commonop =
     *'noheader,nolist,list,style,title,xmin,xmax,ymin,ymax' )
c
      character*20 styles
      parameter    ( styles   = 'connect,step,histo' )
c
      integer      NPLOTV, NPLOTR, NPLOTP
      parameter    ( NPLOTV = 6, NPLOTR = 10, NPLOTP = 5 )
c
      integer      SEL, HEAD, LIST, HANWD, STYLE, DUNIT
      parameter    ( SEL=1, HEAD=2, LIST=3, HANWD=4, STYLE=5, DUNIT=6 )
c
      integer      XLOW, XUPP, YLOW, YUPP, FLXL, FLXU, FLYL, FLYU
      parameter    ( XLOW=1,  XUPP=2,  YLOW=3,  YUPP=4 )
      parameter    ( FLXL=5,  FLXU=6,  FLYL=7,  FLYU=8 )
      integer      XTITLE, YTITLE
      parameter    ( XTITLE=9, YTITLE=10 )
c
      integer      XLABP, YLABP, INFOP, BOXP, TITLE
      parameter    ( XLABP=1, YLABP=2, INFOP=3, BOXP=4, TITLE=5 )
c
      integer      plotvar(  NPLOTV )
      real         plotrnge( NPLOTR )
      character*80 plotpar(  NPLOTP )
c
      integer      NHISTP
      integer      FLAG, VALUE
      integer      NBINP, BINSP, CUMUL, LOGAR, GAUCRV, OUTLIERS
      parameter    ( NHISTP = 6 )
      parameter    ( FLAG=1, VALUE=2 )
      parameter    ( NBINP=1, BINSP=2, CUMUL=3, LOGAR=4, GAUCRV=5 )
      parameter    ( OUTLIERS=6 )
      real         histpar(2,NHISTP)
c
      integer      NHVAR
      parameter    ( NHVAR = 6 )
      integer      NPTS, MEANP, RMSP, MEDIANP, MAXV, MINV
      parameter    ( NPTS=1, MEANP=2, RMSP=3, MEDIANP=4, MAXV=5,MINV=6 )
      real         histvar(NHVAR)
c
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
      double precision crval(MAXNAX), cdelt(MAXNAX)
      character*9  ctype(MAXNAX)
      integer      posmin(MAXNAX), posmax(MAXNAX)
c
      common /VAR/ plotvar, plotrnge
      common /CHR/ plotpar, ctype
      common /HST/ histpar, histvar
      common /CRD/ crval, cdelt, posmin, posmax
c
      real         RIGHT, LEFT
      parameter    ( RIGHT=1.0, LEFT=0.0 )
      real         XOFF, BASE, YOFF, COFF, SC
      parameter    ( XOFF=0.01, BASE=2.3, YOFF=0.7, COFF=0.30, SC=0.7 )
      real         MAGICVAL
      parameter    ( MAGICVAL = -1.e30 )
c <<< /home/vagrant/rpmbuild/BUILD/miriad/src/prog/analysis/imhist.h
      character*40     pginfo, ident
      integer          i
      call pgsch( 1. )
      call pglab( plotpar(XLABP), plotpar(YLABP), ' ' )
      if( plotpar(TITLE).ne.' ' ) then
         call pgtext( plotrnge(XTITLE),plotrnge(YTITLE),plotpar(TITLE) )
      endif
      if( plotvar(HEAD).eq.1 ) then
         call pgsch( SC )
         call pgqinf(  'now', pginfo, i )
         ident = 'IMHIST ' // pginfo
         call pgmtxt( 'T', BASE+2+YOFF, XOFF, LEFT, ident          )
         call pgmtxt( 'T', BASE+1+YOFF, XOFF, LEFT, plotpar(INFOP) )
         call pgmtxt( 'T', BASE  +YOFF, XOFF, LEFT, plotpar(BOXP)  )
      endif
      return
      end
