      program vblank
c
c= VBLANK - mask outlier pixels on the spectral axis
c& thw
c: image-analysis
c+
c       VBLANK takes a cube that has already been masked and masks it
c       further so that unmasked pixels that are surrounded by only
c       masked pixels along axis 1 will be masked.  The default is to
c       mask if both adjacent pixels are masked; one can increase the
c       window size with keyword 'tol'.
c
c       This can be useful for reducing noise in a datacube that has
c       already been masked with a smoothed cube and where the velocity
c       field is assumed to be smooth.  In this case axis 1 would be the
c       velocity axis and spikes far from the channels of actual
c       emission are masked.
c
c       NOTE: Because flagging is only implemented for planes, the
c       spectral axis must be axis 1.  Use REORDER first to make this
c       so:
c         reorder in=file1 out=file2 mode='312'
c         vblank  in=file2
c         reorder in=file2 out=file3 mode='231'
c
c< in
c
c@ tol
c       How far (in pixels) the nearest unmasked pixel can be for the
c       given pixel to stay unmasked.  Default is 1.
c
c$Id: vblank.for,v 1.2 2011/06/01 04:58:16 pteuben Exp $
c--
c  History:
c    Refer to the RCS log, v1.1 includes prior revision information.
c-----------------------------------------------------------------------
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c >>> ../../../src/inc/maxnax.h
      integer      MAXNAX
      parameter    ( MAXNAX = 7 )
c
c <<< ../../../src/inc/maxnax.h
c
      logical   flags(MAXDIM)
      integer   axlen(MAXNAX), blc(MAXNAX), i, i0, i1, iscan, j, k, lIn,
     *          naxis, nflag, nscan, spcAxI, tol, trc(MAXNAX)
      real      buf(MAXDIM)
      character inp*80, msg*80, version*72
c
      external  versan
      character versan*72
c-----------------------------------------------------------------------
      version = versan('vblank',
     *                 '$Revision: 1.2 $',
     *                 '$Date: 2011/06/01 04:58:16 $')
c
c     Get and check the inputs.
      call keyini
c
      call keyf('in', inp, ' ')
      call assertl(inp.ne.' ', 'You must specify an input file')
c
      call keyi('tol', tol, 1)
      call keyfin
c
c     Open the input image.
      naxis = MAXNAX
      call xyopen(lIn, inp, 'old', naxis, axlen)
      do i = 1, naxis
        blc(i) = 1
        trc(i) = axlen(i)
      enddo
c
c     Find the spectral axis.
      call coInit(lIn)
      call coFindAx(lIn, 'spectral', spcAxI)
      call coFin(lIn)
c
      call assertl(spcAxI.ne.0, 'Spectral axis not found in dataset')
      call assertl(spcAxI.eq.1,
     *  'Spectral axis must be axis 1 (use REORDER)')
c
c     Apply masking.
      nflag = 0
      do k = blc(3), trc(3)
        call xysetpl(lIn,1,k)
c
        do j = blc(2), trc(2)
          call xyread(lIn,j,buf)
          call xyflgrd(lIn,j,flags)
c
          do i = blc(1), trc(1)
            if (flags(i)) then
              i0 = max(i-tol,1)
              i1 = min(i+tol,trc(1))
              nscan=0
              do iscan = i0, i1
                if (flags(iscan)) nscan = nscan + 1
              enddo
c
              if (nscan.eq.1) then
                flags(i) = .false.
                nflag = nflag + 1
              endif
            endif
          enddo
c
          call xyflgwr(lIn,j,flags)
        enddo
      enddo
c
c     Report number of extra pixels flagged.
      write(msg,'(i7,'' extra pixels flagged'')') nflag
      call output(msg)
c
c     Update history.
      call hisopen (lIn,'append')
      call hiswrite(lIn,'VBLANK: '//version)
      call hisinput(lIn,'VBLANK')
      call hiswrite(lIn,'VBLANK: '//msg)
      call hisclose(lIn)
c
      call xyclose(lIn)
c
      end
