c************************************************************************
        program atrtfit
c
c= ATRTFIX - Correct for misbehaving round trip system
c& rjs
c: uv analysis
c+
c	ATRTFIX corrects data for errors in the round trip phase measurement.
c	This is only needed when the round trip measurement malfunctions.
c@ vis
c	The input visibility file to correct. No default.
c@ out
c	The output corrected file. No default.
c@ rtphase
c	A text file containing the round trip phase measurements.
c	No default.
c@ options
c	Extra processing options.
c	  glitch  Only modify the phase to account for glitches (not
c	          the gradual variation).
c@ offset
c	Offset, in seconds, between the recorded values and the data
c--
c  History:
c    03jul04 rjs  Original version.
c    25jul04 rjs  Adjust tolerance to determine glitches.
c------------------------------------------------------------------------
        character version*(*)
        parameter(version='AtRpFix: version 1.0 25-Jul-04')
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c >>> ../../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../../src/inc/mirconst.h
c
        character vis*64,out*64,rtphase*64
        real phase,theta
        complex w
        logical glitch
        integer lVis,lOut,nchan,i,pol,npol
        double precision preamble(5)
        complex data(MAXCHAN)
        double precision freq(MAXCHAN)
        logical flags(MAXCHAN)
        real offset
c
c  Externals.
c
        real phget
c
        call output(version)
        call keyini
        call keya('vis',vis,' ')
        call keya('out',out,' ')
        call keya('rtphase',rtphase,' ')
        call getopt(glitch)
        call keyr('offset',offset,0.0)
        call keyfin
c
c  Open the phase file.
c
        call Phopen(rtphase,glitch)
c
c  Open the input and output, and do various housekeeping.
c
        call uvopen(lVis,vis,'old')
        call uvset(lVis,'preamble','uvw/time/baseline',0,0.,0.,0.)
        call varInit(lVis,'channel')
c
        call uvopen(lOut,out,'new')
        call varOnit(lVis,lOut,'channel')
        call uvset(lOut,'preamble','uvw/time/baseline',0,0.,0.,0.)
c
        call hdcopy(lVis,lOut,'history')
        call hisopen(lOut,'append')
        call hiswrite(lOut,'ATRTFIX: Miriad '//version)
        call hisinput(lOut,'ATRFFIX')
        call hisclose(lOut)
c
c  Get first record and then loop.
c
        call uvread(lVis,preamble,data,flags,MAXCHAN,nchan)
        if(nchan.eq.0)call bug('f','No data found')
        dowhile(nchan.gt.0)
          phase = Phget(preamble(4)-offset/86400.d0,preamble(5))
          call uvinfo(lVis,'sfreq',freq)
c
          do i=1,nchan
            theta = PI/180*phase*freq(i)
            w = cmplx(cos(theta),sin(theta))
            data(i) = w*data(i)
          enddo
c
c  Copy to the output.
c
          call varCopy(lVis,lOut)
          call uvrdvri(lVis,'pol',pol,0)
          call uvrdvri(lVis,'npol',npol,0)
          if(npol.gt.0)then
            call uvputvri(lOut,'npol',npol,1)
            call uvputvri(lOut,'pol',pol,1)
          endif
          call uvwrite(lOut,preamble,data,flags,nchan)
          call uvread(lVis,preamble,data,flags,MAXCHAN,nchan)
        enddo
c
c  All said and done.
c
        call uvclose(lOut)
        call uvclose(lVis)
        call Phclose()
c
        end
c************************************************************************
        subroutine PhOpen(file,glitch1)
c
        character file*(*)
        logical glitch1
c------------------------------------------------------------------------
        integer i
c
        double precision time(2)
        real phases(6,2),gphases(6,2)
        logical glitch
        common/phcomm/time,phases,gphases,glitch
c
        glitch = glitch1
        call tinOpen(file,'n')
        call phrec(time(1),phases(1,1))
        call phrec(time(2),phases(1,2))
        do i=1,6
          gphases(i,1) = 0
        enddo
        call phglitch(phases(1,1),gphases(1,1),
     *                phases(1,2),gphases(1,2))
c
        end
c***********************************************************************
        real function PhGet(t,bl)
c
        double precision t,bl
c------------------------------------------------------------------------
        integer i,i1,i2
c
        double precision time(2)
        real phases(6,2),gphases(6,2)
        logical glitch
        common/phcomm/time,phases,gphases,glitch
c
        i = 1
        if(time(1).gt.time(2))i=2
c
        dowhile(t.gt.time(3-i))
          call phrec(time(i),phases(1,i))
          call phglitch(phases(1,3-i),gphases(1,3-i),
     *                  phases(1,i),gphases(1,i))
          i = 3 - i
        enddo
c
        i = 1
c        if(abs(t-time(1)).gt.abs(t-time(2)))i = 2
        if(time(2).lt.time(1))i = 2
        call basant(bl,i1,i2)
        if(glitch)then
          phget = gphases(i1,i) - gphases(i2,i)
        else
          phget = phases(i1,i) - phases(i2,i)
        endif
        end
c************************************************************************
        subroutine phglitch(phase1,gphase1,phase2,gphase2)
c
        real phase1(6),gphase1(6),phase2(6),gphase2(6)
c
c------------------------------------------------------------------------
        real THRESH
        parameter(THRESH=0.5)
        integer i
c
        do i=1,6
          if(abs(phase1(i)-phase2(i)).gt.THRESH)then
            gphase2(i) = gphase1(i) - phase1(i) + phase2(i)
          else
            gphase2(i) = gphase1(i)
          endif
        enddo
        end
c************************************************************************
        subroutine PhRec(time,phases)
c
        double precision time
        real phases(6)
c------------------------------------------------------------------------
        integer i
c
        integer tinNext
c
        if(tinNext().le.0)call bug('f','Error getting RTP data')
        call tinGett(time,0.d0,'atime')
        do i=1,6
          call tinGetr(phases(i),0.)
        enddo
        end
c************************************************************************
        subroutine PhClose
c
c------------------------------------------------------------------------
        call tinClose
        end
c************************************************************************
        subroutine getopt(glitch)
c
        logical glitch
c------------------------------------------------------------------------
        integer NOPTS
        parameter(NOPTS=1)
        character opts(NOPTS)*8
        logical present(NOPTS)
c
        data opts/'glitch  '/
c
        call options('options',opts,present,NOPTS)
        glitch = present(1)
        end
