      PROGRAM calboot
c
c	Calboot -- bootstrap quasar fluxes from known planet fluxes.
c		   using a calibration dataset. See also forthcoming
c                  Arie's FLUXM program.
c
c--
c	1-dec-91    V1.0  PJT+SNV   original version @ Hat Creek
c       2-dec-91    new calbflux() format V6.4 - still fixing some bugs
c	4-dec-91    flux= now implemented - @ Berkeley
c	9-dec-91    finished the flux= implementation
c      27-mar-92    assertl now
c      18-apr-06    longer source names
c
c------ Inline doc (retrieved with doc to a .doc file) ----------------c
c
c= calboot - bootstrap quasar fluxes from known planet fluxes.
c& pjt
c: calibration
c+
c   CALBOOT is a MIRIAD program with which you can bootstrap quasar
c   fluxes from known planet fluxes in a calibration dataset.
c   Be aware of calibration datasets where different quasars intersperse
c   each other and only one was fitted. The other will be quoted
c   erroneous fluxes; the current software cannot handle such cases,
c   CALFIT must be run separately for each quasar, after deleting the
c   old fits using DELHD.
c   CALBOOT can also optionally patch the quasar flux, after which the
c   fit must be redone of course.
c
c@ gcal
c   Name of the input calibration set.
c@ planet
c   The name of one source (normally a planet) that is used to
c   bootstrap the quasar fluxes.
c   Default: the first planet encountered in the dataset.
c@ source
c   Source names to have their fluxes bootstrapped. By default it
c   will use all sources, except the beforementioned planet of course.
c@ flux
c   When a values are given, they will correspond to the fluxes of the
c   quasars listed in the previous keyword. In this mode the fluxes of
c   the quasars are patched. Note that you need to rerun CALFIT or CALIB
c   in order to correct the gain fits. The other option is to run CALMAKE
c   again with your newly acquired flux from this program.
c@ maxvalid
c   Maximum Validity allowed in evalpoly - Expert option, better not use this.
c   see also evalpoly. Reasonable values are 0, 1 or 2.
c   Default: 1.
c@ log
c   Output device. (default is standard user output)
c
c------ Declarations ---------------------------------------------------c
c >>> ../../../src/inc/caldefs.h
c=======================================================================
c [caldefs.h]
c General include file that every calibration routine needs. It does not
c define (space for) data, but defines the constants that should be used
c in defining array-sizes.
c	18-nov-90 	added PI and TWOPI			PJT
c	 6-jul-91	V6.3 new MAXANTHC and MAXBASHC          pjt
c	26-sep-91       increased MAXUVPNT and MAXBREAK
c	 2-dec-91	V6.4 changed calbflux's dimensionality	PJT
c	19-apr-93	MAXSLOT from 20 to 36 for screwed mode-4  PJT
c			added MAXWIN from 8 (hardcoded) to 16
c	15-apr-94	MAXWIN is now in maxdim.h		PJT/mchw
c       19-dec-95       increase MAXANTHC from 6 to 9 !!!       pjt
c        9-Feb-99       increase MAXANTHC from 9 to 10 !       mchw
c       25-Apr-06       Increase MAXANTHC from 10 to 15 for CARMA ! mh
c=======================================================================
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c=======================================================================
c >>> ../../../src/inc/size.h
c=======================================================================
c	[size.h]
c
c  Include file used by calibration software
c
c  UL, RI, ULRI -- parameters which stand for LSB/USB and REAL/IMAG stuff
c
        integer UL, RI, ULRI
        parameter( UL = 2 )
        parameter( RI = 2 )
        parameter( ULRI = 4 )
c
c  SIZE -- lengths of reals/integers in number of bytes (IEEE/2s compl)
c
        integer SIZE
        parameter( SIZE = 4 )
c=======================================================================
c
c <<< ../../../src/inc/size.h
c=======================================================================
c
c  SVERSION -- current version of cal format (consistency checks)
c              Version number is 10*MAJOR+MINOR
c              Datasets with a MAJOR difference, cannot be read
c              anymore, MINOR differences are normally compatible
c
      INTEGER SVERSION
      PARAMETER ( SVERSION = 64)
c
c MAXANTHC -- special Hat Creek MAXANT, to keep it small for us....
c MAXBASHC -- derived from MAXANTHC = max. number of baselines
c
c Now changed the number from Hat-Creek (10) to Cedar-Flat (15)
c although the name is the same.
c
      INTEGER MAXANTHC, MAXBASHC
      PARAMETER (MAXANTHC=15, MAXBASHC=((MAXANTHC*(MAXANTHC-1))/2))
c
c MAXUVPNT -- maximum number of UV points in time per track
c
      INTEGER   MAXUVPNT
      PARAMETER(MAXUVPNT=2000)
c
c  MAXSRC -- maximum number of sources in multi-source databases
c
      INTEGER MAXSRC
      PARAMETER( MAXSRC = 32 )
c
c  MAXBREAK -- maximum number of breakpoints on an interval
c
      INTEGER MAXBREAK
      PARAMETER( MAXBREAK = 10 )
c
c  MAXPOLY -- maximum number of polynomials on an interval
c             is normally one more than number of breakpoints, right?
c
      INTEGER MAXPOLY
      PARAMETER( MAXPOLY = MAXBREAK+1 )
c
c  MAXSLOT -- maximum number of polynomial slots
c             Allows for simultaneous presence of gcal & pcal format
c             The first 4 for gcal, last 16 for pcal
c             CALIB makes use of slot 5 and 6 for phase diff&mean
      INTEGER MAXSLOT
c      PARAMETER( MAXSLOT = 20 )
      PARAMETER( MAXSLOT = 36 )
c
c  MAXORDER -- highest allowed polynomial order
c             Don't set too high, inversions are already needed in
c             double precision this way.
      INTEGER MAXORDER
      PARAMETER( MAXORDER = 5 )
c
c  MAXWIN -- maximum number of windows - normally equals MAXSPECT = 8
c	     but temporarely increased to 16 to deal with mode=4
c	     problems.
c     INTEGER MAXWIN
c     PARAMETER ( MAXWIN = 16 )
c
c=======================================================================
c
c <<< ../../../src/inc/caldefs.h
c >>> ../../../src/inc/calsubs.h
c=======================================================================
c	[calsubs.h]
c
c	Last update:    22-may-90	deleted pmode	PJT
c			18-dec-90	played with  /calmode/ [Arie/Peter]
c			 7-jul-91       new MAXBASHC parameter
c			11-dec-91	documentation formalized in doc format
c
c*calsubs_h -- header file for calibration
c:calibration, include
c&pjt
c+
c
c  time0 -- Julian time offset for all time measurements
        DOUBLE PRECISION time0
c
c  base(nbl) -- the baseline number and order
        INTEGER nbl, base(MAXBASHC)
c
c  version -- the read version of cal format
        INTEGER version
c
c  calsubs -- THE common block
        COMMON /calsubs/ time0, nbl, base, version
c
c
c  kludge before upgrading to the next CAL-SVERSION
        INTEGER scalmode
c
c	0:	(old system) fitting is done in K
c	1:	fitting is done in data space (K/Jy), i.e. the same is above
c      -1:	fitting is done in gain space (Jy/K)
c
c   For now, scalmode is set in calfit and calflag and such.
c   CAopen should get an extra stamp for this, see calio.h
c
        COMMON /calscal/scalmode
c--
c   Mode of phase/amp calibration in calflag/calib/...
c   [may be used in Arie's calib.for - but is internal for now]
c   nmode=1   amp
c	  2   phase
c         3   diff
c
c	INTEGER nmode
c	COMMON /calmode/nmode
c=======================================================================
c
c <<< ../../../src/inc/calsubs.h
c >>> ../../../src/inc/caldata.h
c=======================================================================
c       [caldata.h]
c       Header file for calibration data
c
c       Last update:    14-mar-90       added Sflag     PJT
c                       20-mar-90       srcindex now srcindex(MAXPNT)   PJT
c                        7-may-90       rdata(ULRI,MAXBASHC,MAXUVPNT)   PJT
c                        4-sep-90       baseline based calib stuff      PJT
c                       18-nov-90       more descriptive comments       PJT
c                       18-dec-90       played with rdata()             PJT
c                        7-jul-91       MAXBASE -> BAXMASHC             PJT
c                        2-dec-91       make fluxes time dependant, not
c                                       source dependant; =V6.4 fmt     PJT
c                       11-dec-91       documentation formalized in doc format
c*caldata_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c   The calibration format on disk is split out in memory over several
c   common blocks, each of them described in a separate include file.
c
c
c <general header> (see also calsubs.h)
c       double time0            time to which all times are offset (JD)
c       int    nbl              number of baselines
c       int    base(nbl)        nbl baselines (notation: A1*256+A2)
c       int    version          version of the calibration format
c
c <data> (this caldata.h include file)
c   rtime
c       float  rtime(rcount)             offset times to time0
c   rdata
c       float  rdata(ULRI,nbl,rcount)    correllation data
c   rflag
c       int    rflag(UL,nbl,rcount)      flagged bad? (0=false, 1=true)
c   sindex
c       int    sindex(rcount)            source index number
c   sname
c       char*16 sname(scount)             list of sources
c
c
c
c  Rxxxx -- the actual calibration data
c
      INTEGER   rcount
      REAL      rtime(MAXUVPNT)
      REAL      rdata(ULRI, MAXBASHC, MAXUVPNT)
      INTEGER   rflag(UL, MAXBASHC, MAXUVPNT)
c
c  Sxxxx -- index for multiple source calibration format
c           (plstuff was added later and is not quite needed??)
c
      INTEGER   scount
      CHARACTER sname(MAXSRC)*16
      INTEGER   sindex(MAXUVPNT), sflag(MAXUVPNT)
      REAL      plstuff(4,MAXSRC)
c
c  Bxxxx - list of break points
c
      REAL      btime(MAXBREAK, 2, MAXBASHC)
      INTEGER   bcount(2, MAXBASHC)
c
c  Xxxxx - odds and ends, should not really have to exist
c          are merely there for planets
c
      REAL      volts(MAXUVPNT)
c
c  baseline based calibrator fluxes, per time
c
      REAL      calbflux(MAXBASHC,MAXUVPNT)
c
c
c  the common blocks with all the data
c
      COMMON /caldatr/ rcount, scount, rtime, rdata, rflag,
     *                 sindex, btime, bcount, sflag, plstuff,volts
      COMMON /caldatc/ sname
      COMMON /caldatp/ calbflux
c--
c=======================================================================
c
c <<< ../../../src/inc/caldata.h
c >>> ../../../src/inc/calpoly.h
c=======================================================================
c	[calpoly.h]
c
c	last update: 26-apr-90	new poly's
c		     18-nov-90  descriptive comments
c		      7-jul-91  new MAXBASHC parameter
c*calpoly_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c
c  order -- current polynomial order for a slot
c
      INTEGER order(MAXSLOT)
c
c
c  pcount -- number of polynomials in a (slot,baseline) pair
c
      INTEGER pcount(MAXSLOT, MAXBASHC)
c
c
c  tvalid - validity of time interval
c		(min & max time-range; breakpoints; slotcode; baseline)
c
      REAL tvalid(2,MAXBREAK+1,MAXSLOT,MAXBASHC)
c
c  poly -- actual polynomial coefficients
c		(order; breakpoints, slotcode, baseline)
c
      REAL poly( 0:MAXORDER, MAXBREAK+1, MAXSLOT, MAXBASHC )
c
c
c  CALPOLY -- the common block
c
      COMMON /calpoly/ pcount, order, tvalid, poly
c--
c=======================================================================
c <<< ../../../src/inc/calpoly.h
c
      CHARACTER PVERSION*(*)
      PARAMETER (PVERSION='Version 18-apr-2006')
c
      CHARACTER ctime0*40, line*132, dataset*100, logfile*100
      CHARACTER s2code*4, code*4
      CHARACTER  source(MAXSRC)*17
      CHARACTER bname*5, blname*5, planet*12, quasar*12
      INTEGER   i, j, p, p1, b, nsources, nfluxes
      INTEGER   valid, sum0, pidx, qidx, maxvalid, tno, iostat
      REAL      pgain(2,MAXBASHC), qgain(2), qflux(2), t
      REAL      sum1, sum2, flux(MAXSRC)
      LOGICAL   more
c
      INTEGER   len1, srctime
      REAL      evalpoly
      LOGICAL   chkpoly, isplanet
c
c------ Address the crowd-----------------------------------------------
c
      CALL output('CALBOOT: '//PVERSION)
c
c------ Get the inputs -------------------------------------------------
c       plus some initial error checking
      CALL keyini
      CALL keyf( 'gcal', dataset, ' ' )
      CALL keyf( 'planet', planet, ' ')
      CALL mkeya('source', source, MAXSRC, nsources)
      CALL mkeyr('flux',   flux,   MAXSRC, nfluxes)
      CALL keyi('maxvalid',maxvalid,1)
      CALL keya( 'log',logfile,' ')
      CALL keyfin
c
      CALL assertl(dataset.NE.' ',
     *                  'No calibration data set given (gcal=)')
c        All sources are stored in upper case in the databases, and
c        so for safety we ucase them all.
      CALL ucase(planet)
      DO i=1,nsources
         CALL ucase(source(i))
      ENDDO
      IF (planet.NE.' ' .AND. .NOT.isplanet(planet)) CALL bug('i',
     *            'Hum, '//planet//' is not a known planet to me')
c
c	* open logfile for output
      CALL logopen(logfile,' ')
c
c------ Get the raw data -----------------------------------------------
c       with its breakpoints and polynomials
c
      CALL readset(dataset)
      CALL readbrk(dataset)
      CALL getpoly(dataset)
c
c       In case user wanted to reset the flux of a real calibrator
c       (not a planet!!!) it's done here
c       It's patched in the right spot, and written back to disk
c	after that the program must stop, since the fits are
c	invalid at this point
c
      IF(nfluxes.GT.0)THEN
        IF(nsources.NE.nfluxes) CALL bug('f',
     *      'You need to supply an equal amount of source= and flux=')
        DO i=1,nsources
            CALL setsflag(1,source(i))
            DO j=1,rcount
                IF(sflag(j).EQ.1) THEN
                    DO b=1,nbl
                        calbflux(b,j) = flux(i)
                    ENDDO
                ENDIF
            ENDDO
            WRITE(line,'(A,A,A,F7.2,A)') 'Source ',source(i),
     *          ' patched with new flux = ',flux(i),' Jy/K.'
            CALL output(line)
        ENDDO
        CALL putsrc(dataset)
        CALL hopen(tno,dataset,'old',iostat)
        CALL hisopen(tno,'append')
        CALL hiswrite(tno,'CALBOOT: '//PVERSION)
        CALL hisinput(tno,'CALBOOT')
        CALL hisclose(tno)
        CALL hclose(tno)
        CALL bug('i',
     *        'Do NOT forget to rerun CALFIT/CALIB before CALAPPLY')
        STOP
      ENDIF
c
c       In case no planet name was given, try and retrieve it from the
c       gcal dataset....The list of sources is in sname(1:scount)
      i=1
      DOWHILE(planet.EQ.' ' .AND. i.LE.scount)
         IF(isplanet(sname(i))) planet = sname(i)
         i=i+1
      ENDDO
      IF(planet.EQ.' ') CALL bug('f',
     *      'Could not find a valid planet in '//dataset)
c
c        Now set all sflag's true for that planet only
c        and check how many we've really got..and get an index where it is
c
      pidx = srctime(planet)
      IF(pidx.EQ.0) CALL bug('f',
     *           'Planet '//planet(1:len1(planet))//' not found')
c
c	Scan over all baselines, and U and L sideband and fill in the
c	gains (Jy/K) of the planet in question
c       rtime(pidx) is the time at which we're checking the gain of the planet
c       Note there is no check yet that the planet doesn't have multiple
c       breakpoints...
      t = rtime(pidx)
      CALL julday(time0+t,'H',ctime0)
      WRITE(line,'(a,a,f9.6,a,a,a)') planet,' check at time ',t,' = ',
     *           ' UT=',ctime0
      CALL logwrite(line,more)
      DO b=1,nbl
         bname = blname(base(b))
         DO p=1,3,2
            p1=(p+1)/2
            code=s2code(p)
            IF(.NOT.chkpoly(code))CALL bug('f',
     *         'No AMP polynomial fit present; use CALFIT or CALIB')
            pgain(p1,b) = evalpoly(t, code, base(b),valid)
c            IF(valid.NE.0) CALL bug('i','Evalpoly extrapolated')
         ENDDO
         WRITE(line,100) bname(1:len1(bname)),pgain(1,b),pgain(2,b),
     *                   calbflux(b,pidx)
         CALL logwrite(line,more)
      ENDDO
c
c-----------------------------------------------------------------------
c
c Figure out the list of sources from the gcal file if user didn't supply
c a list by hand - this would include all sources, BUT the planet itself.
c
      IF (nsources.EQ.0) THEN
         i=0
         DO WHILE(i.LT.scount)
            i=i+1
            IF(sname(i).NE.planet) THEN
               nsources = nsources + 1
               source(nsources) = sname(i)
            ENDIF
         ENDDO
      ENDIF
      IF(nsources.EQ.0) CALL bug('w',
     *                 'No more sources besides your planet')
c
c-----------------------------------------------------------------------
c  Now loop over all requested sources, compare the gain with that of
c  the planet, and report what the flux would be.... Do this for all
c  breakpoint interval, for all baselines, and for lower and upper
c  sideband.
c
      CALL bug('i','No breakdown in breakpoints yet')
      DO j=1,nsources
         quasar = source(j)
         qidx = srctime(quasar)
         IF(qidx.GT.0) THEN
            t = rtime(qidx)
            CALL julday(time0+t,'H',ctime0)
            WRITE(line,'(a,a,f9.6,a,a,a)') quasar,' check at time ',t,
     *                 ' UT=',ctime0
            CALL logwrite(line,more)
            sum0 = 0
            sum1 = 0.0
            sum2 = 0.0
            DO b=1,nbl
               bname = blname(base(b))
               DO p=1,3,2
                  p1=(p+1)/2
                  qgain(p1) = 0.0
                  qflux(p1) = 0.0
                  code=s2code(p)
c                                   loop until inside a proper interval
                  valid=maxvalid+1
                  i=qidx
                  DOWHILE(valid.GT.maxvalid .AND. i.LE.rcount)
                    IF(sname(sindex(i)).EQ.quasar .AND.
     *                                          rflag(p1,b,i).EQ.1) THEN
                      t=rtime(i)
                      qgain(p1) = evalpoly(t, code, base(b),valid)
                      IF(valid.GE.0 .AND. valid.LE.maxvalid)THEN
                        qflux(p1)=calbflux(b,qidx)*pgain(p1,b)/qgain(p1)
                        sum0 = sum0 + 1
                        sum1 = sum1 + qflux(p1)
                        sum2 = sum2 + qflux(p1)*qflux(p1)
                      ENDIF
                    ENDIF
                    i=i+1
                  ENDDO
               ENDDO
               IF(valid.GE.0 .AND. valid.LE.maxvalid)THEN
                 WRITE(line,200) bname(1:len1(bname)),qgain(1),qgain(2),
     *                      calbflux(b,qidx),qflux(1),qflux(2)
                 CALL logwrite(line,more)
               ENDIF
            ENDDO
            IF(sum0.GT.0) THEN
              sum1 = sum1/REAL(sum0)
              sum2 = SQRT(sum2/REAL(sum0)-sum1*sum1)
              WRITE(line,300) quasar(1:len1(quasar)),sum1,sum2
              call logwrite(line,more)
            ENDIF
         ELSE
            CALL bug('w','Skipping quasar '//
     *               quasar(1:len1(quasar))//' : no data')
         ENDIF
      ENDDO
c
 100  FORMAT('Bl ',A,' Gain(L,U):  ',F7.2,1x,F7.2,
     *       ' Jy/K;  Assumed flux= ',F7.2,' Jy')
 200  FORMAT('Bl ',A,' Gain(L,U):  ',F7.2,1x,F7.2,
     *       ' Jy/K;Old= ',F7.2,' Jy',';New= ',F7.2,1x,F7.2,' Jy')
 300  FORMAT(10x,'*** Mean value for ',A,': ',F7.2,' +/- ',F5.2,' Jy.')
c
      END
c***********************************************************************
c
      INTEGER FUNCTION srctime(name)
c
      CHARACTER*(*) name
c >>> ../../../src/inc/caldefs.h
c=======================================================================
c [caldefs.h]
c General include file that every calibration routine needs. It does not
c define (space for) data, but defines the constants that should be used
c in defining array-sizes.
c	18-nov-90 	added PI and TWOPI			PJT
c	 6-jul-91	V6.3 new MAXANTHC and MAXBASHC          pjt
c	26-sep-91       increased MAXUVPNT and MAXBREAK
c	 2-dec-91	V6.4 changed calbflux's dimensionality	PJT
c	19-apr-93	MAXSLOT from 20 to 36 for screwed mode-4  PJT
c			added MAXWIN from 8 (hardcoded) to 16
c	15-apr-94	MAXWIN is now in maxdim.h		PJT/mchw
c       19-dec-95       increase MAXANTHC from 6 to 9 !!!       pjt
c        9-Feb-99       increase MAXANTHC from 9 to 10 !       mchw
c       25-Apr-06       Increase MAXANTHC from 10 to 15 for CARMA ! mh
c=======================================================================
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c=======================================================================
c >>> ../../../src/inc/size.h
c=======================================================================
c	[size.h]
c
c  Include file used by calibration software
c
c  UL, RI, ULRI -- parameters which stand for LSB/USB and REAL/IMAG stuff
c
        integer UL, RI, ULRI
        parameter( UL = 2 )
        parameter( RI = 2 )
        parameter( ULRI = 4 )
c
c  SIZE -- lengths of reals/integers in number of bytes (IEEE/2s compl)
c
        integer SIZE
        parameter( SIZE = 4 )
c=======================================================================
c
c <<< ../../../src/inc/size.h
c=======================================================================
c
c  SVERSION -- current version of cal format (consistency checks)
c              Version number is 10*MAJOR+MINOR
c              Datasets with a MAJOR difference, cannot be read
c              anymore, MINOR differences are normally compatible
c
      INTEGER SVERSION
      PARAMETER ( SVERSION = 64)
c
c MAXANTHC -- special Hat Creek MAXANT, to keep it small for us....
c MAXBASHC -- derived from MAXANTHC = max. number of baselines
c
c Now changed the number from Hat-Creek (10) to Cedar-Flat (15)
c although the name is the same.
c
      INTEGER MAXANTHC, MAXBASHC
      PARAMETER (MAXANTHC=15, MAXBASHC=((MAXANTHC*(MAXANTHC-1))/2))
c
c MAXUVPNT -- maximum number of UV points in time per track
c
      INTEGER   MAXUVPNT
      PARAMETER(MAXUVPNT=2000)
c
c  MAXSRC -- maximum number of sources in multi-source databases
c
      INTEGER MAXSRC
      PARAMETER( MAXSRC = 32 )
c
c  MAXBREAK -- maximum number of breakpoints on an interval
c
      INTEGER MAXBREAK
      PARAMETER( MAXBREAK = 10 )
c
c  MAXPOLY -- maximum number of polynomials on an interval
c             is normally one more than number of breakpoints, right?
c
      INTEGER MAXPOLY
      PARAMETER( MAXPOLY = MAXBREAK+1 )
c
c  MAXSLOT -- maximum number of polynomial slots
c             Allows for simultaneous presence of gcal & pcal format
c             The first 4 for gcal, last 16 for pcal
c             CALIB makes use of slot 5 and 6 for phase diff&mean
      INTEGER MAXSLOT
c      PARAMETER( MAXSLOT = 20 )
      PARAMETER( MAXSLOT = 36 )
c
c  MAXORDER -- highest allowed polynomial order
c             Don't set too high, inversions are already needed in
c             double precision this way.
      INTEGER MAXORDER
      PARAMETER( MAXORDER = 5 )
c
c  MAXWIN -- maximum number of windows - normally equals MAXSPECT = 8
c	     but temporarely increased to 16 to deal with mode=4
c	     problems.
c     INTEGER MAXWIN
c     PARAMETER ( MAXWIN = 16 )
c
c=======================================================================
c
c <<< ../../../src/inc/caldefs.h
c >>> ../../../src/inc/calsubs.h
c=======================================================================
c	[calsubs.h]
c
c	Last update:    22-may-90	deleted pmode	PJT
c			18-dec-90	played with  /calmode/ [Arie/Peter]
c			 7-jul-91       new MAXBASHC parameter
c			11-dec-91	documentation formalized in doc format
c
c*calsubs_h -- header file for calibration
c:calibration, include
c&pjt
c+
c
c  time0 -- Julian time offset for all time measurements
        DOUBLE PRECISION time0
c
c  base(nbl) -- the baseline number and order
        INTEGER nbl, base(MAXBASHC)
c
c  version -- the read version of cal format
        INTEGER version
c
c  calsubs -- THE common block
        COMMON /calsubs/ time0, nbl, base, version
c
c
c  kludge before upgrading to the next CAL-SVERSION
        INTEGER scalmode
c
c	0:	(old system) fitting is done in K
c	1:	fitting is done in data space (K/Jy), i.e. the same is above
c      -1:	fitting is done in gain space (Jy/K)
c
c   For now, scalmode is set in calfit and calflag and such.
c   CAopen should get an extra stamp for this, see calio.h
c
        COMMON /calscal/scalmode
c--
c   Mode of phase/amp calibration in calflag/calib/...
c   [may be used in Arie's calib.for - but is internal for now]
c   nmode=1   amp
c	  2   phase
c         3   diff
c
c	INTEGER nmode
c	COMMON /calmode/nmode
c=======================================================================
c
c <<< ../../../src/inc/calsubs.h
c >>> ../../../src/inc/caldata.h
c=======================================================================
c       [caldata.h]
c       Header file for calibration data
c
c       Last update:    14-mar-90       added Sflag     PJT
c                       20-mar-90       srcindex now srcindex(MAXPNT)   PJT
c                        7-may-90       rdata(ULRI,MAXBASHC,MAXUVPNT)   PJT
c                        4-sep-90       baseline based calib stuff      PJT
c                       18-nov-90       more descriptive comments       PJT
c                       18-dec-90       played with rdata()             PJT
c                        7-jul-91       MAXBASE -> BAXMASHC             PJT
c                        2-dec-91       make fluxes time dependant, not
c                                       source dependant; =V6.4 fmt     PJT
c                       11-dec-91       documentation formalized in doc format
c*caldata_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c   The calibration format on disk is split out in memory over several
c   common blocks, each of them described in a separate include file.
c
c
c <general header> (see also calsubs.h)
c       double time0            time to which all times are offset (JD)
c       int    nbl              number of baselines
c       int    base(nbl)        nbl baselines (notation: A1*256+A2)
c       int    version          version of the calibration format
c
c <data> (this caldata.h include file)
c   rtime
c       float  rtime(rcount)             offset times to time0
c   rdata
c       float  rdata(ULRI,nbl,rcount)    correllation data
c   rflag
c       int    rflag(UL,nbl,rcount)      flagged bad? (0=false, 1=true)
c   sindex
c       int    sindex(rcount)            source index number
c   sname
c       char*16 sname(scount)             list of sources
c
c
c
c  Rxxxx -- the actual calibration data
c
      INTEGER   rcount
      REAL      rtime(MAXUVPNT)
      REAL      rdata(ULRI, MAXBASHC, MAXUVPNT)
      INTEGER   rflag(UL, MAXBASHC, MAXUVPNT)
c
c  Sxxxx -- index for multiple source calibration format
c           (plstuff was added later and is not quite needed??)
c
      INTEGER   scount
      CHARACTER sname(MAXSRC)*16
      INTEGER   sindex(MAXUVPNT), sflag(MAXUVPNT)
      REAL      plstuff(4,MAXSRC)
c
c  Bxxxx - list of break points
c
      REAL      btime(MAXBREAK, 2, MAXBASHC)
      INTEGER   bcount(2, MAXBASHC)
c
c  Xxxxx - odds and ends, should not really have to exist
c          are merely there for planets
c
      REAL      volts(MAXUVPNT)
c
c  baseline based calibrator fluxes, per time
c
      REAL      calbflux(MAXBASHC,MAXUVPNT)
c
c
c  the common blocks with all the data
c
      COMMON /caldatr/ rcount, scount, rtime, rdata, rflag,
     *                 sindex, btime, bcount, sflag, plstuff,volts
      COMMON /caldatc/ sname
      COMMON /caldatp/ calbflux
c--
c=======================================================================
c
c <<< ../../../src/inc/caldata.h
c >>> ../../../src/inc/calpoly.h
c=======================================================================
c	[calpoly.h]
c
c	last update: 26-apr-90	new poly's
c		     18-nov-90  descriptive comments
c		      7-jul-91  new MAXBASHC parameter
c*calpoly_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c
c  order -- current polynomial order for a slot
c
      INTEGER order(MAXSLOT)
c
c
c  pcount -- number of polynomials in a (slot,baseline) pair
c
      INTEGER pcount(MAXSLOT, MAXBASHC)
c
c
c  tvalid - validity of time interval
c		(min & max time-range; breakpoints; slotcode; baseline)
c
      REAL tvalid(2,MAXBREAK+1,MAXSLOT,MAXBASHC)
c
c  poly -- actual polynomial coefficients
c		(order; breakpoints, slotcode, baseline)
c
      REAL poly( 0:MAXORDER, MAXBREAK+1, MAXSLOT, MAXBASHC )
c
c
c  CALPOLY -- the common block
c
      COMMON /calpoly/ pcount, order, tvalid, poly
c--
c=======================================================================
c <<< ../../../src/inc/calpoly.h
c
c   For given source name, return an index in the rtime() array where
c   that source appears. Currently it simply returns the first one
c   encountered....
c   Note that the sflag() arrary is modified for this. This routine can
c   also be used to check if a source is present at all, in which case
c   it should return a non-zero index.
c
      INTEGER i
c
c  set all flags true for that source only
      CALL setsflag(1,name)
c
c  check for which the flag is true...and return it's index
      srctime = 0
      DO i=1,rcount
         IF (sflag(i).EQ.1) THEN
            srctime = i
            RETURN
         ENDIF
      ENDDO
      END
c***********************************************************************
      LOGICAL function isplanet(name)
c
      CHARACTER name*(*)
c
c   If 'name' is a known planet, return TRUE, else FALSE.
c   Contains some hardcoded information about our solar system
c   AD 1931. Note we assume 'name' has been set to upper case
c   before calling this routine.
c
      IF (name.EQ.'MERCURY' .OR.
     *    name.EQ.'VENUS'   .OR.
     *    name.EQ.'EARTH'   .OR.
     *    name.EQ.'MARS'    .OR.
     *    name.EQ.'JUPITER' .OR.
     *    name.EQ.'SATURN'  .OR.
     *    name.EQ.'URANUS'  .OR.
     *    name.EQ.'NEPTUNE' .OR.
     *    name.EQ.'PLUTO') THEN
         isplanet = .TRUE.
      ELSE
         isplanet = .FALSE.
      ENDIF
c
      END
