        PROGRAM calfit
c
c	CalFit -- phase/amp fit for gain calibration
c
c  NOTE NOTE:  Any major changes to this program MUST be braught
c               back to CALIB, wherever appropriate
c
c	xx-xxx-xx   Brian Sutin   orignal version
c	7-dec-89    PJT	 history written
c	30-jan-90   PJT  new data structure
c	26-apr-90   PJT  new poly stuff
c	13-jul-90   PJT  vector averaging and Lee's new flipper
c	19-jul-90   pjt  also set default of close=FALSE in code now (aj)
c	19-dec-90   pjt  use mirconst.h now
c	31-jan-91   pjt  addhist new parameters
c        7-feb-91   pjt  verbosities
c	16-may-91   pjt  new doc
c        7-aug-91   pjt  phase fit stastistics, 'nowrite' option added
c	17-oct-91   pjt  removed output bugs, and report dispersions/interval
c 	30-nov-91   pjt  moved setxy() over to calsubs.for
c	 2-dec-91   pjt  new calbflux (V6.4) format
c	10-dec-91   pjt  [comments]
c      27-mar-92    assertl now
c
c------ Inline doc (retrieved with doc to a .doc file) ----------------c
c
c= calfit - Fit polynomial to amp/phase data
c: calibration
c& pjt
c+
c   CALFIT is a MIRIAD task which fits polynomials to the wideband data
c   in a calibration dataset previously made by CALMAKE.
c   By default, CALFIT uses 2nd order for phases and 0th order for
c   amplitudes, without phase closure.  The resulting fit is stored in
c   various items in the calibration dataset, and can be over-written.
c   (See also DELHD, PUTHD, and COPYHD as MIRIAD system utilities)
c   When a 0th order fit is used for amplitude, CALFIT also displays the
c   gain factors (Jy/K) for each baseline, sideband and interval. The
c   dispersion of the phases from the least squares fit (radians) is also
c   reported.
c@ gcal
c   The calibration dataset, as produced by CALMAKE. No default.
c@ source
c   Names of the sources to which the fit is applied.
c   Default is to select all sources present in the calibration set.
c@ order
c   The polynomial orders to be fit to the amplitude and phase. The
c   maximum order for both amplitude and phase is 10. Default: 0,2.
c@ close
c   If set to true, a baseline based phase closure will be used in the
c   gain fits.
c   For 'true', breakpoints are forced to be equal across baselines,
c   taken from the first baseline found in the database.
c   Default: false
c@ taver
c   Taver consists of two numbers, ``tgap'' and ``ttot'', both in minutes,
c   which are used for vector averaging. If the time interval between
c   any two successive data points is greater than ``tgap``, or if the
c   total time between the first data point in a vector average and any
c   succeeding data point exceeds ``ttot'', then a new vector average is
c   started.
c   Default: 0.0,1000.0 (no vector averaging)
c@ mode
c   Fit 'phase' or 'amplitude'. If nothing specified, both
c   will be fit. Default: both.
c@ write
c   A logical, which specifies whether to (over)write the calculated
c   polynomial coefficients. The item ``pdata'' is used for this.
c   This allows a ``dryrun'', only showing gains and phase fits.
c   Default: true.
c@ units
c   Units to work in. Options are: K, K/Jy, Jy/K. See comments in CALFLAG.
c   Default: Jy/K.
c
c------ Declarations ---------------------------------------------------c
c >>> ../../../src/inc/caldefs.h
c=======================================================================
c [caldefs.h]
c General include file that every calibration routine needs. It does not
c define (space for) data, but defines the constants that should be used
c in defining array-sizes.
c	18-nov-90 	added PI and TWOPI			PJT
c	 6-jul-91	V6.3 new MAXANTHC and MAXBASHC          pjt
c	26-sep-91       increased MAXUVPNT and MAXBREAK
c	 2-dec-91	V6.4 changed calbflux's dimensionality	PJT
c	19-apr-93	MAXSLOT from 20 to 36 for screwed mode-4  PJT
c			added MAXWIN from 8 (hardcoded) to 16
c	15-apr-94	MAXWIN is now in maxdim.h		PJT/mchw
c       19-dec-95       increase MAXANTHC from 6 to 9 !!!       pjt
c        9-Feb-99       increase MAXANTHC from 9 to 10 !       mchw
c       25-Apr-06       Increase MAXANTHC from 10 to 15 for CARMA ! mh
c=======================================================================
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c=======================================================================
c >>> ../../../src/inc/size.h
c=======================================================================
c	[size.h]
c
c  Include file used by calibration software
c
c  UL, RI, ULRI -- parameters which stand for LSB/USB and REAL/IMAG stuff
c
        integer UL, RI, ULRI
        parameter( UL = 2 )
        parameter( RI = 2 )
        parameter( ULRI = 4 )
c
c  SIZE -- lengths of reals/integers in number of bytes (IEEE/2s compl)
c
        integer SIZE
        parameter( SIZE = 4 )
c=======================================================================
c
c <<< ../../../src/inc/size.h
c=======================================================================
c
c  SVERSION -- current version of cal format (consistency checks)
c              Version number is 10*MAJOR+MINOR
c              Datasets with a MAJOR difference, cannot be read
c              anymore, MINOR differences are normally compatible
c
      INTEGER SVERSION
      PARAMETER ( SVERSION = 64)
c
c MAXANTHC -- special Hat Creek MAXANT, to keep it small for us....
c MAXBASHC -- derived from MAXANTHC = max. number of baselines
c
c Now changed the number from Hat-Creek (10) to Cedar-Flat (15)
c although the name is the same.
c
      INTEGER MAXANTHC, MAXBASHC
      PARAMETER (MAXANTHC=15, MAXBASHC=((MAXANTHC*(MAXANTHC-1))/2))
c
c MAXUVPNT -- maximum number of UV points in time per track
c
      INTEGER   MAXUVPNT
      PARAMETER(MAXUVPNT=2000)
c
c  MAXSRC -- maximum number of sources in multi-source databases
c
      INTEGER MAXSRC
      PARAMETER( MAXSRC = 32 )
c
c  MAXBREAK -- maximum number of breakpoints on an interval
c
      INTEGER MAXBREAK
      PARAMETER( MAXBREAK = 10 )
c
c  MAXPOLY -- maximum number of polynomials on an interval
c             is normally one more than number of breakpoints, right?
c
      INTEGER MAXPOLY
      PARAMETER( MAXPOLY = MAXBREAK+1 )
c
c  MAXSLOT -- maximum number of polynomial slots
c             Allows for simultaneous presence of gcal & pcal format
c             The first 4 for gcal, last 16 for pcal
c             CALIB makes use of slot 5 and 6 for phase diff&mean
      INTEGER MAXSLOT
c      PARAMETER( MAXSLOT = 20 )
      PARAMETER( MAXSLOT = 36 )
c
c  MAXORDER -- highest allowed polynomial order
c             Don't set too high, inversions are already needed in
c             double precision this way.
      INTEGER MAXORDER
      PARAMETER( MAXORDER = 5 )
c
c  MAXWIN -- maximum number of windows - normally equals MAXSPECT = 8
c	     but temporarely increased to 16 to deal with mode=4
c	     problems.
c     INTEGER MAXWIN
c     PARAMETER ( MAXWIN = 16 )
c
c=======================================================================
c
c <<< ../../../src/inc/caldefs.h
c >>> ../../../src/inc/calsubs.h
c=======================================================================
c	[calsubs.h]
c
c	Last update:    22-may-90	deleted pmode	PJT
c			18-dec-90	played with  /calmode/ [Arie/Peter]
c			 7-jul-91       new MAXBASHC parameter
c			11-dec-91	documentation formalized in doc format
c
c*calsubs_h -- header file for calibration
c:calibration, include
c&pjt
c+
c
c  time0 -- Julian time offset for all time measurements
        DOUBLE PRECISION time0
c
c  base(nbl) -- the baseline number and order
        INTEGER nbl, base(MAXBASHC)
c
c  version -- the read version of cal format
        INTEGER version
c
c  calsubs -- THE common block
        COMMON /calsubs/ time0, nbl, base, version
c
c
c  kludge before upgrading to the next CAL-SVERSION
        INTEGER scalmode
c
c	0:	(old system) fitting is done in K
c	1:	fitting is done in data space (K/Jy), i.e. the same is above
c      -1:	fitting is done in gain space (Jy/K)
c
c   For now, scalmode is set in calfit and calflag and such.
c   CAopen should get an extra stamp for this, see calio.h
c
        COMMON /calscal/scalmode
c--
c   Mode of phase/amp calibration in calflag/calib/...
c   [may be used in Arie's calib.for - but is internal for now]
c   nmode=1   amp
c	  2   phase
c         3   diff
c
c	INTEGER nmode
c	COMMON /calmode/nmode
c=======================================================================
c
c <<< ../../../src/inc/calsubs.h
c >>> ../../../src/inc/caldata.h
c=======================================================================
c       [caldata.h]
c       Header file for calibration data
c
c       Last update:    14-mar-90       added Sflag     PJT
c                       20-mar-90       srcindex now srcindex(MAXPNT)   PJT
c                        7-may-90       rdata(ULRI,MAXBASHC,MAXUVPNT)   PJT
c                        4-sep-90       baseline based calib stuff      PJT
c                       18-nov-90       more descriptive comments       PJT
c                       18-dec-90       played with rdata()             PJT
c                        7-jul-91       MAXBASE -> BAXMASHC             PJT
c                        2-dec-91       make fluxes time dependant, not
c                                       source dependant; =V6.4 fmt     PJT
c                       11-dec-91       documentation formalized in doc format
c*caldata_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c   The calibration format on disk is split out in memory over several
c   common blocks, each of them described in a separate include file.
c
c
c <general header> (see also calsubs.h)
c       double time0            time to which all times are offset (JD)
c       int    nbl              number of baselines
c       int    base(nbl)        nbl baselines (notation: A1*256+A2)
c       int    version          version of the calibration format
c
c <data> (this caldata.h include file)
c   rtime
c       float  rtime(rcount)             offset times to time0
c   rdata
c       float  rdata(ULRI,nbl,rcount)    correllation data
c   rflag
c       int    rflag(UL,nbl,rcount)      flagged bad? (0=false, 1=true)
c   sindex
c       int    sindex(rcount)            source index number
c   sname
c       char*16 sname(scount)             list of sources
c
c
c
c  Rxxxx -- the actual calibration data
c
      INTEGER   rcount
      REAL      rtime(MAXUVPNT)
      REAL      rdata(ULRI, MAXBASHC, MAXUVPNT)
      INTEGER   rflag(UL, MAXBASHC, MAXUVPNT)
c
c  Sxxxx -- index for multiple source calibration format
c           (plstuff was added later and is not quite needed??)
c
      INTEGER   scount
      CHARACTER sname(MAXSRC)*16
      INTEGER   sindex(MAXUVPNT), sflag(MAXUVPNT)
      REAL      plstuff(4,MAXSRC)
c
c  Bxxxx - list of break points
c
      REAL      btime(MAXBREAK, 2, MAXBASHC)
      INTEGER   bcount(2, MAXBASHC)
c
c  Xxxxx - odds and ends, should not really have to exist
c          are merely there for planets
c
      REAL      volts(MAXUVPNT)
c
c  baseline based calibrator fluxes, per time
c
      REAL      calbflux(MAXBASHC,MAXUVPNT)
c
c
c  the common blocks with all the data
c
      COMMON /caldatr/ rcount, scount, rtime, rdata, rflag,
     *                 sindex, btime, bcount, sflag, plstuff,volts
      COMMON /caldatc/ sname
      COMMON /caldatp/ calbflux
c--
c=======================================================================
c
c <<< ../../../src/inc/caldata.h
c ??
c >>> ../../../src/inc/calpoly.h
c=======================================================================
c	[calpoly.h]
c
c	last update: 26-apr-90	new poly's
c		     18-nov-90  descriptive comments
c		      7-jul-91  new MAXBASHC parameter
c*calpoly_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c
c  order -- current polynomial order for a slot
c
      INTEGER order(MAXSLOT)
c
c
c  pcount -- number of polynomials in a (slot,baseline) pair
c
      INTEGER pcount(MAXSLOT, MAXBASHC)
c
c
c  tvalid - validity of time interval
c		(min & max time-range; breakpoints; slotcode; baseline)
c
      REAL tvalid(2,MAXBREAK+1,MAXSLOT,MAXBASHC)
c
c  poly -- actual polynomial coefficients
c		(order; breakpoints, slotcode, baseline)
c
      REAL poly( 0:MAXORDER, MAXBREAK+1, MAXSLOT, MAXBASHC )
c
c
c  CALPOLY -- the common block
c
      COMMON /calpoly/ pcount, order, tvalid, poly
c--
c=======================================================================
c <<< ../../../src/inc/calpoly.h
c >>> ../../../src/inc/calfit.h
c=======================================================================
c	[calfit.h]
c*calfit_h -- calibration include file
c&pjt
c:calibration, include
c+
c  lsq()        matrix used for inversion
c  lsqb()       vector, used as right hand side AND solution
c  xsums()      temporary storage use to fill lsq()
c  ysums()      temporary storage use to fill lsq()
c
c
c  MAXLSQ -- the size of arrays and matrices in the least squares fit
c
      INTEGER MAXLSQ
      PARAMETER(MAXLSQ=(MAXANTHC*(MAXORDER+1)))
c
c  lsq, lsqb -- least squares matrix and rhs
c
      DOUBLE PRECISION lsq( MAXLSQ, MAXLSQ )
      DOUBLE PRECISION lsqb( MAXLSQ )
c
c  xsums, ysums -- sums-of-squares
c
      DOUBLE PRECISION xsums( 0 : 2 * MAXORDER )
      DOUBLE PRECISION ysums( 0 : MAXORDER )
c
      COMMON /lsq/ xsums, ysums, lsq, lsqb
c--
c=======================================================================
c
c <<< ../../../src/inc/calfit.h
c >>> ../../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../../src/inc/mirconst.h
c
        CHARACTER PVERSION*(*)
        PARAMETER (PVERSION='Version 27-mar-92')
c
        CHARACTER   dataset*100, sources(MAXSRC)*8, units*5
        CHARACTER   itoaf*3,s2code*4,acode*4,pcode*4, mode*10
        CHARACTER   stmpp*3, stmpa*3, line*132, bname*5, blname*5
c
        INTEGER i, s, b, phimax, ampmax, count, nsources, tno, iostat
        INTEGER basediff(2,MAXBASHC), bp, s1, pamode
        INTEGER fitpoly, ipol, valid, sum0
        REAL    x(MAXUVPNT), y(MAXUVPNT), taver(2), delphi
        REAL    sum1, sum2, d, evalpoly
        LOGICAL antbase, dowrite
        INTEGER scalunit, len1
c
c------ Address the crowd-----------------------------------------------c
        CALL output('CALFIT: '//PVERSION)
c
c------ Get the inputs -------------------------------------------------c
        CALL  keyini
        CALL  keya('gcal',  dataset, ' ' )
        CALL mkeya('source',sources,MAXSRC,nsources)
        CALL  keyi('order', ampmax, 0 )
        CALL  keyi('order', phimax, 2 )
        CALL  keyl('close', antbase,.FALSE.)
        IF(antbase) THEN
           CALL bug('w','close=true may not work with breakpoints')
        ENDIF
        CALL  keyr('taver', taver(1),0.0)
        CALL  keyr('taver', taver(2),1000.0)
        IF (taver(1).LE.0.0) THEN
            CALL output(' No timeaveraging used')
        ELSE
            CALL output(' Time-averaging used ***')
        ENDIF
        taver(1) = taver(1) / (24.0*60.0)
        taver(2) = taver(2) / (24.0*60.0)
        CALL  keya('mode',  mode,' ')
        CALL  keyl('write', dowrite, .TRUE.)
        CALL  keya('units', units,'Jy/K')
        IF (units.NE.'Jy/K') THEN
            CALL bug('w','units=Jy/K should be used. Use at own risk')
        ENDIF
        CALL  keyfin
c
c  More checking on the user inputs before we're off
c
        CALL assertf(dataset,.TRUE.,'Calibration dataset gcal=')
        CALL assertl(phimax.LE.MAXORDER,
     *                  'Phase polynomial order too large')
        CALL assertl(ampmax.LE.MAXORDER,
     *                  'Amplitude polynomial order too large')
c
        CALL lcase(mode)
        IF (mode(1:1).EQ.'p') THEN
            stmpp = itoaf(phimax)
            CALL output('Only Phases with order='//
     *                  stmpp(1:len1(stmpp))//' done')
            pamode = -1
        ELSE IF (mode(1:1).EQ.'a') THEN
            stmpa = itoaf(ampmax)
            CALL output('Only Amplitudes with order='//
     *                  stmpa(1:len1(stmpa))//' done')
            pamode = 1
        ELSE
            pamode = 0
            IF (mode(1:1).NE.' ') CALL bug('i','Unknown mode')
c
            stmpa = itoaf(ampmax)
            stmpp = itoaf(phimax)
            CALL output('Fit to Amp (order='//
     *                  stmpa(1:len1(stmpa))//') and Phase (order='//
     *                  stmpp(1:len1(stmpp))//') done')
        ENDIF
c
c------ Get the raw data -----------------------------------------------c
c
        CALL readset( dataset )
        CALL readbrk( dataset )
        CALL getpoly( dataset )
        CALL setsflag( nsources,sources )
c-OLD-FLIPPER-METHOD
c	CALL phaseamp( nbl, basediff, .FALSE. )
c-NEW-FLIPPER-METHOD
        CALL flipper(taver(1),taver(2),basediff)
c	* Set scaling mode,
c	* note:actual scaling of data is explicitly done in getxy()
        scalmode = scalunit(units)
c
c	* (re)set the order of the polynomials [*****]
c	order(1) = ampmax
c	order(2) = phimax
c	order(3) = ampmax
c	order(4) = phimax
c	* consistency check to see if breakpoints all the same
        IF (antbase) CALL chkbrk()
c
c------ Loop through sidebands -----------------------------------------c
c
        DO s = 2, 4, 2
            s1 = (s+1)/2
            pcode=s2code(s)
            acode=s2code(s-1)
c
c           * bring phases nicely aligned, sort of we hope
c	    * this is now redundant code - 'basediff' is not used anymore
            DO b = 1, nbl
              delphi = basediff(s/2,b) * TWOPI
              DO i = 1, rcount
                rdata(s,b,i) = rdata(s,b,i) + delphi
              ENDDO
            ENDDO
c
c------ First the Phases  ---------------------------------------------c
c
            IF (pamode.LE.0) THEN
c             * Do the least squares fit in closure mode?
              IF( antbase ) THEN
                CALL lsqclose( s, phimax, x, y , basediff, taver)
c             * Or do it plain baseline based
              ELSE
                DO b = 1, nbl
                  bname=blname(base(b))
c                 * check each interval bounded by a breakpoint
                  DO bp=0,bcount(s1,b)
                    CALL setxy( bp, b, s, count, x, y, taver)
                    IF (count.GT.0) THEN
                      ipol = fitpoly(count,x,y,pcode,base(b),phimax)
c 		      *** Need poly(,,,) here ***
                      IF (ipol.GT.0) poly(0,ipol,s,b) =
     *                      poly(0,ipol,s,b) - basediff(s1,b)*TWOPI
c                     * reporting
                      sum0 = 0
                      sum1 = 0.0
                      sum2 = 0.0
                      DO i=1,count
                         d = y(i) - evalpoly(x(i),pcode,base(b),valid)
                         IF(valid.EQ.0) THEN
                            sum0 = sum0 + 1
                            sum1 = sum1 + d
                            sum2 = sum2 + d*d
                         ENDIF
                      ENDDO
c                      IF(sum0.GT.phimax+1) THEN
                      IF(sum0.GT.0) THEN
                         sum1 = sum1/sum0
                         sum2 = SQRT(sum2/sum0 - sum1*sum1)
                         WRITE(line, 200) bname(1:len1(bname)),
     *                                    acode(2:2),sum0,sum2
                         CALL output(line)
                      ENDIF
                    ENDIF
                  ENDDO
                ENDDO
              ENDIF
            ENDIF
c
 200  FORMAT('Baseline ',A,'[',A,']: ',I5,' points in phase fit.',
     *       ' Dispersion: ',F8.3,' Radians')
 300  FORMAT('Baseline ',A,'[',A,']: ',I5,' points. Gain: ',
     *       F10.5,' ',A)
c
c------ Now the amplitudes ---------------------------------------------c
c
            IF (pamode.GE.0) THEN
              DO b = 1, nbl
                bname = blname(base(b))
                DO bp=0,bcount(s1,b)
                  CALL setxy( bp, b, s-1, count, x, y , taver)
                  IF (count.GT.0) THEN
                    ipol = fitpoly(count,x,y,acode,base(b),ampmax)
                    IF (ampmax.EQ.0) THEN
                       WRITE(line,300) bname(1:len1(bname)),acode(2:2),
     *                              count, poly(0,ipol,s-1,b),units
                       CALL output(line)
                    ENDIF
                  ENDIF
                ENDDO
              ENDDO
            ENDIF
        ENDDO
c
c
c------ Write out the results, if so requested
c
      IF (dowrite) THEN
        CALL putpoly( dataset )
c
c------ Write history (NEEDS SOME WORK!!!)
c
c	CALL addhist(dataset,'CALFIT',
c     *      'phimax='//itoaf(phimax)//
c     -      ' ampmax='//itoaf(ampmax))
c
        CALL hopen(tno,dataset,'old',iostat)
        CALL hisopen(tno,'append')
        CALL hiswrite(tno,'CALFIT: '//PVERSION)
        CALL hisinput(tno,'CALFIT')
        CALL hisclose(tno)
        CALL hclose(tno)
      ELSE
        CALL bug('i',
     *        'Dryrun: Polynomial fits were not written to the dataset')
c
      ENDIF
      END
c***********************************************************************
c +
        SUBROUTINE lsqclose( s, pmax, x, y , basediff, taver)
c
c  lsqclose  -- sum up squares, assuming phase closure
c
c  Inputs:
c	s        -- 2 or 4 (must be a phase), depending on LSB/USB
c	pmax     -- polynomial order to fit
c	x        -- independent variable scratch space
c	y        -- dependent variable scratch space
c	basediff -- 2pi wraps needed for correction of zero'th order phase
c
c >>> ../../../src/inc/caldefs.h
c=======================================================================
c [caldefs.h]
c General include file that every calibration routine needs. It does not
c define (space for) data, but defines the constants that should be used
c in defining array-sizes.
c	18-nov-90 	added PI and TWOPI			PJT
c	 6-jul-91	V6.3 new MAXANTHC and MAXBASHC          pjt
c	26-sep-91       increased MAXUVPNT and MAXBREAK
c	 2-dec-91	V6.4 changed calbflux's dimensionality	PJT
c	19-apr-93	MAXSLOT from 20 to 36 for screwed mode-4  PJT
c			added MAXWIN from 8 (hardcoded) to 16
c	15-apr-94	MAXWIN is now in maxdim.h		PJT/mchw
c       19-dec-95       increase MAXANTHC from 6 to 9 !!!       pjt
c        9-Feb-99       increase MAXANTHC from 9 to 10 !       mchw
c       25-Apr-06       Increase MAXANTHC from 10 to 15 for CARMA ! mh
c=======================================================================
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c=======================================================================
c >>> ../../../src/inc/size.h
c=======================================================================
c	[size.h]
c
c  Include file used by calibration software
c
c  UL, RI, ULRI -- parameters which stand for LSB/USB and REAL/IMAG stuff
c
        integer UL, RI, ULRI
        parameter( UL = 2 )
        parameter( RI = 2 )
        parameter( ULRI = 4 )
c
c  SIZE -- lengths of reals/integers in number of bytes (IEEE/2s compl)
c
        integer SIZE
        parameter( SIZE = 4 )
c=======================================================================
c
c <<< ../../../src/inc/size.h
c=======================================================================
c
c  SVERSION -- current version of cal format (consistency checks)
c              Version number is 10*MAJOR+MINOR
c              Datasets with a MAJOR difference, cannot be read
c              anymore, MINOR differences are normally compatible
c
      INTEGER SVERSION
      PARAMETER ( SVERSION = 64)
c
c MAXANTHC -- special Hat Creek MAXANT, to keep it small for us....
c MAXBASHC -- derived from MAXANTHC = max. number of baselines
c
c Now changed the number from Hat-Creek (10) to Cedar-Flat (15)
c although the name is the same.
c
      INTEGER MAXANTHC, MAXBASHC
      PARAMETER (MAXANTHC=15, MAXBASHC=((MAXANTHC*(MAXANTHC-1))/2))
c
c MAXUVPNT -- maximum number of UV points in time per track
c
      INTEGER   MAXUVPNT
      PARAMETER(MAXUVPNT=2000)
c
c  MAXSRC -- maximum number of sources in multi-source databases
c
      INTEGER MAXSRC
      PARAMETER( MAXSRC = 32 )
c
c  MAXBREAK -- maximum number of breakpoints on an interval
c
      INTEGER MAXBREAK
      PARAMETER( MAXBREAK = 10 )
c
c  MAXPOLY -- maximum number of polynomials on an interval
c             is normally one more than number of breakpoints, right?
c
      INTEGER MAXPOLY
      PARAMETER( MAXPOLY = MAXBREAK+1 )
c
c  MAXSLOT -- maximum number of polynomial slots
c             Allows for simultaneous presence of gcal & pcal format
c             The first 4 for gcal, last 16 for pcal
c             CALIB makes use of slot 5 and 6 for phase diff&mean
      INTEGER MAXSLOT
c      PARAMETER( MAXSLOT = 20 )
      PARAMETER( MAXSLOT = 36 )
c
c  MAXORDER -- highest allowed polynomial order
c             Don't set too high, inversions are already needed in
c             double precision this way.
      INTEGER MAXORDER
      PARAMETER( MAXORDER = 5 )
c
c  MAXWIN -- maximum number of windows - normally equals MAXSPECT = 8
c	     but temporarely increased to 16 to deal with mode=4
c	     problems.
c     INTEGER MAXWIN
c     PARAMETER ( MAXWIN = 16 )
c
c=======================================================================
c
c <<< ../../../src/inc/caldefs.h
c
        INTEGER s, pmax
        REAL x(*), y(*), taver(2)
        INTEGER basediff(2,MAXBASHC)
c-
c >>> ../../../src/inc/calsubs.h
c=======================================================================
c	[calsubs.h]
c
c	Last update:    22-may-90	deleted pmode	PJT
c			18-dec-90	played with  /calmode/ [Arie/Peter]
c			 7-jul-91       new MAXBASHC parameter
c			11-dec-91	documentation formalized in doc format
c
c*calsubs_h -- header file for calibration
c:calibration, include
c&pjt
c+
c
c  time0 -- Julian time offset for all time measurements
        DOUBLE PRECISION time0
c
c  base(nbl) -- the baseline number and order
        INTEGER nbl, base(MAXBASHC)
c
c  version -- the read version of cal format
        INTEGER version
c
c  calsubs -- THE common block
        COMMON /calsubs/ time0, nbl, base, version
c
c
c  kludge before upgrading to the next CAL-SVERSION
        INTEGER scalmode
c
c	0:	(old system) fitting is done in K
c	1:	fitting is done in data space (K/Jy), i.e. the same is above
c      -1:	fitting is done in gain space (Jy/K)
c
c   For now, scalmode is set in calfit and calflag and such.
c   CAopen should get an extra stamp for this, see calio.h
c
        COMMON /calscal/scalmode
c--
c   Mode of phase/amp calibration in calflag/calib/...
c   [may be used in Arie's calib.for - but is internal for now]
c   nmode=1   amp
c	  2   phase
c         3   diff
c
c	INTEGER nmode
c	COMMON /calmode/nmode
c=======================================================================
c
c <<< ../../../src/inc/calsubs.h
c >>> ../../../src/inc/caldata.h
c=======================================================================
c       [caldata.h]
c       Header file for calibration data
c
c       Last update:    14-mar-90       added Sflag     PJT
c                       20-mar-90       srcindex now srcindex(MAXPNT)   PJT
c                        7-may-90       rdata(ULRI,MAXBASHC,MAXUVPNT)   PJT
c                        4-sep-90       baseline based calib stuff      PJT
c                       18-nov-90       more descriptive comments       PJT
c                       18-dec-90       played with rdata()             PJT
c                        7-jul-91       MAXBASE -> BAXMASHC             PJT
c                        2-dec-91       make fluxes time dependant, not
c                                       source dependant; =V6.4 fmt     PJT
c                       11-dec-91       documentation formalized in doc format
c*caldata_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c   The calibration format on disk is split out in memory over several
c   common blocks, each of them described in a separate include file.
c
c
c <general header> (see also calsubs.h)
c       double time0            time to which all times are offset (JD)
c       int    nbl              number of baselines
c       int    base(nbl)        nbl baselines (notation: A1*256+A2)
c       int    version          version of the calibration format
c
c <data> (this caldata.h include file)
c   rtime
c       float  rtime(rcount)             offset times to time0
c   rdata
c       float  rdata(ULRI,nbl,rcount)    correllation data
c   rflag
c       int    rflag(UL,nbl,rcount)      flagged bad? (0=false, 1=true)
c   sindex
c       int    sindex(rcount)            source index number
c   sname
c       char*16 sname(scount)             list of sources
c
c
c
c  Rxxxx -- the actual calibration data
c
      INTEGER   rcount
      REAL      rtime(MAXUVPNT)
      REAL      rdata(ULRI, MAXBASHC, MAXUVPNT)
      INTEGER   rflag(UL, MAXBASHC, MAXUVPNT)
c
c  Sxxxx -- index for multiple source calibration format
c           (plstuff was added later and is not quite needed??)
c
      INTEGER   scount
      CHARACTER sname(MAXSRC)*16
      INTEGER   sindex(MAXUVPNT), sflag(MAXUVPNT)
      REAL      plstuff(4,MAXSRC)
c
c  Bxxxx - list of break points
c
      REAL      btime(MAXBREAK, 2, MAXBASHC)
      INTEGER   bcount(2, MAXBASHC)
c
c  Xxxxx - odds and ends, should not really have to exist
c          are merely there for planets
c
      REAL      volts(MAXUVPNT)
c
c  baseline based calibrator fluxes, per time
c
      REAL      calbflux(MAXBASHC,MAXUVPNT)
c
c
c  the common blocks with all the data
c
      COMMON /caldatr/ rcount, scount, rtime, rdata, rflag,
     *                 sindex, btime, bcount, sflag, plstuff,volts
      COMMON /caldatc/ sname
      COMMON /caldatp/ calbflux
c--
c=======================================================================
c
c <<< ../../../src/inc/caldata.h
c >>> ../../../src/inc/calpoly.h
c=======================================================================
c	[calpoly.h]
c
c	last update: 26-apr-90	new poly's
c		     18-nov-90  descriptive comments
c		      7-jul-91  new MAXBASHC parameter
c*calpoly_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c
c  order -- current polynomial order for a slot
c
      INTEGER order(MAXSLOT)
c
c
c  pcount -- number of polynomials in a (slot,baseline) pair
c
      INTEGER pcount(MAXSLOT, MAXBASHC)
c
c
c  tvalid - validity of time interval
c		(min & max time-range; breakpoints; slotcode; baseline)
c
      REAL tvalid(2,MAXBREAK+1,MAXSLOT,MAXBASHC)
c
c  poly -- actual polynomial coefficients
c		(order; breakpoints, slotcode, baseline)
c
      REAL poly( 0:MAXORDER, MAXBREAK+1, MAXSLOT, MAXBASHC )
c
c
c  CALPOLY -- the common block
c
      COMMON /calpoly/ pcount, order, tvalid, poly
c--
c=======================================================================
c <<< ../../../src/inc/calpoly.h
c >>> ../../../src/inc/calfit.h
c=======================================================================
c	[calfit.h]
c*calfit_h -- calibration include file
c&pjt
c:calibration, include
c+
c  lsq()        matrix used for inversion
c  lsqb()       vector, used as right hand side AND solution
c  xsums()      temporary storage use to fill lsq()
c  ysums()      temporary storage use to fill lsq()
c
c
c  MAXLSQ -- the size of arrays and matrices in the least squares fit
c
      INTEGER MAXLSQ
      PARAMETER(MAXLSQ=(MAXANTHC*(MAXORDER+1)))
c
c  lsq, lsqb -- least squares matrix and rhs
c
      DOUBLE PRECISION lsq( MAXLSQ, MAXLSQ )
      DOUBLE PRECISION lsqb( MAXLSQ )
c
c  xsums, ysums -- sums-of-squares
c
      DOUBLE PRECISION xsums( 0 : 2 * MAXORDER )
      DOUBLE PRECISION ysums( 0 : MAXORDER )
c
      COMMON /lsq/ xsums, ysums, lsq, lsqb
c--
c=======================================================================
c
c <<< ../../../src/inc/calfit.h
c >>> ../../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../../src/inc/mirconst.h
c
        INTEGER b, p, nants, count, slot1, addpoly, ipol
        REAL    signs, valid(2,MAXBASHC), ppoly(0:MAXORDER)
        INTEGER bp, p1, p2, S1, S2
        INTEGER a, a1, a2
        INTEGER nlsq, Z
        CHARACTER pcode*4, s2code*4
c
c	a arith. statement function: Z(a,p)
        Z(a,p) = a + nants * p
c
        slot1 = (s+1)/2
        pcode = s2code(s)
c
c
c  find the number of antennas
c
        nants = 0
        CALL getants( 1, nants, a1, a2 )
        nlsq = nants * ( 1 + pmax )
c
c       * loop for each interval, defined by the breakpoints
        DO bp=0,bcount(slot1,1)
c
c  zero least squares matrix
c
          DO p1 = 1, nlsq
            lsqb(p1) = 0.0
            DO p2 = 1, nlsq
                lsq(p1,p2) = 0.0
            ENDDO
          ENDDO
c
c  loop through baselines
c
          DO b = 1, nbl
c
c  add this baseline to lsq [BUG when points deleted]
c
            CALL setxy( bp, b, s, count, x, y , taver)
c             * if nothing to process, go to next interval (bp)
            IF (count.EQ.0) THEN
                GOTO 100
            ENDIF
c	      * assume that x() was ordered
            valid(1,b) = x(1)
            valid(2,b) = x(count)
c
            CALL squares( count, x, y, pmax )
            CALL getants( b, nants, a1, a2 )
            signs = 1.0
            DO a = a1, a2, a2 - a1
              DO p1 = 0, pmax
                IF( a .eq. a2 ) signs = -1.0
                S1 = Z(a,p1)
                lsqb(S1) = lsqb(S1) + signs * ysums(p1)
                DO p2 = 0, pmax
                  S2 = Z(a1,p2)
                  lsq(S1,S2) = lsq(S1,S2) + signs * xsums(p1+p2)
                  S2 = Z(a2,p2)
                  lsq(S1,S2) = lsq(S1,S2) - signs * xsums(p1+p2)
                ENDDO
              ENDDO
            ENDDO
c
          ENDDO
c
c  zero out one antenna, for nant=3 this MUST be done, other
c  the matrix would be singular.
c  There are 0.5*N*(N-1) - N + 2 equations, and N-1 unknowns
c
          DO p1 = 0, pmax
            S1 = Z(1,p1)
            lsqb(S1) = 0.0
            DO p2 = 1, nlsq
              lsq(S1,p2) = 0.0
            ENDDO
            lsq(S1,S1) = 1.0
          ENDDO
c
          CALL solve( nlsq )
c
c  store baselines as antenna differences
c
          DO b = 1, nbl
            CALL getants( b, nants, a1, a2 )
            DO p=0,pmax
              ppoly(p) = lsqb( Z(a1,p) ) - lsqb( Z(a2,p) )
            ENDDO
c+debug
c	    IF(pmax.EQ.0) THEN
c	       write(*,*)'b=',b,' a1,2=',Z(a1,0),Z(a2,0),
c     *			'phi1,2= ',lsqb( Z(a1,0) ),lsqb( Z(a2,0) )
c	    ENDIF
c-debug
            ipol = addpoly(pcode,base(b),pmax,ppoly,valid(1,b))
c             * since count=0 never gets here, poly should always find a spot
            IF (ipol.EQ.0) CALL bug('w','lsqclose no ipol???')
c	    *** AGAIN NEED poly(,,,) here ***
            poly(0,ipol,s,b) = poly(0,ipol,s,b) -
     *              basediff(slot1,b)*TWOPI
            ENDDO
c         * if count==0, one can jump here for lack of structure in fortran
 100      CONTINUE
c       * end of looping through all breakpoints (bp)
        ENDDO
c
        RETURN
        END
c***********************************************************************
        SUBROUTINE chkbrk
c
c >>> ../../../src/inc/caldefs.h
c=======================================================================
c [caldefs.h]
c General include file that every calibration routine needs. It does not
c define (space for) data, but defines the constants that should be used
c in defining array-sizes.
c	18-nov-90 	added PI and TWOPI			PJT
c	 6-jul-91	V6.3 new MAXANTHC and MAXBASHC          pjt
c	26-sep-91       increased MAXUVPNT and MAXBREAK
c	 2-dec-91	V6.4 changed calbflux's dimensionality	PJT
c	19-apr-93	MAXSLOT from 20 to 36 for screwed mode-4  PJT
c			added MAXWIN from 8 (hardcoded) to 16
c	15-apr-94	MAXWIN is now in maxdim.h		PJT/mchw
c       19-dec-95       increase MAXANTHC from 6 to 9 !!!       pjt
c        9-Feb-99       increase MAXANTHC from 9 to 10 !       mchw
c       25-Apr-06       Increase MAXANTHC from 10 to 15 for CARMA ! mh
c=======================================================================
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c=======================================================================
c >>> ../../../src/inc/size.h
c=======================================================================
c	[size.h]
c
c  Include file used by calibration software
c
c  UL, RI, ULRI -- parameters which stand for LSB/USB and REAL/IMAG stuff
c
        integer UL, RI, ULRI
        parameter( UL = 2 )
        parameter( RI = 2 )
        parameter( ULRI = 4 )
c
c  SIZE -- lengths of reals/integers in number of bytes (IEEE/2s compl)
c
        integer SIZE
        parameter( SIZE = 4 )
c=======================================================================
c
c <<< ../../../src/inc/size.h
c=======================================================================
c
c  SVERSION -- current version of cal format (consistency checks)
c              Version number is 10*MAJOR+MINOR
c              Datasets with a MAJOR difference, cannot be read
c              anymore, MINOR differences are normally compatible
c
      INTEGER SVERSION
      PARAMETER ( SVERSION = 64)
c
c MAXANTHC -- special Hat Creek MAXANT, to keep it small for us....
c MAXBASHC -- derived from MAXANTHC = max. number of baselines
c
c Now changed the number from Hat-Creek (10) to Cedar-Flat (15)
c although the name is the same.
c
      INTEGER MAXANTHC, MAXBASHC
      PARAMETER (MAXANTHC=15, MAXBASHC=((MAXANTHC*(MAXANTHC-1))/2))
c
c MAXUVPNT -- maximum number of UV points in time per track
c
      INTEGER   MAXUVPNT
      PARAMETER(MAXUVPNT=2000)
c
c  MAXSRC -- maximum number of sources in multi-source databases
c
      INTEGER MAXSRC
      PARAMETER( MAXSRC = 32 )
c
c  MAXBREAK -- maximum number of breakpoints on an interval
c
      INTEGER MAXBREAK
      PARAMETER( MAXBREAK = 10 )
c
c  MAXPOLY -- maximum number of polynomials on an interval
c             is normally one more than number of breakpoints, right?
c
      INTEGER MAXPOLY
      PARAMETER( MAXPOLY = MAXBREAK+1 )
c
c  MAXSLOT -- maximum number of polynomial slots
c             Allows for simultaneous presence of gcal & pcal format
c             The first 4 for gcal, last 16 for pcal
c             CALIB makes use of slot 5 and 6 for phase diff&mean
      INTEGER MAXSLOT
c      PARAMETER( MAXSLOT = 20 )
      PARAMETER( MAXSLOT = 36 )
c
c  MAXORDER -- highest allowed polynomial order
c             Don't set too high, inversions are already needed in
c             double precision this way.
      INTEGER MAXORDER
      PARAMETER( MAXORDER = 5 )
c
c  MAXWIN -- maximum number of windows - normally equals MAXSPECT = 8
c	     but temporarely increased to 16 to deal with mode=4
c	     problems.
c     INTEGER MAXWIN
c     PARAMETER ( MAXWIN = 16 )
c
c=======================================================================
c
c <<< ../../../src/inc/caldefs.h
c >>> ../../../src/inc/caldata.h
c=======================================================================
c       [caldata.h]
c       Header file for calibration data
c
c       Last update:    14-mar-90       added Sflag     PJT
c                       20-mar-90       srcindex now srcindex(MAXPNT)   PJT
c                        7-may-90       rdata(ULRI,MAXBASHC,MAXUVPNT)   PJT
c                        4-sep-90       baseline based calib stuff      PJT
c                       18-nov-90       more descriptive comments       PJT
c                       18-dec-90       played with rdata()             PJT
c                        7-jul-91       MAXBASE -> BAXMASHC             PJT
c                        2-dec-91       make fluxes time dependant, not
c                                       source dependant; =V6.4 fmt     PJT
c                       11-dec-91       documentation formalized in doc format
c*caldata_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c   The calibration format on disk is split out in memory over several
c   common blocks, each of them described in a separate include file.
c
c
c <general header> (see also calsubs.h)
c       double time0            time to which all times are offset (JD)
c       int    nbl              number of baselines
c       int    base(nbl)        nbl baselines (notation: A1*256+A2)
c       int    version          version of the calibration format
c
c <data> (this caldata.h include file)
c   rtime
c       float  rtime(rcount)             offset times to time0
c   rdata
c       float  rdata(ULRI,nbl,rcount)    correllation data
c   rflag
c       int    rflag(UL,nbl,rcount)      flagged bad? (0=false, 1=true)
c   sindex
c       int    sindex(rcount)            source index number
c   sname
c       char*16 sname(scount)             list of sources
c
c
c
c  Rxxxx -- the actual calibration data
c
      INTEGER   rcount
      REAL      rtime(MAXUVPNT)
      REAL      rdata(ULRI, MAXBASHC, MAXUVPNT)
      INTEGER   rflag(UL, MAXBASHC, MAXUVPNT)
c
c  Sxxxx -- index for multiple source calibration format
c           (plstuff was added later and is not quite needed??)
c
      INTEGER   scount
      CHARACTER sname(MAXSRC)*16
      INTEGER   sindex(MAXUVPNT), sflag(MAXUVPNT)
      REAL      plstuff(4,MAXSRC)
c
c  Bxxxx - list of break points
c
      REAL      btime(MAXBREAK, 2, MAXBASHC)
      INTEGER   bcount(2, MAXBASHC)
c
c  Xxxxx - odds and ends, should not really have to exist
c          are merely there for planets
c
      REAL      volts(MAXUVPNT)
c
c  baseline based calibrator fluxes, per time
c
      REAL      calbflux(MAXBASHC,MAXUVPNT)
c
c
c  the common blocks with all the data
c
      COMMON /caldatr/ rcount, scount, rtime, rdata, rflag,
     *                 sindex, btime, bcount, sflag, plstuff,volts
      COMMON /caldatc/ sname
      COMMON /caldatp/ calbflux
c--
c=======================================================================
c
c <<< ../../../src/inc/caldata.h
c >>> ../../../src/inc/calsubs.h
c=======================================================================
c	[calsubs.h]
c
c	Last update:    22-may-90	deleted pmode	PJT
c			18-dec-90	played with  /calmode/ [Arie/Peter]
c			 7-jul-91       new MAXBASHC parameter
c			11-dec-91	documentation formalized in doc format
c
c*calsubs_h -- header file for calibration
c:calibration, include
c&pjt
c+
c
c  time0 -- Julian time offset for all time measurements
        DOUBLE PRECISION time0
c
c  base(nbl) -- the baseline number and order
        INTEGER nbl, base(MAXBASHC)
c
c  version -- the read version of cal format
        INTEGER version
c
c  calsubs -- THE common block
        COMMON /calsubs/ time0, nbl, base, version
c
c
c  kludge before upgrading to the next CAL-SVERSION
        INTEGER scalmode
c
c	0:	(old system) fitting is done in K
c	1:	fitting is done in data space (K/Jy), i.e. the same is above
c      -1:	fitting is done in gain space (Jy/K)
c
c   For now, scalmode is set in calfit and calflag and such.
c   CAopen should get an extra stamp for this, see calio.h
c
        COMMON /calscal/scalmode
c--
c   Mode of phase/amp calibration in calflag/calib/...
c   [may be used in Arie's calib.for - but is internal for now]
c   nmode=1   amp
c	  2   phase
c         3   diff
c
c	INTEGER nmode
c	COMMON /calmode/nmode
c=======================================================================
c
c <<< ../../../src/inc/calsubs.h
c
        INTEGER s1, b, bp
c
c
c  consistency check to see if breakpoints were all the same...
c  and force them to be so anyhow - equate them to first baseline
c
        DO s1=1,2
          DO b=2,nbl
            IF (bcount(s1,b).NE.bcount(s1,1)) THEN
                write (*,*) 's1 (1,b) ',s1,1,b,' = ',
     *                  bcount(s1,b),bcount(s1,1)
                CALL bug('f',
     *         'No equal number of breakpoints accross baselines')
            ENDIF
            DO bp=1,bcount(s1,b)
                IF (btime(bp,s1,b).NE.btime(bp,s1,1)) CALL bug('f',
     *              'Times of breakpoint not the same')
            ENDDO
          ENDDO
        ENDDO
c
        END
