        PROGRAM callist
c
c	Callist -- list info from a calibration set
c
c--
c	1-feb-90    V1.0  PJT   original version
c	20-mar-90   V1.1  PJT   adapted for new cal format
c	27-mar-90         PJT   cosmetic
c	 9-apr-90   PJT         print out /calsubs/ common block
c	27-apr-90   PJT		poly and break
c	 5-may-90   PJT         more string space for output
c	 4-sep-90   PJT 	read 'cdata' for calibrator fluxes
c	29-nov-90   PJT		findbase
c	 1-dec-91   PJT		more ANSI - removed some list directed's
c       11-dec-91   PJT         added time= for poly-check
c	31-jan-91   PJT         accumulated various A-based debugging
c      27-mar-92    assertl now
c       07-apr-92   MJS         elim unused vars -> elim compiler warn.
c	23apr93	    pjt		wider format for code(1:4)
c
c------ Inline doc (retrieved with doc to a .doc file) ----------------c
c
c= callist - List items from calibration data set
c& pjt
c: calibration
c+
c	CALLIST is a MIRIAD task that lists some items in a calibration set.
c@ gcal
c	Name of the calibration set. No default.
c@ verbose
c	When set to 'true' is displays information per integration time.
c	Default: 'false', which only displays when source name changes.
c@ poly
c	When set to 'true' is displays information of all polynomials
c	Default: false.
c@ break
c	When set to 'true' is displays information of all breakpoints
c	Default: false.
c@ code
c	Slot code for band/phase/amp to print. Must be 3 or 4
c	characters. If empty, correllations are not printed.
c	First character is A(mp) or P(phase).
c	Second character is U(pper) or L(ower) sideband.
c	Third character is W(ide) band or any of 1..8 for passband.
c	Default: Empty, not used.
c@ base
c	Baseline, combined with slot code specified before, to print.
c	Valid entries must be of the format A1-A2, where A1 and A2 are
c       the two antennas.
c	Default: not used.
c@ time
c	Relative time with respect to the offset time0 in the gcal file
c	for which the complex gains/(phases and amplitude) are shown
c	for each baseline and sideband. Some test calculations
c	for anntenna based conversions are also shown.
c	Default: 0.0, meaning, not used.
c	If a second parameter is used, you need to specify if you want to
c	see Complex(Re/Im) ['c'] or else Amp/Phase pairs. Default: 'c'
c@ log
c	Output device. (default is standard user output)
c------ Declarations ---------------------------------------------------c
c >>> ../../../src/inc/caldefs.h
c=======================================================================
c [caldefs.h]
c General include file that every calibration routine needs. It does not
c define (space for) data, but defines the constants that should be used
c in defining array-sizes.
c	18-nov-90 	added PI and TWOPI			PJT
c	 6-jul-91	V6.3 new MAXANTHC and MAXBASHC          pjt
c	26-sep-91       increased MAXUVPNT and MAXBREAK
c	 2-dec-91	V6.4 changed calbflux's dimensionality	PJT
c	19-apr-93	MAXSLOT from 20 to 36 for screwed mode-4  PJT
c			added MAXWIN from 8 (hardcoded) to 16
c	15-apr-94	MAXWIN is now in maxdim.h		PJT/mchw
c       19-dec-95       increase MAXANTHC from 6 to 9 !!!       pjt
c        9-Feb-99       increase MAXANTHC from 9 to 10 !       mchw
c       25-Apr-06       Increase MAXANTHC from 10 to 15 for CARMA ! mh
c=======================================================================
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c=======================================================================
c >>> ../../../src/inc/size.h
c=======================================================================
c	[size.h]
c
c  Include file used by calibration software
c
c  UL, RI, ULRI -- parameters which stand for LSB/USB and REAL/IMAG stuff
c
        integer UL, RI, ULRI
        parameter( UL = 2 )
        parameter( RI = 2 )
        parameter( ULRI = 4 )
c
c  SIZE -- lengths of reals/integers in number of bytes (IEEE/2s compl)
c
        integer SIZE
        parameter( SIZE = 4 )
c=======================================================================
c
c <<< ../../../src/inc/size.h
c=======================================================================
c
c  SVERSION -- current version of cal format (consistency checks)
c              Version number is 10*MAJOR+MINOR
c              Datasets with a MAJOR difference, cannot be read
c              anymore, MINOR differences are normally compatible
c
      INTEGER SVERSION
      PARAMETER ( SVERSION = 64)
c
c MAXANTHC -- special Hat Creek MAXANT, to keep it small for us....
c MAXBASHC -- derived from MAXANTHC = max. number of baselines
c
c Now changed the number from Hat-Creek (10) to Cedar-Flat (15)
c although the name is the same.
c
      INTEGER MAXANTHC, MAXBASHC
      PARAMETER (MAXANTHC=15, MAXBASHC=((MAXANTHC*(MAXANTHC-1))/2))
c
c MAXUVPNT -- maximum number of UV points in time per track
c
      INTEGER   MAXUVPNT
      PARAMETER(MAXUVPNT=2000)
c
c  MAXSRC -- maximum number of sources in multi-source databases
c
      INTEGER MAXSRC
      PARAMETER( MAXSRC = 32 )
c
c  MAXBREAK -- maximum number of breakpoints on an interval
c
      INTEGER MAXBREAK
      PARAMETER( MAXBREAK = 10 )
c
c  MAXPOLY -- maximum number of polynomials on an interval
c             is normally one more than number of breakpoints, right?
c
      INTEGER MAXPOLY
      PARAMETER( MAXPOLY = MAXBREAK+1 )
c
c  MAXSLOT -- maximum number of polynomial slots
c             Allows for simultaneous presence of gcal & pcal format
c             The first 4 for gcal, last 16 for pcal
c             CALIB makes use of slot 5 and 6 for phase diff&mean
      INTEGER MAXSLOT
c      PARAMETER( MAXSLOT = 20 )
      PARAMETER( MAXSLOT = 36 )
c
c  MAXORDER -- highest allowed polynomial order
c             Don't set too high, inversions are already needed in
c             double precision this way.
      INTEGER MAXORDER
      PARAMETER( MAXORDER = 5 )
c
c  MAXWIN -- maximum number of windows - normally equals MAXSPECT = 8
c	     but temporarely increased to 16 to deal with mode=4
c	     problems.
c     INTEGER MAXWIN
c     PARAMETER ( MAXWIN = 16 )
c
c=======================================================================
c
c <<< ../../../src/inc/caldefs.h
c >>> ../../../src/inc/calsubs.h
c=======================================================================
c	[calsubs.h]
c
c	Last update:    22-may-90	deleted pmode	PJT
c			18-dec-90	played with  /calmode/ [Arie/Peter]
c			 7-jul-91       new MAXBASHC parameter
c			11-dec-91	documentation formalized in doc format
c
c*calsubs_h -- header file for calibration
c:calibration, include
c&pjt
c+
c
c  time0 -- Julian time offset for all time measurements
        DOUBLE PRECISION time0
c
c  base(nbl) -- the baseline number and order
        INTEGER nbl, base(MAXBASHC)
c
c  version -- the read version of cal format
        INTEGER version
c
c  calsubs -- THE common block
        COMMON /calsubs/ time0, nbl, base, version
c
c
c  kludge before upgrading to the next CAL-SVERSION
        INTEGER scalmode
c
c	0:	(old system) fitting is done in K
c	1:	fitting is done in data space (K/Jy), i.e. the same is above
c      -1:	fitting is done in gain space (Jy/K)
c
c   For now, scalmode is set in calfit and calflag and such.
c   CAopen should get an extra stamp for this, see calio.h
c
        COMMON /calscal/scalmode
c--
c   Mode of phase/amp calibration in calflag/calib/...
c   [may be used in Arie's calib.for - but is internal for now]
c   nmode=1   amp
c	  2   phase
c         3   diff
c
c	INTEGER nmode
c	COMMON /calmode/nmode
c=======================================================================
c
c <<< ../../../src/inc/calsubs.h
c >>> ../../../src/inc/caldata.h
c=======================================================================
c       [caldata.h]
c       Header file for calibration data
c
c       Last update:    14-mar-90       added Sflag     PJT
c                       20-mar-90       srcindex now srcindex(MAXPNT)   PJT
c                        7-may-90       rdata(ULRI,MAXBASHC,MAXUVPNT)   PJT
c                        4-sep-90       baseline based calib stuff      PJT
c                       18-nov-90       more descriptive comments       PJT
c                       18-dec-90       played with rdata()             PJT
c                        7-jul-91       MAXBASE -> BAXMASHC             PJT
c                        2-dec-91       make fluxes time dependant, not
c                                       source dependant; =V6.4 fmt     PJT
c                       11-dec-91       documentation formalized in doc format
c*caldata_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c   The calibration format on disk is split out in memory over several
c   common blocks, each of them described in a separate include file.
c
c
c <general header> (see also calsubs.h)
c       double time0            time to which all times are offset (JD)
c       int    nbl              number of baselines
c       int    base(nbl)        nbl baselines (notation: A1*256+A2)
c       int    version          version of the calibration format
c
c <data> (this caldata.h include file)
c   rtime
c       float  rtime(rcount)             offset times to time0
c   rdata
c       float  rdata(ULRI,nbl,rcount)    correllation data
c   rflag
c       int    rflag(UL,nbl,rcount)      flagged bad? (0=false, 1=true)
c   sindex
c       int    sindex(rcount)            source index number
c   sname
c       char*16 sname(scount)             list of sources
c
c
c
c  Rxxxx -- the actual calibration data
c
      INTEGER   rcount
      REAL      rtime(MAXUVPNT)
      REAL      rdata(ULRI, MAXBASHC, MAXUVPNT)
      INTEGER   rflag(UL, MAXBASHC, MAXUVPNT)
c
c  Sxxxx -- index for multiple source calibration format
c           (plstuff was added later and is not quite needed??)
c
      INTEGER   scount
      CHARACTER sname(MAXSRC)*16
      INTEGER   sindex(MAXUVPNT), sflag(MAXUVPNT)
      REAL      plstuff(4,MAXSRC)
c
c  Bxxxx - list of break points
c
      REAL      btime(MAXBREAK, 2, MAXBASHC)
      INTEGER   bcount(2, MAXBASHC)
c
c  Xxxxx - odds and ends, should not really have to exist
c          are merely there for planets
c
      REAL      volts(MAXUVPNT)
c
c  baseline based calibrator fluxes, per time
c
      REAL      calbflux(MAXBASHC,MAXUVPNT)
c
c
c  the common blocks with all the data
c
      COMMON /caldatr/ rcount, scount, rtime, rdata, rflag,
     *                 sindex, btime, bcount, sflag, plstuff,volts
      COMMON /caldatc/ sname
      COMMON /caldatp/ calbflux
c--
c=======================================================================
c
c <<< ../../../src/inc/caldata.h
c >>> ../../../src/inc/calpoly.h
c=======================================================================
c	[calpoly.h]
c
c	last update: 26-apr-90	new poly's
c		     18-nov-90  descriptive comments
c		      7-jul-91  new MAXBASHC parameter
c*calpoly_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c
c  order -- current polynomial order for a slot
c
      INTEGER order(MAXSLOT)
c
c
c  pcount -- number of polynomials in a (slot,baseline) pair
c
      INTEGER pcount(MAXSLOT, MAXBASHC)
c
c
c  tvalid - validity of time interval
c		(min & max time-range; breakpoints; slotcode; baseline)
c
      REAL tvalid(2,MAXBREAK+1,MAXSLOT,MAXBASHC)
c
c  poly -- actual polynomial coefficients
c		(order; breakpoints, slotcode, baseline)
c
      REAL poly( 0:MAXORDER, MAXBREAK+1, MAXSLOT, MAXBASHC )
c
c
c  CALPOLY -- the common block
c
      COMMON /calpoly/ pcount, order, tvalid, poly
c--
c=======================================================================
c <<< ../../../src/inc/calpoly.h
c
        CHARACTER ctime0*40, dataset*100, logfile*100
        CHARACTER s2code*4, code*4, dispcode*4, msg*132, bname*8
        CHARACTER bbb1*8,bbb2*8,bbb3*8, cmode*10
        LOGICAL   qverb, qpoly, qbreak, more, ok, docompl,phonly,donorm
        INTEGER   i, j, k, n, kold, p, p1, b, bl(2), idx, valid
        INTEGER   basediff(2,MAXBASHC), b1(MAXBASHC), b2(MAXBASHC)
        INTEGER   code2s, len1, findbase, a1, a2, nants
        COMPLEX   gains(MAXBASHC,2)
        COMPLEX   sumvm(MAXBASHC)
        COMPLEX   sum(MAXANTHC), gant(MAXANTHC,2)
        REAL      sumvv(MAXBASHC), sum2(MAXANTHC), flux
        REAL      time, amp(2), phi(2), evalpoly
        CHARACTER blname*8
c
c------ Address the crowd-----------------------------------------------c
        CALL output('CALLIST: 19-apr-93')
c
c------ Get the inputs -------------------------------------------------c
c
        CALL my
        CALL keyini
        CALL keyf( 'gcal', dataset, ' ' )
        CALL keyl( 'verbose', qverb, .FALSE.)
        CALL keyl( 'poly',    qpoly, .FALSE.)
        CALL keyl( 'break',   qbreak,.FALSE.)
        CALL keya( 'code',dispcode,' ')
        CALL keya( 'base',bname, ' ')
        CALL keya( 'log',logfile,' ')
        CALL keyr( 'time',time,0.0)
        CALL keya( 'time',cmode,'c')
        CALL keyl( 'phonly',phonly,.TRUE.)
        CALL keyl( 'donorm',donorm,.TRUE.)
        CALL keyfin
c
        CALL my
        CALL assertl(dataset.NE.' ','No calibration data set given')
c		cmode=Real/Imaginary/Complex or
c	              Amplitude/Phase
        CALL lcase(cmode)
        docompl = cmode(1:1).EQ.'c' .OR. cmode(1:1).EQ.'r' .OR.
     *              cmode(1:1).EQ.'i'
        ok = bname.NE.' '
        IF(ok) THEN
          idx = index(bname,'-')
          ok = idx.GE.2
          IF(ok) CALL atoif(bname(1:idx-1),bl(1),ok)
          ok = ok .AND. (idx+1).LE.len1(bname)
          IF(ok) CALL atoif(bname(idx+1:len1(bname)),bl(2),ok)
          IF(.NOT.ok) CALL bug('f','Parsing error in base='//bname)
        ELSE
           bl(1) = 0
           bl(2) = 0
        ENDIF
        IF(bl(1).GT.bl(2)) THEN
           i = bl(1)
           bl(1) = bl(2)
           bl(2) = i
        ENDIF
        write(*,*) 'DEBUG: Bname=',bname,' bl=',bl
c
c	* open logfile for output
        CALL logopen(logfile,' ')
c
c------ Get all the data ----------------------------------------------c
        CALL my
        CALL readset(dataset)
        CALL getpoly(dataset)
        CALL readbrk(dataset)
        CALL my
c           init some variables from this primary data
        DO i=1,nbl
            b1(i) = base(i)/256
            b2(i) = MOD(base(i),256)
        ENDDO
        flux = calbflux(1,1)
c
        CALL julday(time0,'H',ctime0)
        CALL output('Start time: '//ctime0)
        WRITE (msg,'(A,I5)') 'database version ', version
        CALL logwrite(msg,more)
c	WRITE (msg,*) 'nbl, baselines(1:nbl) = ',nbl, (base(i),i=1,nbl)
c	CALL logwrite(msg,more)
        WRITE (msg,'(A,I5)') 'Number of timeslots: Rcount = ',rcount
        CALL logwrite(msg,more)
        WRITE (msg,'(A,I5)') 'Number of sources:   Scount = ',scount
        CALL logwrite(msg,more)
        WRITE (msg,'(A,F5.2)') 'Assumed flux = ',flux
        CALL logwrite(msg,more)
c
        k=1
        kold = -1
        bbb1=blname(base(1))
        bbb2=blname(base(2))
        bbb3=blname(base(3))
        WRITE(msg,'(A)')'Dtime     Source      '//bbb1//bbb2//bbb3
        CALL logwrite(msg,more)
        DO i=1,rcount
            k = sindex(i)
            WRITE(msg,'(F9.6,1X,A,15(1X,F7.2))') rtime(i),sname(k),
     *                          (calbflux(j,i),j=1,nbl)
            IF (k.NE.kold) THEN
                kold = k
                CALL logwrite(msg,more)
            ELSE
                IF (qverb) CALL logwrite(msg,more)
            ENDIF
        ENDDO
c
        CALL my
c
c------- Display data ------ -------------------------------------------c
c
        IF (dispcode .NE. ' ') THEN
c            CALL phaseamp(nbl,basediff,.FALSE.)
            CALL flipper(0.0,1000.0,basediff)
            p = code2s(dispcode)
            p1 = (p+1)/2
            IF (bl(1).EQ.0  .OR. bl(2).EQ.0 .OR. bl(1).EQ.bl(2)) THEN
                CALL bug('f','Illegal baseline base=n,m')
            ELSE
                IF (bl(1).LT.bl(2)) THEN
                    b = findbase(bl(1)*256+bl(2),base,nbl)
                ELSE
                    b = findbase(bl(2)*256+bl(1),base,nbl)
                ENDIF
            ENDIF
            WRITE(msg,'(A)') '   time      data    flag'
            DO i=1,rcount
                WRITE(msg,
     *                '(F9.4,1X,G10.4,1X,I2)') rtime(i),
     *                  rdata(p,b,i),rflag(p1,b,i)
                CALL logwrite(msg,more)
            ENDDO
        ENDIF
c
c------- Display polynomials -------------------------------------------c
c
        IF (qpoly) THEN
            DO b=1,nbl
                DO p=1,MAXSLOT
                    n = pcount(p,b)
                    k = order(p)
                    IF (n.GT.0) THEN
                      code = s2code(p)
                      WRITE (msg,'(A,A4,1X,I2,I2,I2)')
     *                          'code,b,order,npols=',code,b,k,n
                      CALL logwrite(msg,more)
                      DO j=1,n
cNOCRAY
                        WRITE (msg,*) '  tlo,thi,pol=',
     *                    tvalid(1,j,p,b), tvalid(2,j,p,b),
     *                    (poly(i,j,p,b),i=0,k)
                        CALL logwrite(msg,more)
                      ENDDO
                    ENDIF
                ENDDO
            ENDDO
        ENDIF
c
c------- Display breakpoints  -------------------------------------------c
c
        IF (qbreak) THEN
            DO b=1,nbl
                DO p1=1,2
                    n = bcount(p1,b)
                    IF (n.GT.0) THEN
                        WRITE (msg,*) 'b,p1,break=',b,p1,n,
     *                  (btime(i,p1,b),i=1,n)
                        CALL logwrite(msg,more)
                    ENDIF
                ENDDO
            ENDDO
        ENDIF
c
c
c--- Display gains at certain time, and some more complex arithmetic --c
c    to test some antenna based gain conversions/solutions ala selfcal
c
        IF(time.NE.0.0)THEN
            nants = 0
            CALL getants(1,nants,a1,a2)
c-----------------------------------------------------------------------
            WRITE(*,*)
     *        'Baseline              LSB                           USB'
            CALL my
            DO b=1,nbl
                DO p1=1,2
                    amp(p1)=evalpoly(time,s2code(p1*2-1),base(b),valid)
                    phi(p1)=evalpoly(time,s2code(p1*2),  base(b),valid)
                    gains(b,p1) = CMPLX(amp(p1)*COS(phi(p1)),
     *                                  amp(p1)*SIN(phi(p1)))
                ENDDO
                bbb1 = blname(base(b))
                IF(docompl) THEN
                    WRITE(*,*) bbb1//': ',
     *                  gains(b,1),' ; ',gains(b,2),' Re/Im'
                ELSE
                    WRITE(*,*) bbb1//': ',
     *                  amp(1),phi(1),' ; ',amp(2),phi(2),' Amp/Phi'
                ENDIF
        ENDDO
c-----------------------------------------------------------------------
            CALL my
            WRITE(*,*) 'PhaSol & AmPhaSol routine conversions:'
            WRITE(*,*)
     *        'Antenna               LSB                           USB'
            DO p1=1,2
                IF(phonly) THEN
                    CALL phasol(nbl,nants,sum,gains(1,p1),b1,b2,
     *                      gant(1,p1),ok)
                ELSE
                    DO b=1,nbl
                        sumvm(b) = 1/gains(b,p1)*flux*flux
                        sumvv(b) = sumvm(b)*CONJG(sumvm(b)) /
     *                              (flux*flux)
                    ENDDO
                    CALL amphasol(nbl,nants,sum,sum2,sumvm,sumvv,b1,b2,
     *                      gant(1,p1),ok)
                ENDIF
                IF(.NOT.ok)CALL bug('f','bad PHASOL')
                IF(donorm)THEN
                  DO i=nants,1,-1
                    gant(i,p1) = gant(i,p1)/gant(1,p1)*ABS(gant(1,p1))
                  ENDDO
                ENDIF
            ENDDO
            CALL my
            DO i=1,nants
               IF(docompl)THEN
                  WRITE(*,*) i,' ',
     *              gant(i,1),' ; ',gant(i,2),' Re/Im'
               ELSE
                  WRITE(*,*) i,' ',
     *              ABS(gant(i,1)),
     *              ATAN2(AIMAG(gant(i,1)),REAL((gant(i,1)))),
     *              ' ; ',
     *              ABS(gant(i,2)),
     *              ATAN2(AIMAG(gant(i,2)),REAL((gant(i,2)))),
     *              ' Amp/Phi'
               ENDIF
            ENDDO
            CALL my
        ENDIF
        END
c***********************************************************************
        SUBROUTINE my
        END
c***********************************************************************
c	The rest of this code is unmodified code from selfcal.for
c	phasol() and amphasol() to do the B-A conversions of the
c	gains in the form of an A-based solution.
c************************************************************************
        subroutine phasol(Nblines,NAnts,Sum,SumVM,b1,b2,Gain,Convrg)
c
        logical Convrg
        integer Nblines,Nants
        integer b1(NBlines),b2(NBlines)
        complex SumVM(Nblines),Gain(NAnts),Sum(NAnts)
c
c  Solve for the phase corrections which minimise the error. This uses
c  a nonlinear Jacobi iteration, as suggested by Fred Schwab in "Adaptive
c  calibration of radio interferomter data", SPIE Vol. 231, 1980
c  International Optical Computing Conference (pp 18-24). The damping
c  heuristics are copied from AIPS ASCAL.
c
c  Input:
c    NBlines	Number of baselines.
c    Nants	Number of antennae.
c    b1,b2	This gives the antennae pair for each baseline.
c    SumVM	Sum of Model*conjg(Vis)
c  Scratch:
c    Sum
c  Output:
c    Convrg	If .true., then the algorithm converged.
c    Gain	The antenna gain solution.
c------------------------------------------------------------------------
        integer MaxIter
        real Epsi
        parameter(MaxIter=100,Epsi=1.e-5)
c
        real Factor,Change
        complex Temp
        integer Niter,i
c
c  Initialise.
c
        do i=1,NAnts
          Gain(i) = (1.,0.)
          Sum(i) = (0.,0.)
        enddo
c
        Factor = 0.8
        if(Nants.le.6)Factor = 0.5
c
c  Iterate.
c
        Convrg = .false.
        Niter = 0
        do while(.not.Convrg.and.Niter.lt.MaxIter)
          Niter = Niter + 1
c
c  Sum the contributions over the baselines. Note that the following
c  loop has a dependency.
c
          do i=1,nblines
            Sum(b1(i)) = Sum(b1(i)) + Gain(b2(i)) *       SumVM(i)
            Sum(b2(i)) = Sum(b2(i)) + Gain(b1(i)) * conjg(SumVM(i))
          enddo
c
c  Update the gains. The following will be flagged as a short loop
c  on the Cray, if we assume we have fewer than 32 antennae. Hopefully
c  this will vectorise. For "typical" cases, the absolute value function
c  in the next loop takes up about 30-40% of the run time of this routine
c  on a VAX.
c
          Change = 0
          do i=1,nants
            Temp = ( Sum(i)/abs(Sum(i)) )
            Temp = Gain(i) + Factor * ( Temp - Gain(i) )
            Temp = Temp/abs(Temp)
            Change = Change + real(Gain(i)-Temp)**2
     *                      + aimag(Gain(i)-Temp)**2
            Gain(i) = Temp
            Sum(i) = (0.,0.)
          enddo
          Convrg = Change/nants .lt. Epsi
        enddo
        end
c************************************************************************
        subroutine amphasol(NBlines,NAnts,Sum,Sum2,SumVM,SumVV,
     *                                          b1,b2,gain,convrg)
c
        logical Convrg
        integer NBlines,NAnts
        integer B1(NBlines),B2(NBlines)
        complex SumVM(NBlines),Gain(NAnts),Sum(NAnts)
        real SumVV(NBlines),Sum2(NAnts)
c
c  Solve for the amplitudes and phase corrections which minimise
c  error. Algorithm is again Schwab's Jacobi iteration. The damping
c  heuristics are copied from AIPS ASCAL or dreamed up by me (as was the
c  initial gain estimate).
c
c  Input:
c    NBlines	Number of baselines.
c    Nants	Number of antennae.
c    b1,b2	This gives the antennae pair for each baseline.
c    SumVM	Sum of Model*conjg(Vis), for each baseline.
c    SumVV	Sum of Vis*conjg(Vis), for each baseline.
c  Scratch:
c    Sum
c  Output:
c    Convrg	If .true., then the algorithm converged.
c    Gain	The antenna gain solution.
c
c------------------------------------------------------------------------
        integer maxiter
        real Epsi
        parameter(maxiter=100,Epsi=5.e-4)
        integer i,Niter
        real Factor,Change,SumRVV,SumWt,SumRVM
        complex Temp
        real Factors(11)
        data Factors/0.5,0.75,8*0.9,0.5/
c
c  Calculate initial phase gain estimates.
c
        call phasol(NBlines,Nants,Sum,SumVM,b1,b2,gain,convrg)
        if(.not.convrg)return
c
c  Get an initial approximation of the gain solution. This finds a single
c  real gain which minimises the error. This helps stablise the algorithm
c  when the gain solution is very different from 1 (e.g. when we are
c  calculating a priori calibration factors).
c
        SumRVM = 0
        SumRVV = 0
        do i=1,NBlines
          SumRVM = SumRVM + conjg(gain(b1(i)))*gain(b2(i))*SumVM(i)
          SumRVV = SumRVV + SumVV(i)
        enddo
        Factor = sqrt(abs(SumRVM / SumRVV))
c
c  Ready for the amplitude/phase solution.
c
        do i=1,NAnts
          Gain(i) = Factor * Gain(i)
          Sum(i) = 0
          Sum2(i) = 0.
        enddo
c
c  Iterate.
c
        Convrg = .false.
        Niter = 0
        do while(.not.Convrg.and.Niter.lt.MaxIter)
          Niter = Niter + 1
c
c  Set the damping constant. I do not think this is really necessary.
c  AIPS does it.
c
          if(Nants.le.6)then
            Factor = 0.5
          else
            Factor = Factors(min(11,Niter))
          endif
c
c  Sum the contributions over the baselines. Note that the following
c  loop has a dependency.
c
          do i=1,nblines
            Sum(b1(i)) = Sum(b1(i)) + Gain(b2(i)) *       SumVM(i)
            Sum(b2(i)) = Sum(b2(i)) + Gain(b1(i)) * conjg(SumVM(i))
            Sum2(b1(i)) = Sum2(b1(i)) +
     *       (real(Gain(b2(i)))**2 + aimag(Gain(b2(i)))**2) * SumVV(i)
            Sum2(b2(i)) = Sum2(b2(i)) +
     *       (real(Gain(b1(i)))**2 + aimag(Gain(b1(i)))**2) * SumVV(i)
          enddo
c
c  Update the gains. The following should be flagged as a short loop
c  on the Cray, if we assume we have fewer than 32 antennae. Hopefully
c  this will vectorise.
c
          Change = 0
          SumWt = 0
          do i=1,nants
            Temp = Sum(i)/Sum2(i) - Gain(i)
            Gain(i) = Gain(i) + Factor * Temp
            Change = Change + (real(Temp)**2 + aimag(Temp)**2)
            SumWt  = SumWt  + (real(Gain(i))**2 + aimag(Gain(i))**2)
            Sum(i) = (0.,0.)
            Sum2(i) = 0.
          enddo
          Convrg = Change/SumWt .lt. Epsi
        enddo
        end
