        program passmake
c
c   Makes passband calibration data set from a standard passband observation.
c   If a gain calibration file is specified, the data are calibrated
c   before averaging. Output data is intended for the program PASSFIT
c   but is a normal MIRIAD UV data set.
c
c  History:
c    lgm   3may90   Created as a part of a decision to split-up
c                   the passband calibration into two parts
c    pjt  15may90   Got rid of my bug
c    pjt  26jun90   Fixed Lee's bug pha->phas on line 299; input keya->keyf
c    pjt  30jul90   " -> '
c    pjt  29nov90   inline doc/findbase
c    lgm  25jan91   input in two way defaults for mode
c    pjt   7feb91   verbosities and such
c    pjt  21mar91   delete dataset when program failed
c    lgm  23mar91   corrected bug - used ucase when should be lcase
c    pjt  16may91   new doc
c    pjt   8aug91   extra check if no narrow band channels: die!
c    pjt  16sep91   added select= to documentation
c    lgm  02oct91   improved (?) documentation and one output statement.
c    rjs  15jul92   Copied wfreq and wwidth variables, corrected ordering
c		    of wcorr writes.
c    lgm   9jul93   commented out bad channel verbosities
c    pjt   6jan94   cosmetic doc changes to appease Stuart (doc cannot
c                   handle > 70 characters per line !)
c   pjt   16oct96   MAXWIDE now from maxdim.h
c   mchw   9feb99   Changes to calapply.h and caldefs.h
c
c------ Inline doc (retrieved with doc to a .doc file) ----------------c
c
c= passmake - Create passband calibration set
c& lgm
c: calibration
c+
c PASSMAKE is a MIRIAD task that makes a baseline based passband
c calibration dataset, to be used by PASSFIT and CALAPPLY.
c
c@ vis
c The passband calibrator visibility dataset.  Only one input
c dataset allowed. This is normally an observation of a passband
c calibrator (strong continuum source) observed in the same narrow-band
c correlator mode as the science source; alternatively, it may be a
c passband calibrator observed in the broad-band mode (correlator
c mode 4, 40 MHz) provided the frequencies observed in broad-band
c mode include the frequencies observed in the narrow-band observed
c for the science source.
c No default.
c
c< select
c Default: all data selected.
c
c@ gcal
c The phase and amplitude gains solution to apply to data, usually
c obtained from using calfit on broad-band observations of the passband
c calibrator before and after the narrow-band observations; it is not
c used,needed? for mode = self.
c Default is to apply NO gains solution.
c
c@ aver
c Type of time-averaging to apply to passband data. Choices are ``vector'',
c ``scalar'' or ``self calibration''.  If self-calbration is selected,
c the individual digital correlator channels are divided by the wideband
c digital correlator channel AT EACH TIME INTERVAL, then vector averaged.
c aver=Self-cal overrides user input for 'mode'.
c lower sidebands.
c Default: vector
c
c@ mode
c Method for normalizing the digital correlator channels relative to the
c USB and LSB wide-band digital channels. Choices are ``cross'' and
c ``self''.  Cross mode normalizes the channels relative to the wide-band
c data obtained  before and after the narrow-band passband observation,
c using the gain file specified by gcal. The channel data are divided at
c each time interval by  the interpolated fit and averaged according to
c values of aver. Self mode  normalizes the channel data relative to the
c wideband digital correlator by dividing the averaged channel data by the
c averaged wideband data; the division is done after averaging according
c to the value of aver.
c Default is ``cross'' if a gcal file is specified.
c Default is ``self'' is no gcal file is specified.
c
c@ pcal
c Name for output passband calibration data file.  This file may
c be used as an input to calapply to calibrate the passband of
c a dataset or passfit to interactively fit low order polynomials.
c No default.
c------------------------------------------------------------------------
        INTEGER MAXSELS, NVAR
        PARAMETER (MAXSELS = 100)
        PARAMETER (NVAR = 40)
        CHARACTER PVERSION*(*)
        PARAMETER (PVERSION = 'Version 1.0 09-Feb-99')
c
c >>> ../../../src/inc/caldefs.h
c=======================================================================
c [caldefs.h]
c General include file that every calibration routine needs. It does not
c define (space for) data, but defines the constants that should be used
c in defining array-sizes.
c	18-nov-90 	added PI and TWOPI			PJT
c	 6-jul-91	V6.3 new MAXANTHC and MAXBASHC          pjt
c	26-sep-91       increased MAXUVPNT and MAXBREAK
c	 2-dec-91	V6.4 changed calbflux's dimensionality	PJT
c	19-apr-93	MAXSLOT from 20 to 36 for screwed mode-4  PJT
c			added MAXWIN from 8 (hardcoded) to 16
c	15-apr-94	MAXWIN is now in maxdim.h		PJT/mchw
c       19-dec-95       increase MAXANTHC from 6 to 9 !!!       pjt
c        9-Feb-99       increase MAXANTHC from 9 to 10 !       mchw
c       25-Apr-06       Increase MAXANTHC from 10 to 15 for CARMA ! mh
c=======================================================================
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c=======================================================================
c >>> ../../../src/inc/size.h
c=======================================================================
c	[size.h]
c
c  Include file used by calibration software
c
c  UL, RI, ULRI -- parameters which stand for LSB/USB and REAL/IMAG stuff
c
        integer UL, RI, ULRI
        parameter( UL = 2 )
        parameter( RI = 2 )
        parameter( ULRI = 4 )
c
c  SIZE -- lengths of reals/integers in number of bytes (IEEE/2s compl)
c
        integer SIZE
        parameter( SIZE = 4 )
c=======================================================================
c
c <<< ../../../src/inc/size.h
c=======================================================================
c
c  SVERSION -- current version of cal format (consistency checks)
c              Version number is 10*MAJOR+MINOR
c              Datasets with a MAJOR difference, cannot be read
c              anymore, MINOR differences are normally compatible
c
      INTEGER SVERSION
      PARAMETER ( SVERSION = 64)
c
c MAXANTHC -- special Hat Creek MAXANT, to keep it small for us....
c MAXBASHC -- derived from MAXANTHC = max. number of baselines
c
c Now changed the number from Hat-Creek (10) to Cedar-Flat (15)
c although the name is the same.
c
      INTEGER MAXANTHC, MAXBASHC
      PARAMETER (MAXANTHC=15, MAXBASHC=((MAXANTHC*(MAXANTHC-1))/2))
c
c MAXUVPNT -- maximum number of UV points in time per track
c
      INTEGER   MAXUVPNT
      PARAMETER(MAXUVPNT=2000)
c
c  MAXSRC -- maximum number of sources in multi-source databases
c
      INTEGER MAXSRC
      PARAMETER( MAXSRC = 32 )
c
c  MAXBREAK -- maximum number of breakpoints on an interval
c
      INTEGER MAXBREAK
      PARAMETER( MAXBREAK = 10 )
c
c  MAXPOLY -- maximum number of polynomials on an interval
c             is normally one more than number of breakpoints, right?
c
      INTEGER MAXPOLY
      PARAMETER( MAXPOLY = MAXBREAK+1 )
c
c  MAXSLOT -- maximum number of polynomial slots
c             Allows for simultaneous presence of gcal & pcal format
c             The first 4 for gcal, last 16 for pcal
c             CALIB makes use of slot 5 and 6 for phase diff&mean
      INTEGER MAXSLOT
c      PARAMETER( MAXSLOT = 20 )
      PARAMETER( MAXSLOT = 36 )
c
c  MAXORDER -- highest allowed polynomial order
c             Don't set too high, inversions are already needed in
c             double precision this way.
      INTEGER MAXORDER
      PARAMETER( MAXORDER = 5 )
c
c  MAXWIN -- maximum number of windows - normally equals MAXSPECT = 8
c	     but temporarely increased to 16 to deal with mode=4
c	     problems.
c     INTEGER MAXWIN
c     PARAMETER ( MAXWIN = 16 )
c
c=======================================================================
c
c <<< ../../../src/inc/caldefs.h
c >>> ../../../src/inc/calapply.h
c=======================================================================
c   [calappply.h]
c		dark ages	bs/lgm/pjt, in some form
c		17-dec-91	documentation format
c
c*calapply_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c    Include file for reading and writing uv data files used by
c    subroutines ... READNEXT and WRITNEXT ... ???
c    It also has a common block /calcnt/ keeping track of good and
c    bad scans
c
c    Space used:  appr. 2 * MAXCHAN
c
c
      DOUBLE PRECISION preamble(4)
      COMPLEX          data(MAXCHAN)
      LOGICAL          flags(MAXCHAN), wflags(MAXWIDE)
      INTEGER          nread,nwcorr
      COMPLEX          wcorr(MAXWIDE)
      COMMON /uvdata/ preamble, data, flags, nread, wcorr, nwcorr,
     *                wflags
c
      INTEGER calcntg, calcntb
      COMMON /calcnt/calcntg, calcntb
c--
c  History:
c    9feb99 changed wflags and wcorr from 8 to MAXWIDE
c=======================================================================
c <<< ../../../src/inc/calapply.h
c >>> ../../../src/inc/calsubs.h
c=======================================================================
c	[calsubs.h]
c
c	Last update:    22-may-90	deleted pmode	PJT
c			18-dec-90	played with  /calmode/ [Arie/Peter]
c			 7-jul-91       new MAXBASHC parameter
c			11-dec-91	documentation formalized in doc format
c
c*calsubs_h -- header file for calibration
c:calibration, include
c&pjt
c+
c
c  time0 -- Julian time offset for all time measurements
        DOUBLE PRECISION time0
c
c  base(nbl) -- the baseline number and order
        INTEGER nbl, base(MAXBASHC)
c
c  version -- the read version of cal format
        INTEGER version
c
c  calsubs -- THE common block
        COMMON /calsubs/ time0, nbl, base, version
c
c
c  kludge before upgrading to the next CAL-SVERSION
        INTEGER scalmode
c
c	0:	(old system) fitting is done in K
c	1:	fitting is done in data space (K/Jy), i.e. the same is above
c      -1:	fitting is done in gain space (Jy/K)
c
c   For now, scalmode is set in calfit and calflag and such.
c   CAopen should get an extra stamp for this, see calio.h
c
        COMMON /calscal/scalmode
c--
c   Mode of phase/amp calibration in calflag/calib/...
c   [may be used in Arie's calib.for - but is internal for now]
c   nmode=1   amp
c	  2   phase
c         3   diff
c
c	INTEGER nmode
c	COMMON /calmode/nmode
c=======================================================================
c
c <<< ../../../src/inc/calsubs.h
c >>> ../../../src/inc/calpass.h
c  [calpass.h]
c
c	History:
c
c	             Original version
c	02aug91 mjs  Changed to use CALIB-specific MAXBASE/MAXANT
c	19apr93	pjt	assume caldefs.h has been included
c
c   Common and declaration statements for CALPASS program
c
c   Space used: 4*(MAXCHAN+MAXWIN)*MAXBASE ~ 4*MAXCHAN*MAXBASE
c
        REAL      scalamp(MAXBASHC,MAXCHAN), scalphas(MAXBASHC,MAXCHAN),
     *            freqs(MAXBASHC,MAXCHAN)
        INTEGER   starwin(MAXWIN), chanwin(MAXWIN),baseline,b,nwins
        CHARACTER code(MAXBASHC,MAXWIN)*4
        LOGICAL   passflag(MAXBASHC,MAXCHAN)
c
        COMMON /passbandr/ scalamp, scalphas, freqs
        COMMON /passbandi/ starwin, chanwin,baseline,b,nwins
        COMMON /passbandc/ code
        COMMON /passbandl/ passflag
c
c <<< ../../../src/inc/calpass.h
c
        character*80 infile, gainset, outfile
        character aver*10,line*80,mode*10,source*9
        integer inset,outset,npts(MAXBASHC,MAXCHAN),nchans
        integer ic, nloop,wpts(MAXBASHC,MAXWIDE),test
        integer winflag(MAXCHAN)
        real amp, phas, pdiff,sels(MAXSELS),evalpoly,float
        real lsbamp, lsbphi, usbamp, usbphi, time
        real wamp(MAXBASHC,MAXWIDE),wphas(MAXBASHC,MAXWIDE)
        complex wecspec(MAXBASHC,MAXWIDE),lsb, usb
        complex vecspec(MAXBASHC,MAXCHAN),widecor,wides(MAXBASHC,2)
        integer findbase, iwin, ivar, valid, badpoly
        double precision pream(MAXBASHC,4),start,stop
        logical wpassfl(MAXBASHC,MAXWIDE),chkpoly,firstime(MAXBASHC)
        character variable(NVAR)*8
c
        external evalpoly
        integer  len1
c
cccc
c DANGER: how to keep this list up to date if variables have been added
c	  at least increase the parameter NVAR
        DATA variable/'antpos  ','axisrms ','ddec    ','dec     ',
     *     'dra     ','epoch   ','instrume','inttime ','jyperk  ',
     *     'lst     ','nants   ','npols   ','obsdec  ','nchan   ',
     *     'observer','obsra   ','plangle ','plmaj   ','plmin   ',
     *     'pltb    ','pol     ','ra      ','source  ','telescop',
     *     'ut      ','veldop  ','veltype ','vsource ','cormode ',
     *     'ischan  ','nschan  ','nspect  ','restfreq','corfin  ',
     *     'sdf     ','sfreq   ','systemp ','corbw   ','wfreq   ',
     *     'wwidth  '/
        DATA firstime/MAXBASHC*.true. /
c
c
c------------------ announce entry ---------------------------C
c
        call output('PASSMAKE: '//pversion)
c
c---------------- initialize arrays ---------------------C
c
        do b =1,MAXBASHC
        do ic=1,MAXCHAN
           npts(b,ic) = 0
           scalamp(b,ic) = 0.0
           scalphas(b,ic) = 0.0
           vecspec(b,ic) = cmplx(0.0,0.0)
        enddo
        do ic=1,MAXWIDE
           wpts(b,ic) = 0
           wamp(b,ic) = 0.0
           wphas(b,ic) = 0.0
           wecspec(b,ic) = cmplx(0.0,0.0)
        enddo
        enddo
c
c---------- get the inputs --------------------------------------------C
c
        call keyini
c
        call keyf( 'vis', infile, ' ')
        if(infile.eq.' ') call bug('f',
     *                              'Input file vis= must be specified')
c
        call SelInput('select',sels,maxsels)
c
c------------------- Read in gain fits from disk ----------------------C
c           See that all four fits are present in gainset
c
c   ASSUMES that NBL and BASE are filled properly by getpoly
c
        call keyf( 'gcal', gainset, ' ')
        if(gainset .ne. ' ') then
           call getpoly( gainset )
           badpoly = 0
           if(.not. chkpoly('ALW ')) badpoly = 1
           if(.not. chkpoly('PLW ')) badpoly = badpoly + 1
           if(badpoly .gt. 0)
     *     call logwrite('No fit to lower sideband in gainset',.true.)
           if(.not. chkpoly('AUW ')) badpoly = badpoly + 10
           if(.not. chkpoly('PUW ')) badpoly = badpoly + 10
           if(badpoly .gt. 10)
     *     call logwrite('No fit to upper sideband in gainset',.true.)
           if(badpoly .gt. 0) call bug('f','Improper gainset gcal=')
        else
           nbl = 0
           do b=1,MAXBASHC
              base(b) = 0.0
           enddo
        endif
c
        call keya( 'aver', aver, 'vector')
           call lcase(aver)
           if(aver(1:2) .eq. 'sc') then
              aver = 'scalar'
           else if(aver(1:2) .eq. 've') then
              aver = 'vector'
           else if(aver(1:2) .eq. 'se') then
              aver = 'self'
              call output(' aver=self overrides selection for mode')
           else
              call bug('f','Bad option: aver=scalar, vector or self')
           endif
           call output(' Averaging type: aver='//aver)
c
        call keya( 'mode',mode,' ')
           call lcase(mode)
           if(mode(1:1) .eq. ' ') then
              if(gainset(1:1) .eq. ' ') then
                 mode = 'self'
              else
                 mode = 'cross'
              endif
           endif
           if(mode(1:1) .eq. 'c') then
              mode = 'cross'
              if(gainset(1:1) .eq. ' ')
     *             call bug('f','Gain set needed for mode=cross')
           else if(mode(1:1) .eq. 's') then
              mode = 'self'
           else
              call bug('f','Bad option: mode=cross or self')
           endif
           call output(' Normalization mode='//mode)
c
        call keya( 'pcal',outfile, ' ')
        if(outfile(1:1) .eq. ' ')
     *         call bug('f','Output file pcal= must be specified')
c
        call keyfin
c
c---------------- Open old uv file and apply selects ------------------C
c		Also force to read narrow band data
        call output('Reading dataset vis='//infile(1:len1(infile)))
        call uvopen(inset, infile, 'old')
        call uvset(inset,'data','channel',0,1.0,1.0,1.0)
        call uvselect(inset,'clear',0.0d0,0.0d0,.true.)
        call SelApply(inset,sels,.true.)
c
c------------------- set-up probes on variables -----------------------C
c
       do 20 ivar=1,nvar
          call uvtrack(inset,variable(ivar),'c')
   20  continue
c
c------------------- Open output uv file ------------------------------C
c
       call uvopen(outset,outfile,'new')
c
c------------------ begin loop reading uv records ---------------------C
c
        nloop = 0
   25   continue
        nloop = nloop + 1
c
c------------------ read in uv data ------------------C
c
        call uvread(inset,preamble,data,flags,maxchan,nread)
        if(nread .le. 0) go to 195
c
c------------- read in wide band channel data ----------C
c
        call uvwread(inset,wcorr,wflags,MAXWIDE,nwcorr)
        baseline = preamble(4)
        stop     = preamble(3)
c
c-------- on first loop, write tracked variables to output file -------C
c
        if(nloop .eq. 1) then
           call uvcopyvr(inset,outset)
           call hdcopy(inset,outset,'history')
           call hisopen(outset,'append')
           call hiswrite(outset,'PASSMAKE: '//pversion)
           call hisinput(outset,'PASSMAKE')
           call uvgetvri(inset,'nchan',nchans,1)
           call uvgetvri(inset,'nspect',nwins,1)
           call uvgetvri(inset,'ischan',starwin,nwins)
           call uvgetvri(inset,'nschan',chanwin,nwins)
           call uvgetvra(inset,'source',source)
           start = preamble(3)
           do 50 iwin=1,nwins
           do 50 ic = starwin(iwin),starwin(iwin)+chanwin(iwin)-1
              if(iwin .le. nwins/2) then
                 winflag(ic) = -iwin
              else
                 winflag(ic) = iwin - nwins/2
              endif
  50       continue
        endif
        if(nchans .ne. nread) call output(
     *             'Warning: Number of narrowband channels changed')
c
c------ Reverse baseline if the first ant is greater than the second---C
c       This should never occur anymore....
c
        if(baseline/256 .gt. mod(baseline,256)) then
           call bug('w','PASSMAKE: Illegal baseline order')
           baseline = 256*mod(baseline,256) + baseline/256
           preamble(4) = baseline
           do 60 ic=1,nread
              data(ic) = conjg(data(ic))
   60      continue
           do 70 ic=1,nwcorr
              wcorr(ic) = conjg(wcorr(ic))
   70      continue
        endif
c
c------------------ check baselines -----------------------------------C
c   if there is a gain calib set make sure that the data baseline is in there
c   if no gain calib set, keep track of baselines and make sure there aren't
c   too many.
c
        if(gainset .ne. ' ') then
           b = findbase(baseline,base,nbl)
           if( b .eq. 0 ) then
                call rmdata(outfile)
                call bug('f','Baselines incompatable with file gcal=')
           endif
           if(firstime(b)) then
              do ic=1,4
                 pream(b,ic) = preamble(ic)
              enddo
              firstime(b) = .false.
           endif
        else
           b = findbase(baseline,base,nbl)
           if( b .eq. 0 ) then
              nbl = nbl + 1
              if(nbl.gt.MAXBASHC) then
                  call rmdata(outfile)
                  call bug('f','Too many baselines')
              endif
              base(nbl) = baseline
              b = nbl
              do ic=1,4
                 pream(b,ic) = preamble(ic)
              enddo
           endif
        endif
c---------- evaluate the phase and amp corrections --------------------C
c  Use value from poly fits of specified otherwise set to cmplx(1,0)
c
        if(gainset .ne. ' ') then
            time = preamble(3) - time0
            lsbamp = evalpoly(time,'ALW ',baseline,valid)
            lsbphi = evalpoly(time,'PLW ',baseline,valid)
            usbamp = evalpoly(time,'AUW ',baseline,valid)
            usbphi = evalpoly(time,'PUW ',baseline,valid)
            lsb = lsbamp * cmplx(cos(lsbphi),sin(lsbphi))
            usb = usbamp * cmplx(cos(usbphi),sin(usbphi))
        else
            lsb = cmplx(1.0,0.0)
            usb = cmplx(1.0,0.0)
        endif
c
c------ accumulate vector and scalar data for wideband correlations ---C
c
        do 100 ic=1,nwcorr
           if(wflags(ic)) then
              test = mod(ic,2)
              if(test .eq. 1) then
                 wcorr(ic) = wcorr(ic)/lsb
              else
                 wcorr(ic) = wcorr(ic)/usb
              endif
              if(aver(1:3) .eq. 'sca') then
                 call amphase(wcorr(ic),amp,phas)
                 phas = 3.14159265 * phas/180.0
                 wamp(b,ic) = wamp(b,ic) + amp
                 if(nloop .gt. 1) then
                    pdiff = phas - wphas(b,ic)
                    if(abs(pdiff) .gt. 3.0)
     *                phas = phas - sign(6.283185,pdiff)
                 endif
                 wphas(b,ic) = wphas(b,ic) + phas
              endif
              wecspec(b,ic)  = wecspec(b,ic) + wcorr(ic)
              wpts(b,ic)     = wpts(b,ic) + 1
           else
              write(line,'(''Bad Wideband: '',i3,''  Baseline: '',i5)')
     *                    ic,b
              call output(line)
           endif
  100   continue
c
c---------- accumulate vector and scalar data for each channel --------C
c                apply gain correction before accumulation
c
        do 150 ic=1,nread
           if(flags(ic)) then
              if(winflag(ic) .lt. 0) then
                 data(ic) = data(ic) / lsb
                 widecor = wcorr(1)
              else
                 data(ic) = data(ic) / usb
                 widecor = wcorr(2)
              endif
              if(aver(1:3) .eq. 'sca') then
                 call amphase(data(ic),amp,phas)
                 phas = 3.14159265 * phas/180.0
                 scalamp(b,ic)  = scalamp(b,ic) + amp
                 if(nloop .gt. 1) then
                    pdiff = phas - scalphas(b,ic)
                    if(abs(pdiff) .gt. 3.0)
     *                phas = phas - sign(6.283185,pdiff)
                 endif
                 scalphas(b,ic) = scalphas(b,ic) + phas
              endif
              if(aver(1:3) .eq. 'sel') then
                 vecspec(b,ic)  = vecspec(b,ic) + data(ic)/widecor
              else
                 vecspec(b,ic)  = vecspec(b,ic) + data(ic)
              endif
              npts(b,ic)     = npts(b,ic) + 1
           else
c              write(line,'(''Bad Channel: '',i3,''  Baseline: '',i5)')
c     1                    ic,b
c              call output(line)
           endif
  150   continue
c
        go to 25
c
c------------ end of uv read and accumulation loop --------------------C
c
  195   continue
        if(nloop .eq. 1) then
           call rmdata(outfile)
           call bug('f','No channel data read from file vis=')
        endif
c
c--------- normalize spectra by number of time samples ----------------C
c
        do 260 b=1,nbl
        do 200 ic=1,nchans
           if(npts(b,ic) .gt. 0) then
              if(aver(1:3) .eq. 'vec') then
                 vecspec(b,ic)  = vecspec(b,ic)/ float(npts(b,ic))
              else if (aver(1:3) .eq. 'sca') then
                 scalphas(b,ic) = scalphas(b,ic)/float(npts(b,ic))
                 if(abs(scalphas(b,ic)) .gt. 3.14159) scalphas(b,ic) =
     *                 scalphas(b,ic) - sign(6.283185,scalphas(b,ic))
                 scalamp(b,ic)  = scalamp(b,ic)/ float(npts(b,ic))
                 vecspec(b,ic)  = cmplx(scalamp(b,ic)*cos(
     *                     scalphas(b,ic)),
     *                     scalamp(b,ic)*sin(scalphas(b,ic)))
              else
                 if(winflag(ic) .lt. 0) then
                    widecor = wecspec(b,1)
                 else
                    widecor = wecspec(b,2)
                 endif
                 vecspec(b,ic) = vecspec(b,ic)/float(npts(b,ic))
              endif
              passflag(b,ic) = .true.
           else
              passflag(b,ic) = .false.
              scalamp(b,ic)  = 0.0
              scalphas(b,ic) = 0.0
           endif
  200   continue
c
        do 250 ic=1,nwcorr
           if(wpts(b,ic) .gt. 0) then
              if(aver(1:3) .eq. 'vec') then
                 wecspec(b,ic)  = wecspec(b,ic)/ float(wpts(b,ic))
              else if (aver(1:3) .eq. 'sca') then
                 wphas(b,ic) = wphas(b,ic)/float(wpts(b,ic))
                 if(abs(wphas(b,ic)) .gt. 3.14159) wphas(b,ic) =
     *                     wphas(b,ic) - sign(6.283185,wphas(b,ic))
                 wamp(b,ic)  = wamp(b,ic)/ float(wpts(b,ic))
                 wphas(b,ic) = 1.74533e-2 * wphas(b,ic)
                 wecspec(b,ic)  = cmplx(wamp(b,ic)*cos(wphas(b,ic)),
     *                     wamp(b,ic)*sin(wphas(b,ic)))
              endif
              wpassfl(b,ic) = .true.
           else
              wpassfl(b,ic) = .false.
              wecspec(b,ic) = cmplx(0.0,0.0)
           endif
  250   continue
  260   continue
c
        call uvclose(inset)
c
c--------- pick up gainset amps and phase if MODE = cross -------------C
c
        if(mode(1:3) .eq. 'cro')
     *          call getwides(gainset,start,stop,source,wides,outfile)
c
c------------- Normalize data for aver was not self-cal ---------------C
c
        if(aver(1:3) .ne. 'sel') then
           if(mode(1:3) .eq. 'sel') then
              do 300 b=1,nbl
              do 300 ic=1,nchans
                 if(winflag(ic) .lt. 0) then
                    vecspec(b,ic) = vecspec(b,ic)/wecspec(b,1)
                 else
                    vecspec(b,ic) = vecspec(b,ic)/wecspec(b,2)
                 endif
  300         continue
           else
              do 350 b=1,nbl
              do 350 ic=1,nchans
                 if(winflag(ic) .lt. 0) then
                    vecspec(b,ic) = vecspec(b,ic)/wides(b,1)
                 else
                    vecspec(b,ic) = vecspec(b,ic)/wides(b,2)
                 endif
  350         continue
           endif
        endif
c
c------------ Loop thru writing out baselines ------------------------C
c
        do 500 b=1,nbl
           do 400 ic=1,4
              preamble(ic) = pream(b,ic)
  400      continue
           do 450 ic=1,nwcorr
              wcorr(ic) = wecspec(b,ic)
              wflags(ic) = wpassfl(b,ic)
  450      continue
           call uvwwrite(outset,wcorr,wflags,nwcorr)
           do 420 ic=1,nchans
              data(ic)  = vecspec(b,ic)
              flags(ic) = passflag(b,ic)
  420      continue
           call uvwrite(outset,preamble,data,flags,nchans)
  500   continue
c
        call hiswrite(outset,'PASSMAKE: Successfully completed')
        call hisclose(outset)
        call uvclose(outset)
c
c------------- Write extra info needed for polynomial fits --------C
c
        call caopen(outset,outfile,preamble(3),nbl,base,SVERSION,
     *              'append')
        call caclose(outset)
c
        stop
        end
        subroutine getwides(gainset,start,stop,source,wides,outfile)
c
c-----------------------------------------------------------------------
c
c   Reads wideband data from gcal dataset for use in cross-cal
c   normalization of passband data.
c   If program fails, it also needs to delete output dataset
c
c >>> ../../../src/inc/caldefs.h
c=======================================================================
c [caldefs.h]
c General include file that every calibration routine needs. It does not
c define (space for) data, but defines the constants that should be used
c in defining array-sizes.
c	18-nov-90 	added PI and TWOPI			PJT
c	 6-jul-91	V6.3 new MAXANTHC and MAXBASHC          pjt
c	26-sep-91       increased MAXUVPNT and MAXBREAK
c	 2-dec-91	V6.4 changed calbflux's dimensionality	PJT
c	19-apr-93	MAXSLOT from 20 to 36 for screwed mode-4  PJT
c			added MAXWIN from 8 (hardcoded) to 16
c	15-apr-94	MAXWIN is now in maxdim.h		PJT/mchw
c       19-dec-95       increase MAXANTHC from 6 to 9 !!!       pjt
c        9-Feb-99       increase MAXANTHC from 9 to 10 !       mchw
c       25-Apr-06       Increase MAXANTHC from 10 to 15 for CARMA ! mh
c=======================================================================
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c=======================================================================
c >>> ../../../src/inc/size.h
c=======================================================================
c	[size.h]
c
c  Include file used by calibration software
c
c  UL, RI, ULRI -- parameters which stand for LSB/USB and REAL/IMAG stuff
c
        integer UL, RI, ULRI
        parameter( UL = 2 )
        parameter( RI = 2 )
        parameter( ULRI = 4 )
c
c  SIZE -- lengths of reals/integers in number of bytes (IEEE/2s compl)
c
        integer SIZE
        parameter( SIZE = 4 )
c=======================================================================
c
c <<< ../../../src/inc/size.h
c=======================================================================
c
c  SVERSION -- current version of cal format (consistency checks)
c              Version number is 10*MAJOR+MINOR
c              Datasets with a MAJOR difference, cannot be read
c              anymore, MINOR differences are normally compatible
c
      INTEGER SVERSION
      PARAMETER ( SVERSION = 64)
c
c MAXANTHC -- special Hat Creek MAXANT, to keep it small for us....
c MAXBASHC -- derived from MAXANTHC = max. number of baselines
c
c Now changed the number from Hat-Creek (10) to Cedar-Flat (15)
c although the name is the same.
c
      INTEGER MAXANTHC, MAXBASHC
      PARAMETER (MAXANTHC=15, MAXBASHC=((MAXANTHC*(MAXANTHC-1))/2))
c
c MAXUVPNT -- maximum number of UV points in time per track
c
      INTEGER   MAXUVPNT
      PARAMETER(MAXUVPNT=2000)
c
c  MAXSRC -- maximum number of sources in multi-source databases
c
      INTEGER MAXSRC
      PARAMETER( MAXSRC = 32 )
c
c  MAXBREAK -- maximum number of breakpoints on an interval
c
      INTEGER MAXBREAK
      PARAMETER( MAXBREAK = 10 )
c
c  MAXPOLY -- maximum number of polynomials on an interval
c             is normally one more than number of breakpoints, right?
c
      INTEGER MAXPOLY
      PARAMETER( MAXPOLY = MAXBREAK+1 )
c
c  MAXSLOT -- maximum number of polynomial slots
c             Allows for simultaneous presence of gcal & pcal format
c             The first 4 for gcal, last 16 for pcal
c             CALIB makes use of slot 5 and 6 for phase diff&mean
      INTEGER MAXSLOT
c      PARAMETER( MAXSLOT = 20 )
      PARAMETER( MAXSLOT = 36 )
c
c  MAXORDER -- highest allowed polynomial order
c             Don't set too high, inversions are already needed in
c             double precision this way.
      INTEGER MAXORDER
      PARAMETER( MAXORDER = 5 )
c
c  MAXWIN -- maximum number of windows - normally equals MAXSPECT = 8
c	     but temporarely increased to 16 to deal with mode=4
c	     problems.
c     INTEGER MAXWIN
c     PARAMETER ( MAXWIN = 16 )
c
c=======================================================================
c
c <<< ../../../src/inc/caldefs.h
c >>> ../../../src/inc/calsubs.h
c=======================================================================
c	[calsubs.h]
c
c	Last update:    22-may-90	deleted pmode	PJT
c			18-dec-90	played with  /calmode/ [Arie/Peter]
c			 7-jul-91       new MAXBASHC parameter
c			11-dec-91	documentation formalized in doc format
c
c*calsubs_h -- header file for calibration
c:calibration, include
c&pjt
c+
c
c  time0 -- Julian time offset for all time measurements
        DOUBLE PRECISION time0
c
c  base(nbl) -- the baseline number and order
        INTEGER nbl, base(MAXBASHC)
c
c  version -- the read version of cal format
        INTEGER version
c
c  calsubs -- THE common block
        COMMON /calsubs/ time0, nbl, base, version
c
c
c  kludge before upgrading to the next CAL-SVERSION
        INTEGER scalmode
c
c	0:	(old system) fitting is done in K
c	1:	fitting is done in data space (K/Jy), i.e. the same is above
c      -1:	fitting is done in gain space (Jy/K)
c
c   For now, scalmode is set in calfit and calflag and such.
c   CAopen should get an extra stamp for this, see calio.h
c
        COMMON /calscal/scalmode
c--
c   Mode of phase/amp calibration in calflag/calib/...
c   [may be used in Arie's calib.for - but is internal for now]
c   nmode=1   amp
c	  2   phase
c         3   diff
c
c	INTEGER nmode
c	COMMON /calmode/nmode
c=======================================================================
c
c <<< ../../../src/inc/calsubs.h
c >>> ../../../src/inc/caldata.h
c=======================================================================
c       [caldata.h]
c       Header file for calibration data
c
c       Last update:    14-mar-90       added Sflag     PJT
c                       20-mar-90       srcindex now srcindex(MAXPNT)   PJT
c                        7-may-90       rdata(ULRI,MAXBASHC,MAXUVPNT)   PJT
c                        4-sep-90       baseline based calib stuff      PJT
c                       18-nov-90       more descriptive comments       PJT
c                       18-dec-90       played with rdata()             PJT
c                        7-jul-91       MAXBASE -> BAXMASHC             PJT
c                        2-dec-91       make fluxes time dependant, not
c                                       source dependant; =V6.4 fmt     PJT
c                       11-dec-91       documentation formalized in doc format
c*caldata_h -- header file for calibration data
c:calibration, include
c&pjt
c+
c   The calibration format on disk is split out in memory over several
c   common blocks, each of them described in a separate include file.
c
c
c <general header> (see also calsubs.h)
c       double time0            time to which all times are offset (JD)
c       int    nbl              number of baselines
c       int    base(nbl)        nbl baselines (notation: A1*256+A2)
c       int    version          version of the calibration format
c
c <data> (this caldata.h include file)
c   rtime
c       float  rtime(rcount)             offset times to time0
c   rdata
c       float  rdata(ULRI,nbl,rcount)    correllation data
c   rflag
c       int    rflag(UL,nbl,rcount)      flagged bad? (0=false, 1=true)
c   sindex
c       int    sindex(rcount)            source index number
c   sname
c       char*16 sname(scount)             list of sources
c
c
c
c  Rxxxx -- the actual calibration data
c
      INTEGER   rcount
      REAL      rtime(MAXUVPNT)
      REAL      rdata(ULRI, MAXBASHC, MAXUVPNT)
      INTEGER   rflag(UL, MAXBASHC, MAXUVPNT)
c
c  Sxxxx -- index for multiple source calibration format
c           (plstuff was added later and is not quite needed??)
c
      INTEGER   scount
      CHARACTER sname(MAXSRC)*16
      INTEGER   sindex(MAXUVPNT), sflag(MAXUVPNT)
      REAL      plstuff(4,MAXSRC)
c
c  Bxxxx - list of break points
c
      REAL      btime(MAXBREAK, 2, MAXBASHC)
      INTEGER   bcount(2, MAXBASHC)
c
c  Xxxxx - odds and ends, should not really have to exist
c          are merely there for planets
c
      REAL      volts(MAXUVPNT)
c
c  baseline based calibrator fluxes, per time
c
      REAL      calbflux(MAXBASHC,MAXUVPNT)
c
c
c  the common blocks with all the data
c
      COMMON /caldatr/ rcount, scount, rtime, rdata, rflag,
     *                 sindex, btime, bcount, sflag, plstuff,volts
      COMMON /caldatc/ sname
      COMMON /caldatp/ calbflux
c--
c=======================================================================
c
c <<< ../../../src/inc/caldata.h
c
        character source*9,gainset*(*),line*80,outfile*(*)
        integer spoint,idxmin1,idxmin2,i,b,wpts(MAXBASHC,2)
        integer idx(200),baseline,npts,valid
        real tstart,tstop,time1(MAXUVPNT),time2(MAXUVPNT)
        real tmin1,tmin2,lsbamp,lsbpha,usbpha,usbamp
        real evalpoly
        double precision start,stop
        complex wides(MAXBASHC,2),lsb,usb
c
c------------- init output array -------------------C
c
        do 10 b=1,nbl
        do 10 i=1,2
           wides(b,i) = cmplx(0.0,0.0)
           wpts(b,i)  = 0
   10   continue
c
c------------- Read in gainset data from file ----------------------C
c
        call readset(gainset)
c
c------------- Match input source name with sources in data ----------C
c
        spoint = 0
        do 20 i=1,scount
           if(source(1:8) .eq. sname(i)) spoint = i
   20   continue
        if(spoint .eq. 0) then
           call rmdata(outfile)
           write(line,'(''Source name '',a,'' not in gainset'')') source
           call bug('f',line)
        endif
c
c--------- Find relative times of source obs compared to passband obs ---C
c
        tstart = start - time0
        tstop  = stop - time0
        do 100 i=1,rcount
           if(sindex(i) .eq. spoint) then
              if(rtime(i) .lt. tstart) then
                 time1(i) = abs(rtime(i) - tstart)
              else
                 time1(i) = 9999.0
              endif
              if(rtime(i) .gt. tstop) then
                 time2(i) = abs(rtime(i) - tstop)
              else
                 time2(i) = 9999.0
              endif
           else
              time1(i) = 9999.0
              time2(i) = 9999.0
           endif
  100   continue
c
c-------- identify the data that are closest to the star and stop times ---C
c
        tmin1 = time1(1)
        tmin2 = time2(1)
        idxmin1 = 1
        idxmin2 = 1
        do 200 i=2,rcount
           if(time1(i) .lt. tmin1) then
              tmin1 = time1(i)
              idxmin1 = i
           endif
           if(time2(i) .lt. tmin2) then
              tmin2 = time2(i)
              idxmin2 = i
           endif
  200   continue
        if(tmin1 .gt. 0.03) then
           idxmin1 = 0
           call output('No Wideband data within 45 min before passband')
        endif
        if(tmin2 .gt. 0.03) then
           idxmin2 = 0
           call output('No wideband data within 45 min after passband')
        endif
        if(idxmin1 .eq. 0 .and. idxmin2 .eq. 0) then
           call rmdata(outfile)
           call bug('f','Wideband data required in mode=cross')
        endif
c
c------- Identify a continuous string of source observations ----C
c
        npts = 1
        idx(1) = idxmin1
        do 300 i=1,100
           if(idxmin1-i .ge. 1 .and. sindex(idxmin1-i) .eq.
     *                scount) then
              npts = npts + 1
              idx(npts) = idxmin1-i
           else
              go to 305
           endif
  300   continue
  305   continue
        npts = npts + 1
        idx(npts) = idxmin2
        do 320 i=1,100
           if(idxmin2+i .ge. 1 .and. sindex(idxmin2+i) .eq.
     *                scount) then
              npts = npts + 1
              idx(npts) = idxmin2+i
           else
              go to 325
           endif
  320   continue
  325   continue
c
c------- apply gain calibration and accumulate wideband correlations ---C
c
        do 400 i=1,npts
        do 400 b=1,nbl
           baseline = base(b)
           lsbamp = evalpoly(rtime(idx(i)),'ALW ',baseline,valid)
           lsbpha = evalpoly(rtime(idx(i)),'PLW ',baseline,valid)
           usbamp = evalpoly(rtime(idx(i)),'AUW ',baseline,valid)
           usbpha = evalpoly(rtime(idx(i)),'PUW ',baseline,valid)
           lsb = lsbamp * cmplx(cos(lsbpha),sin(lsbpha))
           usb = usbamp * cmplx(cos(usbpha),sin(usbpha))
           if(rflag(1,b,idx(i)) .eq. 1) then
              wides(b,1) = wides(b,1) + cmplx(rdata(1,b,idx(i)),
     *                  rdata(2,b,idx(i)))/lsb
              wpts(b,1) = wpts(b,1) + 1
           endif
           if(rflag(2,b,idx(i)) .eq. 1) then
              wides(b,2) = wides(b,2) + cmplx(rdata(3,b,idx(i)),
     *                   rdata(4,b,idx(i)))/usb
              wpts(b,2) = wpts(b,2) + 1
           endif
  400   continue
c
c------- Re-normalize wideband correlations by number of points ----C
c
        do 500 b=1,nbl
           wides(b,1) = wides(b,1)/wpts(b,1)
           wides(b,2) = wides(b,2)/wpts(b,2)
  500   continue
c
        return
        end
