c************************************************************************
        program pjyperk
c
c= jyperk - Compute new baseline based jyperk from jyperka
c& pjt
c: uv analysis
c+
c	JYPERK computes new baseline based jyperk values from the
c       jyperka array.
c
c       All CARMA data prior to <sometime>-october-2006 suffer from the
c       wrong value for jyperk, so this task should be used early in
c       the data reduction process.
c
c@ vis
c	The names of the input uv data sets. No default.
c@ select
c	Standard visibility data selection. See the help on "select" for
c	more information. The default is to select all data.
c@ out
c	The name of the output uv data set. No default.
c@ jyperka
c       An array of antenna based JY/K values to override the jyperka
c       array in the dataset.
c
c--
c  History:
c    25oct06 pjt  Written, for CARMA
c    22feb07 pjt  warning added
c------------------------------------------------------------------------
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c >>> ../../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../../src/inc/mirconst.h
        character version*(*)
        integer MAXSELS
        parameter(version='jyperk: version 22-feb-07')
        parameter(MAXSELS=256)
c
        real sels(MAXSELS),jyperk
        integer lVis,lOut,vnif,ant1,ant2
        character vis*128,out*128
        integer nchan,nants,nanta,nwcorr
c
        complex data(MAXCHAN),wcorr(MAXWIDE)
        logical flags(MAXCHAN),wflags(MAXWIDE)
        double precision preamble(6)
        real jyperka(MAXANT)
c
        call output(version)
        call bug('i','This program is not needed anymore for CARMA ' //
     *    'data filled after 2007-01-31')
        call keyini
        call keya('vis',vis,' ')
        call keya('out',out,' ')
        call selInput('select',sels,MAXSELS)
        call mkeyr('jyperka',jyperka,MAXANT,nanta)
        call keyfin
c
c  Check the inputs.
c
        if(vis.eq.' ')call bug('f','An input must be given: vis=')
        if(out.eq.' ')call bug('f','An output must be given: out=')
c
c  Get ready to copy the data.
c
        call uvopen(lVis,vis,'old')
        call SelApply(lVis,sels,.true.)
        call uvset(lVis,'preamble','uvw/time/baseline',0,0.,0.,0.)
        call varInit(lVis,'channel')
        call varWinit(lVis)
c
        call uvvarIni(lVis,vnif)
c
c  Open the output, and make its history.
c
        call uvopen(lOut,out,'new')
        call varOnit(lVis,lOut,'channel')
        call uvset(lOut,'preamble','uvw/time/baseline',0,0.,0.,0.)
c
        call hdcopy(lVis,lOut,'history')
        call hisopen(lOut,'append')
        call hiswrite(lOut,'JYPERK: Miriad '//version)
        call hisinput(lOut,'JYPERK')
        call hisclose(lOut)
c
c  Get first record of the final pass.
c  warning: we're only getting jyperka() once here
c
        call uvread(lVis,preamble,data,flags,MAXCHAN,nchan)
        if(nchan.eq.0)call bug('f','No data found')
        call uvwread(lVis,wcorr,wflags,MAXWIDE,nwcorr)
        call uvrdvri(lVis,'nants',nants,0)
        if (nanta.eq.0) then
           call uvgetvrr(lVis,'jyperka',jyperka,nants)
           DO ant1=1,nants
              write(*,*) 'jyperka ',ant1,jyperka(ant1)
           ENDDO
        else if (nanta.lt.nants) then
           call bug('f','Not enough values for jyperka array')
        endif
c
        dowhile(nchan.gt.0)
          call varCopy(lVis,lOut)
          call basant(preamble(5),ant1,ant2,.TRUE.)
          jyperk = sqrt(jyperka(ant1)*jyperka(ant2))
          call uvputvrr(lOut,'jyperk',jyperk,1)
          call uvwwrite(lOut,wcorr,wflags,nwcorr)
          call uvwrite(lOut,preamble,data,flags,nchan)
          call uvread(lVis,preamble,data,flags,MAXCHAN,nchan)
          call uvwread(lVis,wcorr,wflags,MAXWIDE,nwcorr)
        enddo
c
        call uvclose(lVis)
        call uvclose(lOut)
        end
c************************************************************************
