c************************************************************************
      PROGRAM vel2fel
c
c= vel2fel - Change the value of or add a single header item
c& jhz
c: utility
c+
c   VEL2FEL is a MIRIAD task to convert the velocity
c   convention from radio to optical in an image header.
c@ in
c   The name of a data set.
c--
c  History:
c    jhz   24aug06  Original version.
c
c-----------------------------------------------------------------------
        character PVERSION*(*)
        parameter(PVERSION='Version 1.0 24-aug-06')
c >>> ../../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../../src/inc/mirconst.h
        integer lin,iostat, iaxis
        character in*80,item*32
        character vtype*72,iitem
        double precision fel1,dfel
c
c  Externals.
c
        integer len1
c-----------------------------------------------------------------------
        call output( 'VEL2FEL: '//PVERSION )
c
        call keyini
        call keya('in',in,' ')
        call keyfin
        if(in.eq.' ')call bug('f','Input data-set name is missing')
        call hopen(lin,in,'old',iostat)
        if(iostat.ne.0)then
           call bug('i','Error opening dataset '//in(1:len1(in)))
           call bugno('f',iostat)
        endif
c
        call getFelInfo(lin,in,fel1,dfel,vtype,iaxis)
c
              if(iaxis.eq.1) iitem='1'
              if(iaxis.eq.2) iitem='2'
              if(iaxis.eq.3) iitem='3'
              if(iaxis.eq.4) iitem='4'
              if(iaxis.eq.5) iitem='5'
c
            item = 'ctype'//iitem(1:1)
            call wrhda(lIn,item,vtype)
            item = 'crval'//iitem(1:1)
            call wrhdd(lIn,item,fel1)
            item = 'cdelt'//iitem(1:1)
            call wrhdd(lIn,item,dfel)
c
c  History
c
        call hisopen(lin,'append')
        call hiswrite(lin,'VEL2FEL: Miriad PutHd: '//PVERSION)
        call hisinput(lin,'VEL2FEL')
        call hisclose(lin)
c
c  Close up and go home.
c
        call hclose(lin)
        end
c************************************************************************
        subroutine getFelInfo(tno,in,fel1,dfel,vtype,iaxis)
c
        integer tno
        character in*(*)
c
c  Give a summary about an image.
c------------------------------------------------------------------------
c >>> ../../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../../src/inc/mirconst.h
      double precision dval,restf,fel1,dfel
      real vobs
      integer ival,iaxis
      character line*80,vtype*72
       logical hdprsnt
c
c  Externals.
c
c
c  Dimension info.
c
      call rdhdi (tno, 'naxis', ival, 0)
      write (line,'(a,i2,a)')'This image has',ival,' axes.'
      if(hdprsnt(tno,'vobs')) call rdhdr(tno,'vobs',vobs,0.0)
c
      call rdhdd(tno,'restfreq',dval,0.d0)
      if(dval.gt.0)then
        write(line,'(a,f13.6,a)')
     *    'Rest frequency:         ',dval,' GHz'
           restf=dval
c
      call dovel2fel (tno,ival,restf,vobs,fel1,dfel,vtype,iaxis)
      end if
c
c
      end
c************************************************************************
      subroutine dovel2fel (tno,naxis,restf,vobs,fel1,dfel,vtype,iaxis)
c
      integer tno,naxis,iaxis
c
c     List axes
c-----------------------------------------------------------------------
c >>> ../../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../../src/inc/mirconst.h
      double precision crval,cdelt,restf
      real crpix,vobs
      integer n,i
      character aval*72,str*2,vtype*72
      character itoaf*2
      double precision fel1,dfel,fac
c
c restf:  rest frequency
c  obsf:  reference observing frequency (sky frequency)
c  fel1:  reference velocity in optical convention
c  dfel:  velocity width in optical convention
c  vtype: velocity type
c  iaxis: the ith axis corresponding to velocity
c  Externals.
c
      do i=1,naxis
        str = itoaf(i)
        call rdhda(tno, 'ctype'//str, aval, 'none')
        call rdhdi(tno,'naxis'//str,n,0)
        call rdhdd (tno, 'crval'//str, crval,0.0d0)
        call rdhdr (tno, 'crpix'//str, crpix,0.0)
        call rdhdd (tno, 'cdelt'//str, cdelt,0.0d0)
        if(aval(1:5).eq.'FELO-') then
        call bug('w','already in optical-convention.')
        stop
        end if
c
        if(aval(1:5).eq.'VELO-')then
        iaxis=i
c
c determine the conversion factor
c
        fac = DCMKS/(DCMKS-crval*1.0d3)
c determine the reference velocity in optical convention
        fel1 = crval*fac
c determine the reference frequency
        dfel = cdelt*fac*fac
        vtype=aval
        vtype(1:1) = 'F'
        end if
      enddo
      end
