      program  offpol
c
c= offpol -- Generate ATCA primary beam polarimetric response.
c& rjs
c: utility
c+
c       OFFPOL generates images of the primary beam response (both total
c       intensity and polarimetric) of the ATCA antennas.  It performs a
c       simple simulation of an observation.
c
c@ out
c       Output name template.  No default.
c@ imsize
c       Two values, the image size.  Second value defaults to first.
c       Default 255.
c@ cell
c       Two values, the image cell size in RA and Dec, in arcsec.
c       Second value defaults to first.  Each defaults separately to
c       twice the primary beam FWHM divided by imsize.
c@ ra
c       Right ascension of the field centre.  (Used solely for setting
c       the output header, it has no effect on the computation.)
c       Default 0.
c@ dec
c       Declination of the field centre.  Default -45 deg.
c@ harange
c       Hour angle range to simulate - the start and end hour angles,
c       and the simulation step size.  The default is to simulate a
c       snapshot at 0 hours.  The default step size is 0.1 hours
c       (6 min) which might be inadequate for sources with a declination
c       near -30 deg.
c@ freq
c       Frequency of interest (GHz).  The default is 1.384 GHz.
c@ options
c       Task enrichment parameters. Several can be given.
c         raw      Generate images of the XX,YY,XY and YX responses,
c                  rather than the Stokes responses.
c         subtract Subtract off the circularly symmetric portion of the
c                  primary beam response from I, XX and YY responses.
c
c$Id: offpol.for,v 1.3 2012/02/15 15:31:38 pteuben Exp $
c--
c  History:
c    Refer to the RCS log, v1.1 includes prior revision information.
c-----------------------------------------------------------------------
c >>> ../../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../../src/inc/maxdim.h
c >>> ../../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../../src/inc/mirconst.h
c
      logical    ROTATE
      parameter (ROTATE = .true.)
c
      real       CHIOFF
      parameter (CHIOFF = 0.25*PI)
c
c     Observatory latitude.
      double precision LAT
      parameter (LAT = -30d0*DD2R)
c
      logical   doraw, dosub, flag(MAXDIM)
      integer   coObj, i, ic, iha, ipol, j, jc, lout, nha, nx, ny,
     *          pbObj, stokes(8), toff(4)
      real      c2chi, chi, cutoff, Jones(2,2), maxrad, off(MAXDIM,4),
     *          pb, pbfwhm, psi, q, rad, s2chi, u, x, xx, xy, y, yx, yy
      double precision cdelt1, cdelt2, crpix1, crpix2, crval1, crval2,
     *          dha, freq, ha, ha0, ha1
      character stokId(8)*2, out*64, version*80
c
      external  len1, versan
      integer   len1
      character versan*80
c
      data stokes / 1,   2,   3,   4,   -5,   -6,   -7,   -8 /
      data stokId /'i', 'q', 'u', 'v', 'xx', 'yy', 'xy', 'yx'/
c-----------------------------------------------------------------------
      version = versan('offpol',
     *                 '$Revision: 1.3 $',
     *                 '$Date: 2012/02/15 15:31:38 $')
c
c     Get the inputs.
      call keyini
c
      call keya('out',out,' ')
      lout = len1(out)
      if (lout.eq.0) call bug('f','An output must be given')
c
      call keyi('imsize',nx,255)
      call keyi('imsize',ny,nx)
      if (nx.le.0 .or. MAXDIM.lt.nx) call bug('f','Invalid image size')
      if (ny.le.0 .or. MAXDIM.lt.ny) call bug('f','Invalid image size')
c
      call keyd('cell',cdelt1,0d0)
      call keyd('cell',cdelt2,cdelt1)
c
      call keyt('ra', crval1,'hms',0d0)
      call keyt('dec',crval2,'dms',-0.25d0*DPI)
c
      call keyt('harange',ha0,'hms',0d0)
      call keyt('harange',ha1,'hms',ha0)
      call keyt('harange',dha,'hms',0.1d0)
      nha = nint((ha1 - ha0)/dha) + 1
c
      call keyd('freq',freq,1.384d0)
c
      call GetOpt(doraw,dosub)
      call keyfin
c
      ic = nx/2 + 1
      jc = ny/2 + 1
      crpix1 = dble(ic)
      crpix2 = dble(jc)
c
c     Determine the FWHM of the primary beam at this frequency.
      call coCreate(3, coObj)
      call coAxSet(coObj, 1, 'RA---SIN', 0d0, 0d0, 1d0)
      call coAxSet(coObj, 2, 'DEC--SIN', 0d0, 0d0, 1d0)
      call coAxSet(coObj, 3, 'FREQ', 0d0, freq, 0.1d0*freq)
      call coReinit(coObj)
      call pbInit(pbObj,'atca',coObj)
      call pbInfo(pbObj,pbfwhm,cutoff,maxrad)
c
      if (cdelt1.eq.0d0) then
        cdelt1 = 2d0 * pbfwhm / real(nx)
      else
        cdelt1 = -abs(cdelt1 * DAS2R)
      endif
c
      if (cdelt2.eq.0d0) then
        cdelt2 = 2d0 * pbfwhm / real(ny)
      else
        cdelt2 =  abs(cdelt2 * DAS2R)
      endif
c
c     Open an output map for each polarization.
      do ipol = 1, 4
        i = ipol
        if (doraw) i = i + 4
c
        call mkopen(toff(ipol), nx, ny, crpix1, crpix2, cdelt1, cdelt2,
     *    crval1, crval2, freq, stokes(i),
     *    out(1:lout) // '.' // stokId(i), version)
      enddo
c
c     Loop over the map.
      do j = 1, ny
        y = (j - jc)*cdelt2
c
        do i = 1, nx
          do ipol = 1, 4
            off(i,ipol) = 0.0
          enddo
c
c         Blank the corners.
          x = (i - ic)*cdelt1
          flag(i) = sqrt(x*x + y*y).lt.pbfwhm
        enddo
c
        do iha = 1, nha
          ha = dha*(iha-1) + ha0
          call parang(0d0,crval2,ha,LAT,chi)
          chi = chi + CHIOFF
c
          if (ROTATE) then
            c2chi = cos(2.0*chi)
            s2chi = sin(2.0*chi)
          else
            c2chi = 1.0
            s2chi = 0.0
          endif
c
          do i = 1, nx
            if (i.eq.ic .and. j.eq.jc) then
              xx = 1.0
              yy = 1.0
              xy = 0.0
              yx = 0.0
              pb = 1.0
            else
c             Compute the Jones matrix at this point.
              x = (i - ic)*cdelt1
              rad = sqrt(x*x + y*y)
              psi = atan2(x,y)
              call atjones(rad,psi-chi,freq,Jones,pb)
c
c             Coherence matrix.
              xx = Jones(1,1)*Jones(1,1) + Jones(1,2)*Jones(1,2)
              yy = Jones(2,1)*Jones(2,1) + Jones(2,2)*Jones(2,2)
              xy = Jones(1,1)*Jones(2,1) + Jones(1,2)*Jones(2,2)
              yx = xy
            endif
c
c           Subtract the primary beam response?
            if (dosub) then
              xx = xx - pb
              yy = yy - pb
            endif
c
            if (doraw) then
c             Generate images of the XX, YY, XY, and YX responses.
              off(i,1) = off(i,1) + xx
              off(i,2) = off(i,2) + yy
              off(i,3) = off(i,3) + xy
              off(i,4) = off(i,4) + yx
c
            else
c             Stokes-I.
              off(i,1) = off(i,1) + 0.5*(xx + yy)
c
c             Stokes-Q, and -U.
              q = 0.5*(xx - yy)
              u = 0.5*(xy + yx)
              off(i,2) = off(i,2) + q*c2chi - u*s2chi
              off(i,3) = off(i,3) + q*s2chi + u*c2chi
c
c             Stokes-V (zero, because xy and yx are real).
c             off(i,4) = off(i,4) + 0.5*real((0.0,-1.0)*(xy-yx))
            endif
          enddo
        enddo
c
        do ipol = 1, 4
          do i = 1, nx
            off(i,ipol) = off(i,ipol) / real(nha)
          enddo
c
          call xywrite(toff(ipol), j, off(1,ipol))
          call xyflgwr(toff(ipol), j, flag)
        enddo
      enddo
c
      do ipol = 1, 4
        call xyclose(toff(ipol))
      enddo
c
      end
c***********************************************************************
      subroutine GetOpt(doraw,dosub)
c
      logical doraw, dosub
c-----------------------------------------------------------------------
      integer    NOPTS
      parameter (NOPTS=2)
c
      character opts(NOPTS)*8
      logical   present(NOPTS)
c
      data opts/'raw     ','subtract'/
c-----------------------------------------------------------------------
      call options('options',opts,present,NOPTS)
      doraw = present(1)
      dosub = present(2)
c
      end
c***********************************************************************
      subroutine mkopen(tno, nx, ny, crpix1, crpix2, cdelt1, cdelt2,
     *  crval1, crval2, sfreq, stokes, name, version)
c
      integer   tno, nx, ny
      double precision crpix1, crpix2, cdelt1, cdelt2, crval1, crval2,
     *          sfreq
      integer   stokes
      character name*(*),version*(*)
c-----------------------------------------------------------------------
      integer nsize(4),coObj
      character line*64
c-----------------------------------------------------------------------
      call coCreate(4, coObj)
      call coAxSet(coObj,1,'RA---SIN',crpix1,crval1,cdelt1)
      call coAxSet(coObj,2,'DEC--SIN',crpix2,crval2,cdelt2)
      call coAxSet(coObj,3,'FREQ',    1d0,sfreq,0.1d0)
      call coAxSet(coObj,4,'STOKES',  1d0,dble(stokes),1d0)
      call coReInit(coObj)
c
      nsize(1) = nx
      nsize(2) = ny
      nsize(3) = 1
      nsize(4) = 1
c
      call xyopen(tno,name,'new',4,nsize)
      call hisopen(tno,'write')
      line = 'OFFPOL: Miriad '//version
      call hiswrite(tno,line)
      call hisinput(tno,'OFFPOL')
      call hisclose(tno)
      call coWrite(coObj,tno)
      call coFin(coObj)
      call wrhda(tno,'telescop','ATCA')
c
      end
