c-----------------------------------------------------------------------
c* Axistype - Find the axis label and plane value in user fiendly units
c& mchw
c: plotting
c+
      subroutine axisType(lIn,axis,plane,ctype,label,value,valstr)
c
      integer   lIn, axis, plane
      character ctype*(*), label*13
      double precision value
      character valstr*13
c  ---------------------------------------------------------------------
c  Find the axis label and plane value in user friendly string.
c
c  Input:
c    lIn        The handle of the image.
c    axis       The image axis.
c    plane      The image plane along this axis.
c
c  Output:
c    ctype      The official ctype for the input axis.
c    label      A nice label for this axis.
c    value      The value at the plane along this axis.
c    valstr     value as a formatted string.
c--
c $Id: axistype.for,v 1.3 2012/05/11 04:46:42 pteuben Exp $
c-----------------------------------------------------------------------
c >>> ../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../src/inc/mirconst.h
c
      integer   k, naxis
      character axtype*16, cax*2, units*8, wtype*16
c
      external  angles, itoaf, len1
      integer   len1
      character angles*13, itoaf*2
c-----------------------------------------------------------------------
c     Get coordinate type and value for input plane.
      cax = itoaf(axis)
      call rdhdi(lIn, 'naxis', naxis, 1)
      if (axis.le.naxis) then
        call coInit(lIn)
        call coCvt1(lIn, axis, 'ap', dble(plane), 'aw', value)
        call coGetA(lIn, 'ctype'//cax, ctype)
        call coAxType(lIn, axis, axtype, wtype, units)
c
c       Convert to user friendly label and formatted value.
        k = (13 - len1(wtype))/2 + 1
        if (wtype.eq.'RA') then
          label(k:) = wtype
          valstr = angles(value*DR2D/15d0)
c
        else if (units.eq.'rad') then
          label(k:) = wtype
          valstr = angles(value*DR2D)
c
        else if (units.eq.'km/s') then
          label(k:) = wtype
          write(valstr,'(g13.3)') value
c
        else if (units.eq.'GHz') then
          label = '  Frequency  '
          write(valstr,'(g13.6)') value
c
        else
          label(k:) = wtype(:13)
          write(valstr,'(g13.6)') value
        endif
c
      else
        value  = 0d0
        label  = 'no axis '//cax
        valstr = ' '
      endif
c
      end
