c************************************************************************
c* GetXY -- Return information on the XY phase of a dataset.
c& rjs
c: utilities
c+
        subroutine GetXY(tno,fac,xyphase,count,mxant,nants)
c
        integer tno,mxant,nants,count(mxant)
        real xyphase(mxant),fac
c
c  This determines the average XY phase, and its standard deviation, of
c  the gains of a visibility data-set. The information is derived
c  from the "gains" item.
c
c  Input:
c    tno	Handle of the input visibility data-set.
c    mxant	Max number of antennae.
c  Output:
c    fac        RMS gain amplitude.
c    xyphase	Mean xyphase, in radians.
c    count	Number of gains used in the average. This can be 0!
c    nants	Number of antenna found. This will be zero of the xyphase
c		could not be determined at all.
c--
c  History:
c    rjs  ???????	Original version
c    rjs  01may09	Fix bug to return RMS gain amplitude.
c------------------------------------------------------------------------
c >>> ../../src/inc/maxdim.h
c src/inc/cfg_maxdim.h.  Generated from maxdim.h.in by configure.
c TELESCOPE=carma
c=======================================================================
        CHARACTER*(*) MIRTEL
        PARAMETER(MIRTEL='carma')
        INTEGER   MAXBUF
        PARAMETER(MAXBUF=10000000)
c
        INTEGER   MAXDIM, MAXDIM2
        PARAMETER(MAXDIM=65536,MAXDIM2=8192)
c
        INTEGER   MAXANT,MAXANT2
        PARAMETER(MAXANT=64,MAXANT2=28)
c		maximum number of baselines
        INTEGER   MAXBASE
        PARAMETER(MAXBASE=1058)
        INTEGER   MAXBASE2
        PARAMETER(MAXBASE2=1058)
c
        INTEGER   MAXCHAN
        PARAMETER(MAXCHAN=8192)
c
        INTEGER   MAXWIN
        PARAMETER(MAXWIN=32)
c
        INTEGER   MAXWIDE
        PARAMETER(MAXWIDE=32)
c
        INTEGER   MAXFBIN
        PARAMETER(MAXFBIN=32)
c=======================================================================
c
c <<< ../../src/inc/maxdim.h
c >>> ../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../src/inc/mirconst.h
        integer item,iostat,ngains,nfeeds,ntau,nsols,i,j,k,off,ntot
        real theta,sumsq
        complex gx,gy,gains(3*MAXANT)
c
c  Initialise the counters.
c
        fac = 0
        sumsq = 0
        ntot = 0
        do i=1,mxant
          count(i) = 0
          xyphase(i) = 0
        enddo
c
c  Determine the number of gains, antennae, etc.
c
        call rdhdi(tno,'ngains',ngains,0)
        call rdhdi(tno,'nfeeds',nfeeds,1)
        call rdhdi(tno,'ntau',  ntau,  0)
c
        nants = 0
        if(nfeeds.ne.2) return
        call haccess(tno,item,'gains','read',iostat)
        if(iostat.ne.0) return
c
        nants = ngains / (nfeeds + ntau)
        if(nants.le.0) call bug('f','Bad number of antennae gains')
        if(nants.gt.MAXANT)
     *    call bug('f','Too many antennae to handle')
        call rdhdi(tno,'nsols',nsols,0)
c
c  Read through the gains, compute the XY phase at each antenna, and
c  accumulate it.
c
        do j=1,nsols
          off = 8*(ngains+1)*(j-1) + 16
          call hreadr(item,Gains,off,8*ngains,iostat)
          if(iostat.ne.0)then
            call bug('w','Error while reading the gains table')
            call bugno('f',iostat)
          endif
          k = 1
          do i=1,min(mxant,nants)
            gx = gains(k)
            gy = gains(k+1)
            if(abs(real(gx))+abs(aimag(gx)).gt.0.and.
     *         abs(real(gy))+abs(aimag(gy)).gt.0)then
              sumsq = sumsq + real(gx)**2 + aimag(gx)**2 +
     *                        real(gy)**2 + aimag(gy)**2
              ntot = ntot + 2
              gx = gx / gy
              theta = atan2(aimag(gx),real(gx))
              if(count(i).gt.0)theta = theta +
     *            2*pi*nint((xyphase(i)/count(i) - theta)/(2*pi))
              count(i) = count(i) + 1
              xyphase(i) = xyphase(i) + theta
            endif
            k = k + nfeeds + ntau
          enddo
        enddo
c
        call hdaccess(item,iostat)
c
c  We have the needed info. Now work out the mean and standard deviation.
c
        do i=1,mxant
          if(count(i).gt.0)then
            theta = xyphase(i) / count(i)
            xyphase(i) = theta
          endif
        enddo
        if(sumsq.gt.0)fac = sqrt(ntot/sumsq)
c
        end
