
/* System dependent initialisation. */

#define FORTRAN_CVT_INT 0
#define FORTRAN_CVT_LOG 0
typedef int fort_integer;
typedef int fort_logical;
typedef long fort_ptrdiff;
typedef float fort_real;
typedef double fort_double;
char *zterm(char *string,int length);
void pad(char *string,int length);
int *iface_iarr(int n);
void iface_f2c_icvt(fort_integer *in,int *out,int n);
void iface_c2f_icvt(int *in,fort_integer *out,int n);
int *iface_larr(int n);
void iface_f2c_lcvt(fort_logical *in,int *out,int n);
void iface_c2f_lcvt(int *in,fort_logical *out,int n);
/************************************************************************/
/*									*/
/*	This converts from a FORTRAN calling sequence into a C.		*/
/*									*/
/*  History:								*/
/*    rjs  16jul09  Original version. Adapted from wrap.f2c.		*/
/************************************************************************/

#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
#include "config.h"
#endif

#include <limits.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include "miriad.h"
#include "sysdep.h"

/************************************************************************/
void hsize3_(size1,item)
fort_integer *size1;
fort_integer *item;
{
  off_t size;
  fort_integer *s;
  off_t a,b,x;

  size = hsize_c(*item);
  x = (off_t)INT_MAX/2;
  b = size/x;
  a = size - b*x;
  s = size1;
  *s++ = a; *s++ = b; *s++ = x;
}
/************************************************************************/
void hread3b_(item,buffer,offset,length,iostat,buffer_len)
fort_integer *item;
char *buffer;
int buffer_len;
fort_integer *offset;
fort_integer *length;
fort_integer *iostat;
{
  int temp;
  fort_integer *s;
  off_t a,b,x,off;

  s = offset;
  a = *s++; b = *s++; x = *s++;
  off = a + b*x;
  hreadb_c(*item,buffer,off,*length,&temp);
  *(iostat) = temp;
}
/************************************************************************/
void hwrite3b_(item,buffer,offset,length,iostat,buffer_len)
fort_integer *item;
char *buffer;
int buffer_len;
fort_integer *offset;
fort_integer *length;
fort_integer *iostat;
{
  int temp;
  fort_integer *s;
  off_t a,b,x,off;

  s = offset;
  a = *s++; b = *s++; x = *s++;
  off = a + b*x;
  hwriteb_c(*item,buffer,off,*length,&temp);
  *(iostat) = temp;
}
/************************************************************************/
void hread3j_(item,buffer,offset,length,iostat)
fort_integer *item;
fort_integer *buffer;
fort_integer *offset;
fort_integer *length;
fort_integer *iostat;
{
#if FORTRAN_CVT_INT
  int temp,*tbuffer;
#endif
  fort_integer *s;
  off_t a,b,x,off;

  s = offset;
  a = *s++; b = *s++; x = *s++;
  off = a + b*x;
#if FORTRAN_CVT_INT
  tbuffer = iface_iarr(*length);
  hreadj_c(*item,tbuffer,off,*length,&temp);
  iface_c2f_icvt(tbuffer,buffer,*length);
  *(iostat) = temp;
#else
  hreadj_c(*item,buffer,off,*length,iostat);
#endif
}
/************************************************************************/
void hwrite3j_(item,buffer,offset,length,iostat)
fort_integer *item;
fort_integer *buffer;
fort_integer *offset;
fort_integer *length;
fort_integer *iostat;
{
  int temp,*tbuffer;
  fort_integer *s;
  off_t a,b,x,off;

  s = offset;
  a = *s++; b = *s++; x = *s++;
  off = a + b*x;

#if FORTRAN_CVT_INT
  tbuffer = iface_iarr(*length);
  iface_f2c_icvt(buffer,tbuffer,*length);
#else
  tbuffer = buffer;
#endif
  hwritej_c(*item,tbuffer,off,*length,&temp);
  *(iostat) = temp;
}
/************************************************************************/
void hread3i_(item,buffer,offset,length,iostat)
fort_integer *item;
fort_integer *buffer;
fort_integer *offset;
fort_integer *length;
fort_integer *iostat;
{
#if FORTRAN_CVT_INT
  int temp,*tbuffer;
#endif
  fort_integer *s;
  off_t a,b,x,off;

  s = offset;
  a = *s++; b = *s++; x = *s++;
  off = a + b*x;

#if FORTRAN_CVT_INT
  tbuffer = iface_iarr(*length);
  hreadi_c(*item,tbuffer,off,*length,&temp);
  iface_c2f_icvt(tbuffer,buffer,*length);
  *(iostat) = temp;
#else
  hreadi_c(*item,buffer,off,*length,iostat);
#endif
}
/************************************************************************/
void hwrite3i_(item,buffer,offset,length,iostat)
fort_integer *item;
fort_integer *buffer;
fort_integer *offset;
fort_integer *length;
fort_integer *iostat;
{
  int temp,*tbuffer;
  fort_integer *s;
  off_t a,b,x,off;

  s = offset;
  a = *s++; b = *s++; x = *s++;
  off = a + b*x;

#if FORTRAN_CVT_INT
  tbuffer = iface_iarr(*length);
  iface_f2c_icvt(buffer,tbuffer,*length);
#else
  tbuffer = buffer;
#endif
  hwritei_c(*item,tbuffer,off,*length,&temp);
  *(iostat) = temp;
}
/************************************************************************/
void hread3r_(item,buffer,offset,length,iostat)
fort_integer *item;
fort_real *buffer;
fort_integer *offset;
fort_integer *length;
fort_integer *iostat;
{
  int temp;
  fort_integer *s;
  off_t a,b,x,off;

  s = offset;
  a = *s++; b = *s++; x = *s++;
  off = a + b*x;

  hreadr_c(*item,buffer,off,*length,&temp);
  *(iostat) = temp;
}
/************************************************************************/
void hwrite3r_(item,buffer,offset,length,iostat)
fort_integer *item;
fort_real *buffer;
fort_integer *offset;
fort_integer *length;
fort_integer *iostat;
{
  int temp;
  fort_integer *s;
  off_t a,b,x,off;

  s = offset;
  a = *s++; b = *s++; x = *s++;
  off = a + b*x;

  hwriter_c(*item,buffer,off,*length,&temp);
  *(iostat) = temp;
}
/************************************************************************/
void hread3d_(item,buffer,offset,length,iostat)
fort_integer *item;
fort_double *buffer;
fort_integer *offset;
fort_integer *length;
fort_integer *iostat;
{
  int temp;
  fort_integer *s;
  off_t a,b,x,off;

  s = offset;
  a = *s++; b = *s++; x = *s++;
  off = a + b*x;

  hreadd_c(*item,buffer,off,*length,&temp);
  *(iostat) = temp;
}
/************************************************************************/
void hwrite3d_(item,buffer,offset,length,iostat)
fort_integer *item;
fort_double *buffer;
fort_integer *offset;
fort_integer *length;
fort_integer *iostat;
{
  int temp;
  fort_integer *s;
  off_t a,b,x,off;

  s = offset;
  a = *s++; b = *s++; x = *s++;
  off = a + b*x;

  hwrited_c(*item,buffer,off,*length,&temp);
  *(iostat) = temp;
}
