c************************************************************************
c  History:
c    rjs   ???89
c    nebk  10sep89  Install missing ENDIF.
c    rjs   24aug01  Added jinc and chat functions.
c    dpr   27aug01  Declare your variables rjs!
c************************************************************************
c*J1xByx -- Calculate j1(x)/x
c:special-functions,bessel-functions
c*
        real function j1xbyx(arg)
c
        real arg
c
c   J1xByx calculates the bessel function,  j1(x)/x  from the
c   polynomial approximations. See Abramowitz and Stegun, Handbook of
c   Mathematical Functions, sections 9.4.4 and 9.4.6, page 370.
c
c  Input:
c    arg	The value of x.
c  Output:
c    j1xbyx	The calculated value of j1(x)/x
c--
c------------------------------------------------------------------c
      real f,x,t
c
c  For the range -3.le.arg.le.+3, the maximum error is 1.3 e-8.
c
      if(abs(arg).lt.3)then
        X = (ARG/3.)*(ARG/3.)
        J1XBYX = 0.5 + X*(      - 0.56249985
     *               + X*(      + 0.21093573
     *               + X*(      - 0.03954289
     *               + X*(      + 0.00443319
     *               + X*(      - 0.00031761
     *               + X*(      + 0.00001109))))))
c
c   for the range  abs(arg).gt.3.  , max. error is 1. e-7
c
      else
        X = 3./ABS(ARG)
        F = .79788456   + X*( +.00000156
     *                  + X*( +.01659667
     *                  + X*( +.00017105
     *                  + X*( -.00249511
     *                  + X*( +.00113653
     *                  + X*( -.00020033 ))))))
        T = ARG - 2.35619449
     *                  + X*( .12499612
     *                  + X*( .00005650
     *                  + X*(-.00637879
     *                  + X*( .00074348
     *                  + X*( .00079824
     *                  + X*(-.00029166 ))))))
        J1XBYX = F*COS(T)/ARG/SQRT(ARG)
      end if
c
      END
c************************************************************************
        real function jinc(x)
c
        real x
c
c  Bracewell's jinc function = J1(pi*x)/(2*x)
c
c------------------------------------------------------------------------
c >>> ../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../src/inc/mirconst.h
        real j1xbyx
c
        jinc = 0.5*PI*j1xbyx(PI*x)
        end
c************************************************************************
        real function chat(x)
c
        real x
c
c  Polynomial approximation to Bracewell's Chinese hat - chat - function.
c
c  chat(x) = 0.5*(acos(x)-abs(x)*sqrt(1-x*x))
c
c  This is the autocorrelation of uniform disks
c
c------------------------------------------------------------------------
        real t
        t = abs(x)
        if(t.gt.1)then
          chat = 0
        else
          chat = 0.78639 + t * (-1.02669
     *                   + t * ( 0.15909
     *                   + t * (-0.16691
     *                   + t *   0.24491 )))
        endif
        end
