
/* System dependent initialisation. */

#define FORTRAN_CVT_INT 0
#define FORTRAN_CVT_LOG 0
typedef int fort_integer;
typedef int fort_logical;
typedef long fort_ptrdiff;
typedef float fort_real;
typedef double fort_double;
char *zterm(char *string,int length);
void pad(char *string,int length);
int *iface_iarr(int n);
void iface_f2c_icvt(fort_integer *in,int *out,int n);
void iface_c2f_icvt(int *in,fort_integer *out,int n);
int *iface_larr(int n);
void iface_f2c_lcvt(fort_logical *in,int *out,int n);
void iface_c2f_lcvt(int *in,fort_logical *out,int n);
/***********************************************************************/
/*
 *  The key routines provide keyword-oriented access to the command line.
 *
 *  History:
 *    rjs   Dark-ages Original version.
 *    bs    ???88     Converted it to use iargc and getarg. Added -f flag.
 *    rjs   8sep89    Improved documentation.
 *    nebk  10sep89   Added mkeyr.  I think rjs will not like it (Too right!).
 *    rjs   19oct89   Major rewrite to handle @ files.
 *    rjs   15nov89   Added keyf routine, and did the rework needed to support
 *		      this. Added mkeyf. Modified mkeyr.
 *    pjt   26mar90   Added mkeya. like mkeyr (again, bobs will not like this)
 *    pjt   10apr90   some more verbose bug calls.
 *    rjs   23apr90   Made pjt's last changes into standard FORTRAN (so the
 *		      Cray will accept it).
 *    pjt   10may90   Make it remember the programname in keyini (se key.h)
 *                    for bug calls - introduced progname
 *    rjs   22oct90   Check for buffer overflow in keyini.
 *    pjt   21jan91   Added mkeyi, variable index is now idx, exp is expd
 *    pjt    5mar91   atod -> atodf
 *    pjt   18mar91   increased arg buffer-length in keyini 80->256
 *                    (working on -h flag; bug verbosities)
 *    pjt   25jun91   allow .TRUE. and .FALSE. as logicals too (keyl)
 *    nebk  14oct91   Provide more space in PBUF for contents of @files
 *                    in KEYREAD
 *    pjt    1feb92   Added -k flag to display keywords
 *    pjt   13jul92   Provide more space in keyini: 256 char / argument
 *    rjs    2sep92   Correctly handle blanks at the end of line in @ files.
 *		      Add -? flag.
 *    nebk  23nov92   Add mkeyd.  rjs spits dummy.
 *    rjs   04dec92   Rewrite keyi, so that mchw's new hex/octal conversion
 *		      routine is used.
 *    rjs   04jan93   Eliminate progname, and call buglabel instead. General
 *		      tidying.
 *     jm   11jan93   Modified keyini to strip off the leading path
 *		      information of the application name so the command
 *		      line arguments work correctly.  Modified keyput to
 *		      pass in the task name and to handle local variable
 *		      names.  Added local integer function keyindex to
 *		      find the index of the entry associated with the
 *		      input key word.  Modified keyput, keyget, and
 *		      keyprsnt to use the function keyindex.
 *     jm   02nov94   Stripped away almost every routine from the
 *                    Fortran file to create this one.  This collection
 *                    of wrappers permit Fortran calls to the C
 *                    routines (key.c).  The KeyIni and MKey[AF]
 *                    routines still remain in Fortran since they can
 *                    not be easily switched to C.  Note also that the
 *                    calling syntax of keyini differs from keyini_c.
 *     jm   17nov94   Added forward declartion prototypes for each key
 *                    function called.
 *     jm   27dec94   Added new function keyinit().
 *     pjt  12jun10   Added keyputc for ATNF compatibility
 */

#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
#include "config.h"
#endif

#include "sysdep.h" /* Include necessary for definition of ARGS macro. */

/***********************************************************************/
void keyinit_(task,task_len)
char *task;
int task_len;
{
  extern void keyinit_c ARGS((Const char *));
  keyinit_c(zterm(task,task_len));
}
/***********************************************************************/
void keyputc_(arg,arg_len)
char *arg;
int arg_len;
{
  extern void keyputc_c ARGS((char *));
  keyputc_c(zterm(arg,arg_len));
}
/***********************************************************************/
void keyput_(task,arg,task_len,arg_len)
char *task;
int task_len;
char *arg;
int arg_len;
{
  extern void keyput_c ARGS((Const char *, char *));
  keyput_c(zterm(task,task_len), zterm(arg,arg_len));
}
/***********************************************************************/
void keyfin_()
{
  extern void keyfin_c ARGS((void));
  keyfin_c();
}
/***********************************************************************/
fort_logical keyprsnt_(key,key_len)
char *key;
int key_len;
{
  extern int keyprsnt_c ARGS((Const char *));
  return(keyprsnt_c(zterm(key,key_len)));
}
/***********************************************************************/
void keya_(key,value,defval,key_len,value_len,defval_len)
char *key;
int key_len;
char *value;
int value_len;
char *defval;
int defval_len;
{
  extern void keya_len_c ARGS((Const char *, char *, size_t, Const char *));
  keya_len_c(zterm(key,key_len), value, value_len, zterm(defval,defval_len));
  pad(value,value_len);
}
/***********************************************************************/
void keyf_(key,value,defval,key_len,value_len,defval_len)
char *key;
int key_len;
char *value;
int value_len;
char *defval;
int defval_len;
{
  extern void keyf_c ARGS((Const char *, char *, Const char *));
  keyf_c(zterm(key,key_len), value, zterm(defval,defval_len));
  pad(value,value_len);
}
/***********************************************************************/
void keyd_(key,value,defval,key_len)
char *key;
int key_len;
fort_double *value;
fort_double *defval;
{
  extern void keyd_c ARGS((Const char *, double *, double));
  keyd_c(zterm(key,key_len), value, *defval);
}
/***********************************************************************/
void keyr_(key,value,defval,key_len)
char *key;
int key_len;
fort_real *value;
fort_real *defval;
{
  extern void keyr_c ARGS((Const char *, float *, float));
  keyr_c(zterm(key,key_len), value, *defval);
}
/***********************************************************************/
void keyi_(key,value,defval,key_len)
char *key;
int key_len;
fort_integer *value;
fort_integer *defval;
{
  extern void keyi_c ARGS((Const char *, int *, int));
  keyi_c(zterm(key,key_len), value, *defval);
}
/***********************************************************************/
void keyl_(key,value,defval,key_len)
char *key;
int key_len;
fort_logical *value;
fort_logical *defval;
{
  extern void keyl_c ARGS((Const char *, int *, int));
  keyl_c(zterm(key,key_len), value, *defval);
}
/***********************************************************************/
void mkeyd_(key,value,nmax,n,key_len)
char *key;
int key_len;
fort_double *value;
fort_integer *nmax;
fort_integer *n;
{
  extern void mkeyd_c ARGS((Const char *, double [], int, int *));
  mkeyd_c(zterm(key,key_len), value, *nmax, n);
}
/***********************************************************************/
void mkeyr_(key,value,nmax,n,key_len)
char *key;
int key_len;
fort_real *value;
fort_integer *nmax;
fort_integer *n;
{
  extern void mkeyr_c ARGS((Const char *, float [], int, int *));
  mkeyr_c(zterm(key,key_len), value, *nmax, n);
}
/***********************************************************************/
void mkeyi_(key,value,nmax,n,key_len)
char *key;
int key_len;
fort_integer *value;
fort_integer *nmax;
fort_integer *n;
{
  extern void mkeyi_c ARGS((Const char *, int [], int, int *));
  mkeyi_c(zterm(key,key_len), value, *nmax, n);
}
