
/* System dependent initialisation. */

#define FORTRAN_CVT_INT 0
#define FORTRAN_CVT_LOG 0
typedef int fort_integer;
typedef int fort_logical;
typedef long fort_ptrdiff;
typedef float fort_real;
typedef double fort_double;
char *zterm(char *string,int length);
void pad(char *string,int length);
int *iface_iarr(int n);
void iface_f2c_icvt(fort_integer *in,int *out,int n);
void iface_c2f_icvt(int *in,fort_integer *out,int n);
int *iface_larr(int n);
void iface_f2c_lcvt(fort_logical *in,int *out,int n);
void iface_c2f_lcvt(int *in,fort_logical *out,int n);
/************************************************************************/
/*									*/
/* Routines to access socket facilities from FORTRAN. The interface	*/
/* is socket-like. Most (all?) the routines are trivial.		*/
/*									*/
/* History:								*/
/*  rjs  Dark-ages Original version.					*/
/*  rjs  25jan90   Changed "addr" to "adr" to avoid a conflict on Convex*/
/*  rjs  23dec92   Rewrite in f2c format, mere all versions.		*/
/*  mjs  21sep93   #define bcopy -> memcpy, bzero -> memset on sun.	*/
/*  pjt  20jun02   pedantic headers                                     */
/*									*/
/************************************************************************/

#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
#include "config.h"
#endif

#include <stddef.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>

#ifdef sun
#define bcopy(a,b,c) memcpy((b),(a),(c))
#define bzero(a,b)   memset((a),0,(b))
#endif
#ifdef vms
extern int vmserrno;
#  define ERRNO vmserrno
#else
#  define ERRNO errno
#endif

/************************************************************************/
void tcpnode_(name,inet,name_len)
char *name;
int name_len;
fort_integer *inet;
/*

  This returns the internet address of a host. If the host is not found, 
  a zero address is returned.

  Input:
    name	Name of host.

  Output:
    inet	Inet address of host. If the host was not found, this
		returns 0. Bytes are in network (not host!) order.

------------------------------------------------------------------------*/
{
  char *p;
  struct hostent *h;
  unsigned long int s;

/* It could be a dot-notation address or a name. Handle it right. */

  p = zterm(name,name_len);
  *(inet) = 0;
  if(isdigit(*p)){
    s = inet_addr(p);
    bcopy((char *)&s,(char *)inet,sizeof(s));
  } else {
    h = gethostbyname(p);
    if(h != NULL)bcopy(h->h_addr,(char *)inet,h->h_length);
#ifdef unicos
    *(inet) >>= 32;
#endif
  }
}
/************************************************************************/
fort_integer tcpsock_(handle)
fort_integer *handle;
/*
  Perform a "socket" call.
------------------------------------------------------------------------*/
{
  *(handle) = socket(AF_INET,SOCK_STREAM,0);
  return(*handle < 0 ? ERRNO : 0);
}
/************************************************************************/
fort_integer tcpconn_(handle,inet,port)
fort_integer *handle;
fort_integer *inet;
fort_integer *port;
/*
  Perform a "connect" call.
------------------------------------------------------------------------*/
{
   struct sockaddr_in sin;

   sin.sin_family = AF_INET;
   sin.sin_addr.s_addr = *(inet);
   sin.sin_port = htons(*port);
   bzero((char *)sin.sin_zero,sizeof(sin.sin_zero));
   if(connect(*handle,(struct sockaddr *)&sin,sizeof(sin)) < 0)return(ERRNO);
   return(0);
}
/************************************************************************/
fort_integer tcpclose_(handle)
fort_integer *handle;
/*
  Close down a socket.
------------------------------------------------------------------------*/
{
  return(close(*handle) ? 0 : ERRNO);
}
/************************************************************************/
fort_integer tcpwrite_(handle,buff,length)
fort_integer *handle;
fort_integer *buff;
fort_integer *length;
/*
  Write some crap out to the socket.
------------------------------------------------------------------------*/
{
  return(*(length) == write(*handle,(char *)buff,*length)
	? 0 : ERRNO);
}
/************************************************************************/
fort_integer tcpread_(handle,buff,length,nread)
fort_integer *handle;
fort_integer *buff;
fort_integer *length;
fort_integer *nread;
/*
  Read some crap from the socket. 
------------------------------------------------------------------------*/
{
  *(nread) = read(*handle,(char *)buff,*length);
  return(*nread >= 0 ? 0 : ERRNO);
}
