c***********************************************************************
c* Title -  Write title in standard format into LogFile.
c& mchw
c: image analysis,log-file
c+
      subroutine Title(lIn,naxis,blc,trc,cbof)
c
      integer lIn,naxis,blc(naxis),trc(naxis)
c  ---------------------------------------------------------------------
c  Write title in standard format into LogFile.
c
c  Inputs:
c    lIn        The handle of the Image.
c    naxis      Number of axes of the Image.
c    blc,trc    Corners of region of interest.
c  Output:
c    cbof       Beam oversampling factor.
c-----------------------------------------------------------------------
c >>> ../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../src/inc/mirconst.h
c
      double precision RTS
      parameter (RTS=3600d0*180d0/PI)
c
      logical   more
      integer   lblc, ltrc
      real      bmaj, bmin, cbof, DperJy, freq, omega
      character bunit*20, ctype1*9, ctype2*9, ctype3*9, line*80,
     *          txtblc*20, txttrc*20
c
      external  len1
      integer   len1
c-----------------------------------------------------------------------
c     Get beam size etc.
      call GetBeam(lIn,naxis,bunit,bmaj,bmin,omega,cbof)
      call rdhdr(lIn,'restfreq',freq,0.0)
      if (freq.eq.0.0) then
        call rdhda(lIn,'ctype3',ctype3,' ')
        if (ctype3(1:4).eq.'FREQ') then
          call rdhdr(lIn,'crval3',freq,0.0)
        endif
      endif
c
      DperJy=1.0
      if (freq.ne.0.0) then
        DperJy= (0.3/freq)**2 /(2.0*1.38e3*omega)
      endif
c
      write(line,'(a,a,a,f10.6,a,f15.4)')
     *      '  bunit: ', bunit(1:len1(bunit)),
     *      '  frequency: ', freq,
     *      '    K/Jy: ', DperJy
      call LogWrite(line,more)
c
      call rdhda(lIn,'ctype1',ctype1,' ')
      call rdhda(lIn,'ctype2',ctype2,' ')
      write(line,'(a,a,a,a,a,f6.2,a,f6.2,a)')
     *  '  Axes: ',ctype1,' x ',ctype2,
     *  '   Beam: ',bmaj*RTS,' x ',bmin*RTS,' arcsecs'
      call LogWrite(line,more)
c
      call mitoaf(blc,3,txtblc,lblc)
      call mitoaf(trc,3,txttrc,ltrc)
      line = '  Bounding region is Blc=('//txtblc(1:lblc)//
     *                          '),Trc=('//txttrc(1:ltrc)//')'
      call LogWrite(line,more)
c
      end
