
/* System dependent initialisation. */

#define FORTRAN_CVT_INT 0
#define FORTRAN_CVT_LOG 0
typedef int fort_integer;
typedef int fort_logical;
typedef long fort_ptrdiff;
typedef float fort_real;
typedef double fort_double;
char *zterm(char *string,int length);
void pad(char *string,int length);
int *iface_iarr(int n);
void iface_f2c_icvt(fort_integer *in,int *out,int n);
void iface_c2f_icvt(int *in,fort_integer *out,int n);
int *iface_larr(int n);
void iface_f2c_lcvt(fort_logical *in,int *out,int n);
void iface_c2f_lcvt(int *in,fort_logical *out,int n);
/******************************************************************************/
/*                                                                            */
/* Conversion for the xyzio routine from FORTRAN to C                         */
/*                                                                            */
/*  History:                                                                  */
/*    bpw  ??????? Original version.                                          */
/*    rjs  22dec92 Converted from xyziowrap.h                                 */
/*    bpw  15jul93 Add xyzflush and xyzmkbuf                                  */
/*    rjs  18sep05 Support FORTRAN/C integer difference.		      */
/*    rjs  26nov05 Support FORTRAN/C logical difference.		      */
/******************************************************************************/

#include <strings.h>
#include "maxdimc.h"
#include "miriad.h"

/************************************************************************/
void xyzopen_(tno,name,status,naxis,axlen,name_len,status_len)
fort_integer *tno;
char *name;
int name_len;
char *status;
int status_len;
fort_integer *naxis;
fort_integer *axlen;
{
#if FORTRAN_CVT_INT
  int ttno,isnew,*taxes,tnaxis;
  tnaxis = *naxis;
  taxes = iface_iarr(tnaxis);
  if(!strcmp(zterm(status,status_len),"new")){
    iface_f2c_icvt(axlen,taxes,tnaxis);
    isnew = 1;
  }else isnew = 0;
  xyzopen_c(&ttno, zterm(name,name_len), zterm(status,status_len), &tnaxis, taxes );
  if(!isnew){
    iface_c2f_icvt(taxes,axlen,tnaxis);
    *(naxis) = tnaxis;
  }
  *(tno) = ttno;
#else
  xyzopen_c(tno, zterm(name,name_len), zterm(status,status_len), naxis, axlen );
#endif
}
/************************************************************************/
void xyzclose_(tno)
fort_integer *tno;
{
  xyzclose_c(*tno);
}
/************************************************************************/
void xyzflush_(tno)
fort_integer *tno;
{
  xyzflush_c(*tno);
}
/************************************************************************/
void xyzmkbuf_()
{
  xyzmkbuf_c();
}
/************************************************************************/
void xyzsetup_(tno,subcube,blc,trc,viraxlen,vircubesize,subcube_len)
fort_integer *tno;
char *subcube;
int subcube_len;
fort_integer *blc;
fort_integer *trc;
fort_integer *viraxlen;
fort_ptrdiff *vircubesize;
{
#if FORTRAN_CVT_INT
  int tblc[MAXNAX],ttrc[MAXNAX],tviraxlen[MAXNAX];
  ptrdiff tvircubesize[MAXNAX];
  int naxis,dimsub;

  xyzdim_c(*tno,&naxis,&dimsub);
  iface_f2c_icvt(blc,tblc,naxis);
  iface_f2c_icvt(trc,ttrc,naxis);
  xyzsetup_c(*tno,zterm(subcube,subcube_len),tblc,ttrc,tviraxlen,tvircubesize);
  iface_c2f_icvt(tviraxlen,viraxlen,naxis);
  iface_c2f_icvt(tvircubesize,vircubesize,naxis);
#else
  xyzsetup_c(*tno,zterm(subcube,subcube_len),blc,trc,
	viraxlen,(long *)vircubesize);
#endif
}
/************************************************************************/
void xyzs2c_(tno,subcubenr,coords)
fort_integer *tno;
fort_ptrdiff *subcubenr;
fort_integer *coords;
{
#if FORTRAN_CVT_INT
  int dimsub,naxis,tcoords[MAXNAX+1];
  xyzdim_c(*tno,&naxis,&dimsub);
  xyzs2c_c( *tno, *subcubenr-1, tcoords );
  iface_c2f_icvt(tcoords,coords,naxis-dimsub);
#else
  xyzs2c_c( *tno, (long)(*subcubenr-1), coords );
#endif
}
/************************************************************************/
void xyzc2s_(tno,coords,subcubenr)
fort_integer *tno;
fort_integer *coords;
fort_ptrdiff *subcubenr;
{
#if FORTRAN_CVT_INT
  int dimsub,naxis,tcoords[MAXNAX+1],tsubcubenr;
  xyzdim_c(*tno,&naxis,&dimsub);
  iface_f2c_icvt(coords,tcoords,naxis-dimsub);
  xyzc2s_c( *tno, tcoords, &tsubcubenr);
  *(subcubenr) = tsubcubenr + 1;
#else
  xyzc2s_c( *tno, coords,(long *)subcubenr ); *(subcubenr) += 1;
#endif
}
/************************************************************************/
void xyzread_(tno,coords,data,mask,ndata)
fort_integer *tno;
fort_integer *coords;
fort_real *data;
fort_logical *mask;
fort_integer *ndata;
{
#if FORTRAN_CVT_INT || FORTRAN_CVT_LOG
  int dimsub,naxis,tcoords[MAXNAX+1],tndata,*tbuffer,npix;
  xyzdim_c(*tno,&naxis,&dimsub);
  npix = xyzpix_c(*tno,0);
  tbuffer = iface_larr(npix);
  iface_f2c_icvt(coords,tcoords,naxis-dimsub);
  xyzread_c(*tno,tcoords,data,tbuffer,&tndata);
  if(tndata != npix)bug_c('f',"Number of pixel mismatch in xyzread");
  iface_c2f_lcvt(tbuffer,mask,npix);
  *(ndata) = tndata;
#else
  xyzread_c(*tno,coords,data,mask,ndata);
#endif
}
/************************************************************************/
void xyzpixrd_(tno,pixelnr,data,mask)
fort_integer *tno;
fort_ptrdiff *pixelnr;
fort_real *data;
fort_logical *mask;
{
  int temp;
  xyzpixrd_c(*tno,(long)*pixelnr,data,&temp);
  *(mask) = temp;
}
/************************************************************************/
void xyzprfrd_(tno,profilenr,data,mask,ndata)
fort_integer *tno;
fort_integer *profilenr;
fort_real *data;
fort_logical *mask;
fort_integer *ndata;
{
  int tndata;
#if FORTRAN_CVT_LOG
  int *tbuffer,npix;
  npix = xyzpix_c(*tno,1);
  tbuffer = iface_larr(npix);
  xyzprfrd_c(*tno,*profilenr,data,tbuffer,&tndata);
  if(tndata != npix)bug_c('f',"Number of pixel mismatch in xyzprfrd");
  iface_c2f_lcvt(tbuffer,mask,npix);
#else
  xyzprfrd_c(*tno,*profilenr,data,mask,&tndata);
#endif
  *(ndata) = tndata;
}
/************************************************************************/
void xyzplnrd_(tno,planenr,data,mask,ndata)
fort_integer *tno;
fort_integer *planenr;
fort_real *data;
fort_logical *mask;
fort_integer *ndata;
{
  int tndata;
#if FORTRAN_CVT_LOG
  int *tbuffer,npix;
  npix = xyzpix_c(*tno,2);
  tbuffer = iface_larr(npix);
  xyzplnrd_c(*tno,*planenr,data,tbuffer,&tndata);
  if(tndata != npix)bug_c('f',"Number of pixel mismatch in xyzplnrd");
  iface_c2f_lcvt(tbuffer,mask,npix);
#else
  xyzplnrd_c(*tno,*planenr,data,mask,&tndata);
#endif
  *(ndata) = tndata;
}
/************************************************************************/
void xyzwrite_(tno,coords,data,mask,ndata)
fort_integer *tno;
fort_integer *coords;
fort_real *data;
fort_logical *mask;
fort_integer *ndata;
{
#if FORTRAN_CVT_INT || FORTRAN_CVT_LOG
  int dimsub,naxis,tcoords[MAXNAX+1],tndata,npix,*tbuffer;
  xyzdim_c(*tno,&naxis,&dimsub);
  npix = xyzpix_c(*tno,0);
  tbuffer = iface_larr(npix);
  iface_f2c_lcvt(mask,tbuffer,npix);
  iface_f2c_icvt(coords,tcoords,naxis-dimsub);
  tndata = *ndata;
  xyzwrite_c(*tno,tcoords,data,tbuffer,&tndata);
#else
  xyzwrite_c(*tno,coords,data,mask,ndata);
#endif
}
/************************************************************************/
void xyzpixwr_(tno,pixelnr,data,mask)
fort_integer *tno;
fort_ptrdiff *pixelnr;
fort_real *data;
fort_logical *mask;
{
  int temp;
  temp = *(mask);
  xyzpixwr_c(*tno,(long)*pixelnr,data,&temp);
}
/************************************************************************/
void xyzprfwr_(tno,profilenr,data,mask,ndata)
fort_integer *tno;
fort_integer *profilenr;
fort_real *data;
fort_logical *mask;
fort_integer *ndata;
{
  int tndata;
#if FORTRAN_CVT_LOG
  int *tbuffer,npix;
  tndata = *ndata;
  npix = xyzpix_c(*tno,1);
  tbuffer = iface_larr(npix);
  iface_f2c_lcvt(mask,tbuffer,npix);
  xyzprfwr_c(*tno,*profilenr,data,tbuffer,&tndata);
#else
  tndata = *ndata;
  xyzprfwr_c(*tno,*profilenr,data,mask,&tndata);
#endif
}
/************************************************************************/
void xyzplnwr_(tno,planenr,data,mask,ndata)
fort_integer *tno;
fort_integer *planenr;
fort_real *data;
fort_logical *mask;
fort_integer *ndata;
{
  int tndata;
#if FORTRAN_CVT_LOG
  int *tbuffer,npix;
  tndata = *ndata;
  npix = xyzpix_c(*tno,2);
  tbuffer = iface_larr(npix);
  iface_f2c_lcvt(mask,tbuffer,npix);
  xyzplnwr_c(*tno,*planenr,data,tbuffer,&tndata);
#else
  tndata = *ndata;
  xyzplnwr_c(*tno,*planenr,data,mask,&tndata);
#endif
}
